#!/usr/bin/perl

# Analyze the networks specified in beagle.log or by the "algorithm" and
# "inputs" command line arguments and print their sizes and parallel steps.
# Modified from the example parallel.pl included in the Algorithm::Networksort
# perl module distribution.
# 
# Usage: network.pl [option]... beagle.log
#
# The algorithm choices are bosenelson, batcher, hibbard, and best. 

# Add directory to the @INC search directory list.
use lib '/home/vkv/work/Algorithm-Networksort/lib';
use lib '/home/vkv/work/Algorithm-Networksort/t';

use Getopt::Long;
use Algorithm::Networksort ':all';

use strict;
use warnings;

# Algorithm name and number of network inputs.
my $algorithm = undef;
my $inputs = 0;

# Command line options.  For example, the algorithm name can be specified
# --algorithm=batcher.
GetOptions(
    'algorithm=s' => \$algorithm,
    'inputs=i' => \$inputs,
);

my @network = ();
if ($algorithm) {
    @network = nw_comparators($inputs, algorithm => $algorithm);
}
else {
    my $genotype = 0;
    while (<>) {
        $genotype = 1 if /Genotype/;
        $inputs = $1 if $genotype && /numInputs="(\d+)"/;
        if ($genotype && /<Comparators.+>(.+)<\/Comparators>/) {
            my @compvec = split ";", $1;
            for (my $i = 0; $i < @compvec/2; $i++) {
                push @network, [$compvec[2*$i], $compvec[2*$i+1]];
            }
            last;
        }
    }
}

my @grouped_network = nw_group(\@network, $inputs, grouping=>'parallel');

print "There are ", scalar @network,
	" comparators in this network, grouped into ",
	scalar @grouped_network, " parallel operations.\n\n";

foreach my $group (@grouped_network)
{
	print nw_format($group), "\n";
}

@grouped_network = nw_group(\@network, $inputs);
print "\nThis will be graphed in ", scalar @grouped_network,
	" columns.\n";

exit(0);
