/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2005 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/ENSO.hpp
 *  \brief  File to include to import all the ENSO specific Beagle classes.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

/*!
 *  \namespace Beagle::ENSO
 *  \brief     Namespace of all the classes of the Open Beagle ENSO framework.
 *  \ingroup ENSOF
 */

/*!
 *  \defgroup ENSOF ENSO Framework
 *  \brief Specialized framework for evolving network symmetry.
 *
 *  TODO Fill in a detailed description of the framework.
 *
 */

#ifndef Beagle_ENSO_hpp
#define Beagle_ENSO_hpp


// Beagle generic components

#include "beagle/Beagle.hpp"

// ENSO framework

#include "beagle/ENSO/EvaluationOp.hpp"
#include "beagle/ENSO/GenerationalOp.hpp"
#include "beagle/ENSO/Parameters.hpp"
#include "beagle/ENSO/ParameterProperties.hpp"
#include "beagle/ENSO/Individual.hpp"
#include "beagle/ENSO/InitFullySymmetricOp.hpp"
#include "beagle/ENSO/InitializationOp.hpp"
#include "beagle/ENSO/MutationBreakSymmetryOp.hpp"
#include "beagle/ENSO/MutationComparatorsOp.hpp"
#include "beagle/ENSO/MutationParametersOp.hpp"
#include "beagle/ENSO/MutationRandomColorSplitOp.hpp"
#include "beagle/ENSO/NeuralNetworkEvalOp.hpp"
#include "beagle/ENSO/NeuralNetworkInitOp.hpp"
#include "beagle/ENSO/NeuralNetworkParams.hpp"
#include "beagle/ENSO/PackageBase.hpp"
#include "beagle/ENSO/SelectDebTournamentOp.hpp"
#include "beagle/ENSO/SelectTournamentOp.hpp"
#include "beagle/ENSO/SortingNetworkEvalOp.hpp"
#include "beagle/ENSO/SortingNetworkInitOp.hpp"
#include "beagle/ENSO/Tree.hpp"

#endif // Beagle_ENSO_hpp

