
/*!
 *  \file   beagle/ENSO/EvaluationOp.hpp
 *  \brief  Definition of the class ENSO::EvaluationOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_EvaluationOp_hpp
#define Beagle_ENSO_EvaluationOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AbstractAllocT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/EvaluationOp.hpp"
#include "beagle/ENSO/Individual.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class EvaluationOp beagle/ENSO/EvaluationOp.hpp "beagle/ENSO/EvaluationOp.hpp"
 *  \brief Abstract ENSO evaluation operator class.
 *  \ingroup ENSOF
 */
class EvaluationOp : public Beagle::EvaluationOp
{

public:

	//! ENSO::EvaluationOp allocator type.
	typedef AbstractAllocT<EvaluationOp,Beagle::EvaluationOp::Alloc>
	Alloc;
	//! ENSO::EvaluationOp handle type.
	typedef PointerT<EvaluationOp,Beagle::EvaluationOp::Handle>
	Handle;
	//! ENSO::EvaluationOp bag type.
	typedef ContainerT<EvaluationOp,Beagle::EvaluationOp::Bag>
	Bag;

	explicit EvaluationOp(Beagle::EvaluationOp::Handle inPhenotypeEvalOp=NULL,
                          const std::string& inName="ENSO-EvaluationOp");
	virtual ~EvaluationOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void init(Beagle::System& ioSystem);

	virtual Fitness::Handle evaluate(Beagle::Individual& inIndividual, Beagle::Context& ioContext);

	/*!
	 *  \brief Evaluate the fitness of the given ENSO individual.
	 *  \param ioIndividual Current ENSO individual to evaluate.
	 *  \param ioContext Evolutionary context.
	 *  \return Handle to the fitness value of the ENSO individual.
	 */
	virtual Fitness::Handle evaluate(ENSO::Individual& ioIndividual, Beagle::Context& ioContext) =0;

protected:

    Beagle::EvaluationOp::Handle    mPhenotypeEvalOp;    //!< Evaluation operator for phenotype.

};

}
}


#endif // Beagle_ENSO_EvaluationOp_hpp
