
/*!
 *  \file   beagle/ENSO/InitFullySymmetricOp.hpp
 *  \brief  Definition of the class ENSO::InitFullySymmetricOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_InitFullySymmetricOp_hpp
#define Beagle_ENSO_InitFullySymmetricOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/ENSO/InitializationOp.hpp"
#include "beagle/ENSO/MutationBreakSymmetryOp.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class InitFullySymmetricOp beagle/ENSO/InitFullySymmetricOp.hpp "beagle/ENSO/InitFullySymmetricOp.hpp"
 *  \brief ENSO genotype trees are initalized so as to produce fully symmetric
 *         complete or null phenotype graphs.  Optionally, symmetry can further
 *         be broken by specifying a non-zero lattice index.
 *  \ingroup ENSOF
 */
class InitFullySymmetricOp : public ENSO::InitializationOp
{

public:

	//! ENSO::InitFullySymmetricOp allocator type.
	typedef AllocatorT<InitFullySymmetricOp,ENSO::InitializationOp::Alloc>
	Alloc;
	//! ENSO::InitFullySymmetricOp handle type.
	typedef PointerT<InitFullySymmetricOp,ENSO::InitializationOp::Handle>
	Handle;
	//! ENSO::InitFullySymmetricOp bag type.
	typedef ContainerT<InitFullySymmetricOp,ENSO::InitializationOp::Bag>
	Bag;

	explicit InitFullySymmetricOp(std::string inReproProbaName="ec.repro.prob",
                                  std::string inName="ENSO-InitFullySymmetricOp");
	virtual ~InitFullySymmetricOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void init(Beagle::System& ioSystem);

	virtual void initTree(ENSO::Tree& outTree, Context& ioContext) const;
	virtual void initParameters(ENSO::Tree& ioTree, Context& ioContext) const;

protected:

    ENSO::MutationBreakSymmetryOp::Handle   mBreakSymmetryOp;   //!< Symmetry break operator used to initialize genotype trees.
	Float::Handle                           mNullGraphProba;    //!< Probability that a null phenotype graph is initialized.
	UInt::Handle                            mLatticeIndex;      //!< Lattice index corresponding to initial symmetry.
	UIntArray::Handle                       mEdgeIndices;       //!< Indices of colors to be made edges.
	UIntArray::Handle                       mNonedgeIndices;    //!< Indices of colors to be made non-edges.

};

}
}

#endif // Beagle_ENSO_InitFullySymmetricOp_hpp
