
/*!
 *  \file   beagle/ENSO/InitializationOp.hpp
 *  \brief  Definition of the class ENSO::InitializationOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_InitializationOp_hpp
#define Beagle_ENSO_InitializationOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AbstractAllocT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/Operator.hpp"
#include "beagle/BreederOp.hpp"
#include "beagle/Individual.hpp"
#include "beagle/ENSO/Tree.hpp"

namespace Beagle
{
namespace ENSO
{

/*!
 *  \class InitializationOp beagle/InitializationOp.hpp "beagle/InitializationOp.hpp"
 *  \brief Abstract initialization operator class.
 *  \ingroup ENSOF
 */
class InitializationOp : public Beagle::InitializationOp
{

public:

	//! InitializationOp allocator type.
	typedef AbstractAllocT<InitializationOp,Beagle::InitializationOp::Alloc>
	Alloc;
	//! InitializationOp handle type.
	typedef PointerT<InitializationOp,Beagle::InitializationOp::Handle>
	Handle;
	//! InitializationOp  bag type.
	typedef ContainerT<InitializationOp,Beagle::InitializationOp::Bag>
	Bag;

	explicit InitializationOp(std::string inReproProbaName="ec.repro.prob",
	                          std::string inName="ENSO-InitializationOp");
	virtual ~InitializationOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void initIndividual(Beagle::Individual& outIndividual, Beagle::Context& ioContext);

	/*!
	 *  \brief Initialize a genotype.
	 *  \param outTree Tree to initialize.
	 *  \param ioContext Evolution context.
	 */
	virtual void initTree(ENSO::Tree& outTree, Context& ioContext) const =0;

protected:

	UInt::Handle      mNumVertices;     //!< Number of vertices in phenotype graph.

};

}
}

#endif // Beagle_ENSO_InitializationOp_hpp
