
/*!
 *  \file   beagle/ENSO/MutationComparatorsOp.hpp
 *  \brief  Definition of the type ENSO::MutationComparatorsOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_MutationComparatorsOp_hpp
#define Beagle_ENSO_MutationComparatorsOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class MutationComparatorsOp beagle/ENSO/MutationComparatorsOp.hpp "beagle/ENSO/MutationComparatorsOp.hpp"
 *  \brief Mutation that modifies the comparators of ENSO genotype tree.
 *  \ingroup ENSOF
 */
class MutationComparatorsOp : public Beagle::MutationOp
{

public:

	//! ENSO::MutationComparatorsOp allocator type.
	typedef AllocatorT<MutationComparatorsOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! ENSO::MutationComparatorsOp handle type.
	typedef PointerT<MutationComparatorsOp,Beagle::MutationOp::Handle>
	Handle;
	//! ENSO::MutationComparatorsOp bag type.
	typedef ContainerT<MutationComparatorsOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationComparatorsOp(std::string inMutationPbName="enso.mutcomps.indpb",
                                   std::string inName="ENSO-MutationComparatorsOp");
	virtual ~MutationComparatorsOp()
	{ }

	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

    bool         addComparators(ENSO::Tree& ioTree, Beagle::Context& ioContext) const;

};

}
}

#endif // Beagle_ENSO_MutationComparatorsOp_hpp
