
/*!
 *  \file   beagle/ENSO/MutationParametersOp.hpp
 *  \brief  Definition of the type ENSO::MutationParametersOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_MutationParametersOp_hpp
#define Beagle_ENSO_MutationParametersOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class MutationParametersOp beagle/ENSO/MutationParametersOp.hpp "beagle/ENSO/MutationParametersOp.hpp"
 *  \brief Mutation that modifies the parameters of ENSO genotype tree nodes.
 *  \ingroup ENSOF
 */
class MutationParametersOp : public Beagle::MutationOp
{

public:

	//! ENSO::MutationParametersOp allocator type.
	typedef AllocatorT<MutationParametersOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! ENSO::MutationParametersOp handle type.
	typedef PointerT<MutationParametersOp,Beagle::MutationOp::Handle>
	Handle;
	//! ENSO::MutationParametersOp bag type.
	typedef ContainerT<MutationParametersOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationParametersOp(std::string inMutationPbName="enso.mutparams.indpb",
                                  std::string inParametersKeyName="enso.mutparams.paramskey",
                                  std::string inApplyToVertexName="enso.mutparams.applyvert",
                                  std::string inMutateValueProbaName="enso.mutparams.valpb",
                                  std::string inMutateGaussSigmaName="enso.mutparams.sigma",
                                  std::string inName="ENSO-MutationParametersOp");
	virtual ~MutationParametersOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);
	virtual void readWithSystem(PACC::XML::ConstIterator inIter, System& ioSystem);
	virtual void writeContent(PACC::XML::Streamer& ioStreamer, bool inIndent=true) const;

protected:

    String::Handle      mParametersKey;          //!< Key of the parameters entry to be mutated.
    std::string         mParametersKeyName;      //!< Name of the parameters key to be mutated.
    Bool::Handle        mApplyToVertex;          //!< Whether to apply mutation to vertex/edge parameters.
    std::string         mApplyToVertexName;      //!< Name of apply mutation to vertex/edge parameters.
	Float::Handle       mMutateValueProba;       //!< Probability of mutating a single value in vectors.
	std::string         mMutateValueProbaName;   //!< Name of the probability of mutating a single value.
	Float::Handle       mMutateGaussSigma;       //!< Sigma of the Gaussian mutation of float parameters.
	std::string         mMutateGaussSigmaName;   //!< Name of the Gaussian mutation sigma.
	Float::Handle       mMinIsEdgeMut;           //!< Minimum generation interval between isedge mutations.
	Float::Handle       mIncIsEdgeMut;           //!< Increment in minimum interval between isedge mutations.

};

}
}

#endif // Beagle_ENSO_MutationParametersOp_hpp
