
/*!
 *  \file   beagle/ENSO/MutationRandomColorSplitOp.hpp
 *  \brief  Definition of the type ENSO::MutationRandomColorSplitOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_MutationRandomColorSplitOp_hpp
#define Beagle_ENSO_MutationRandomColorSplitOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"
#include "beagle/RouletteT.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class MutationRandomColorSplitOp beagle/ENSO/MutationRandomColorSplitOp.hpp "beagle/ENSO/MutationRandomColorSplitOp.hpp"
 *  \brief Mutation that complexifies an ENSO genotype tree by randomly splitting colors.
 *  \ingroup ENSOF
 */
class MutationRandomColorSplitOp : public Beagle::MutationOp
{

public:

	//! ENSO::MutationRandomColorSplitOp allocator type.
	typedef AllocatorT<MutationRandomColorSplitOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! ENSO::MutationRandomColorSplitOp handle type.
	typedef PointerT<MutationRandomColorSplitOp,Beagle::MutationOp::Handle>
	Handle;
	//! ENSO::MutationRandomColorSplitOp bag type.
	typedef ContainerT<MutationRandomColorSplitOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationRandomColorSplitOp(std::string inMutationPbName="enso.mutrndcol.indpb",
                                        std::string inName="ENSO-MutationRandomColorSplitOp");
	virtual ~MutationRandomColorSplitOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

protected:

	UInt::Handle                    mNumVertices;       //!< Number of vertices in phenotype graph.
	Float::Handle                   mMinSymmetryBreak;  //!< Minimum generation interval between symmetry breaks.
	Float::Handle                   mIncSymmetryBreak;  //!< Increment in minimum interval between symmetry breaks.
    Bool::Handle                    mPruneLattice;      //!< Whether to prune the subgroup lattice.
    Float::Handle                   mRestoreProba;      //!< Probability that symmetry is restored.

    void splitNodeColors(ENSO::Tree& inTree, std::vector< std::vector<bool> >& outChildColors, Beagle::Context& ioContext) const;

};

}
}

#endif // Beagle_ENSO_MutationRandomColorSplitOp_hpp
