
/*!
 *  \file   beagle/ENSO/NeuralNetworkEvalOp.hpp
 *  \brief  Definition of the type NeuralNetworkEvalOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_NeuralNetworkEvalOp_hpp
#define Beagle_ENSO_NeuralNetworkEvalOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AbstractAllocT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/EvaluationOp.hpp"
#include "beagle/ENSO/EvaluationOp.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class NeuralNetworkEvalOp beagle/ENSO/NeuralNetworkEvalOp.hpp "beagle/ENSO/NeuralNetworkEvalOp.hpp"
 *  \brief ENSO evaluation operator for neural networks.
 *  \ingroup ENSOF
 */
class NeuralNetworkEvalOp : public ENSO::EvaluationOp
{

public:

	//! NeuralNetworkEvalOp allocator type.
	typedef AllocatorT<NeuralNetworkEvalOp,ENSO::EvaluationOp::Alloc>
	Alloc;
	//! NeuralNetworkEvalOp handle type.
	typedef PointerT<NeuralNetworkEvalOp,ENSO::EvaluationOp::Handle>
	Handle;
	//! NeuralNetworkEvalOp bag type.
	typedef ContainerT<NeuralNetworkEvalOp,ENSO::EvaluationOp::Bag>
	Bag;

	explicit NeuralNetworkEvalOp(Beagle::EvaluationOp::Handle inPhenotypeEvalOp=NULL,
                                 const std::string& inName="ENSO-NeuralNetworkEvalOp");
	virtual ~NeuralNetworkEvalOp()
    { }

	virtual Fitness::Handle evaluate(Beagle::ENSO::Individual& ioIndividual, Beagle::Context& ioContext);

};

}
}


#endif // Beagle_ENSO_NeuralNetworkEvalOp_hpp
