
/*!
 *  \file   beagle/ENSO/NeuralNetworkInitOp.hpp
 *  \brief  Definition of the type NeuralNetworkInitOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_NeuralNetworkInitOp_hpp
#define Beagle_ENSO_NeuralNetworkInitOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/ENSO/InitFullySymmetricOp.hpp"
#include "beagle/NN.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class NeuralNetworkInitOp beagle/ENSO/NeuralNetworkInitOp.hpp "beagle/ENSO/NeuralNetworkInitOp.hpp"
 *  \brief ENSO initialization operator for neural networks.
 *  \ingroup ENSOF
 */
class NeuralNetworkInitOp : public ENSO::InitFullySymmetricOp
{

public:

	//! NeuralNetworkInitOp allocator type.
	typedef AllocatorT<NeuralNetworkInitOp,ENSO::InitFullySymmetricOp::Alloc>
	Alloc;
	//! NeuralNetworkInitOp handle type.
	typedef PointerT<NeuralNetworkInitOp,ENSO::InitFullySymmetricOp::Handle>
	Handle;
	//! NeuralNetworkInitOp bag type.
	typedef ContainerT<NeuralNetworkInitOp,ENSO::InitFullySymmetricOp::Bag>
	Bag;

	explicit NeuralNetworkInitOp(std::string inReproProbaName="ec.repro.prob",
                                 std::string inName="ENSO-NeuralNetworkInitOp");
	virtual ~NeuralNetworkInitOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void init(Beagle::System& ioSystem);

	virtual void initParameters(ENSO::Tree& ioTree, Context& ioContext) const;

protected:

	String::Handle mNetworkFile;      //!< Name of file containing initial neural network.
	Double::Handle mMinEdgeWeight;    //!< Minimum initial weight of edges.
	Double::Handle mMaxEdgeWeight;    //!< Maximum initial weight of edges.
	Double::Handle mMinNodeBias;      //!< Minimum initial bias of nodes.
	Double::Handle mMaxNodeBias;      //!< Maximum initial bias of nodes.
	Double::Handle mMinNodeSlope;     //!< Minimum initial slope of nodes.
	Double::Handle mMaxNodeSlope;     //!< Maximum initial slope of nodes.

    NN::Graph      mInitNetwork;      //!< Initial neural network read from file.

};

}
}


#endif // Beagle_ENSO_NeuralNetworkInitOp_hpp
