
/*!
 *  \file   beagle/ENSO/NeuralNetworkParams.hpp
 *  \brief  Definition of the type ENSO::NeuralNetworkParams.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */


#ifndef Beagle_ENSO_NeuralNetworkParams_hpp
#define Beagle_ENSO_NeuralNetworkParams_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/Allocator.hpp"
#include "beagle/Pointer.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/ENSO/ParameterProperties.hpp"


namespace Beagle
{
namespace ENSO
{


/*!
 *  \class NeuralNetworkParams beagle/ENSO/NeuralNetworkParams.hpp "beagle/ENSO/NeuralNetworkParams.hpp"
 *  \brief The class keeping track of neural network parameter properties in the ENSO genotype tree nodes.
 *  \ingroup ENSOF
 */
class NeuralNetworkParams : public ParameterProperties
{

public:

	//! ENSO::NeuralNetworkParams allocator type.
	typedef AllocatorT<NeuralNetworkParams,ParameterProperties::Alloc>
	Alloc;
	//! ENSO::NeuralNetworkParams handle type.
	typedef PointerT<NeuralNetworkParams,ParameterProperties::Handle>
	Handle;
	//! ENSO::NeuralNetworkParams bag type.
	typedef ContainerT<NeuralNetworkParams,ParameterProperties::Bag>
	Bag;

	NeuralNetworkParams()
    { }
	virtual ~NeuralNetworkParams()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void init(Beagle::System& ioSystem);
	virtual void readWithSystem(PACC::XML::ConstIterator inIter, Beagle::System& ioSystem);
	virtual void write(PACC::XML::Streamer& ioStreamer, bool inIndent=true) const;

protected:

	Double::Handle mMinEdgeWeight;    //!< Minimum initial weight of edges.
	Double::Handle mMaxEdgeWeight;    //!< Maximum initial weight of edges.
	Double::Handle mMinNodeBias;      //!< Minimum initial bias of nodes.
	Double::Handle mMaxNodeBias;      //!< Maximum initial bias of nodes.
	Double::Handle mMinNodeSlope;     //!< Minimum initial slope of nodes.
	Double::Handle mMaxNodeSlope;     //!< Maximum initial slope of nodes.

};

}
}


#endif // Beagle_ENSO_NeuralNetworkParams_hpp
