
/*!
 *  \file   beagle/ENSO/PackageBase.hpp
 *  \brief  Definition of the class ENSO::PackageBase.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_PackageBase_hpp
#define Beagle_ENSO_PackageBase_hpp

#include <map>
#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/System.hpp"
#include "beagle/Evolver.hpp"
#include "beagle/Package.hpp"


namespace Beagle
{

namespace ENSO
{

/*!
 *  \class ENSO::PackageBase beagle/ENSO/PackageBase.hpp "beagle/ENSO/PackageBase.hpp"
 *  \brief Package for adding objects related to ENSO.
 *  \ingroup ENSOF
 */
class PackageBase : public Beagle::Package
{

public:

	//! ENSO::PackageBase allocator type.
	typedef AllocatorT< PackageBase, Beagle::Package::Alloc > Alloc;
	//! ENSO::PackageBase handle type.
	typedef PointerT< PackageBase, Beagle::Package::Handle > Handle;
	//! ENSO::PackageBase bag type.
	typedef ContainerT< PackageBase, Beagle::Package::Bag > Bag;

	PackageBase(void);

	virtual void                  configure(Beagle::System& ioSystem);
	virtual Beagle::Package::Bag  listDependencies(void);

};

}
}

#endif // Beagle_ENSO_PackageBase_hpp
