
/*!
 *  \file   beagle/ENSO/ParameterProperties.hpp
 *  \brief  Definition of the type ENSO::ParameterProperties.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */


#ifndef Beagle_ENSO_ParameterProperties_hpp
#define Beagle_ENSO_ParameterProperties_hpp

#include <string>
#include <map>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/Allocator.hpp"
#include "beagle/Pointer.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/HashString.hpp"
#include "beagle/Genotype.hpp"

#if defined(BEAGLE_HAVE_STD_HASHMAP) | defined(BEAGLE_HAVE_STDEXT_HASHMAP)
#include <hash_map>
#endif // defined(BEAGLE_HAVE_STD_HASHMAP) | defined(BEAGLE_HAVE_STDEXT_HASHMAP)

#ifdef BEAGLE_HAVE_GNUCXX_HASHMAP
#include <ext/hash_map>
#endif // BEAGLE_HAVE_GNUCXX_HASHMAP


namespace Beagle
{
namespace ENSO
{


/*!
 *  \class ParameterProperties beagle/ENSO/ParameterProperties.hpp "beagle/ENSO/ParameterProperties.hpp"
 *  \brief The class keeping track of the parameter properties of ENSO genotype tree nodes.
 *  \ingroup ENSOF
 */
#if defined(BEAGLE_HAVE_STD_HASHMAP)
class ParameterProperties : public Component, public std::hash_map<std::string,Genotype::Handle,HashString>
#elif defined(BEAGLE_HAVE_GNUCXX_HASHMAP)
class ParameterProperties : public Component, public __gnu_cxx::hash_map<std::string,Genotype::Handle,HashString>
#elif defined(BEAGLE_HAVE_STDEXT_HASHMAP)
class ParameterProperties : public Component, public stdext::hash_map<std::string,Genotype::Handle,HashString>
#else // No hash_map found
class ParameterProperties : public Component, public std::map< std::string,Genotype::Handle,std::less<std::string> >
#endif
{

public:

	//! ENSO::ParameterProperties allocator type.
	typedef AllocatorT<ParameterProperties,Component::Alloc>
	Alloc;
	//! ENSO::ParameterProperties handle type.
	typedef PointerT<ParameterProperties,Component::Handle>
	Handle;
	//! ENSO::ParameterProperties bag type.
	typedef ContainerT<ParameterProperties,Component::Bag>
	Bag;

	ParameterProperties();
	virtual ~ParameterProperties()
	{ }

};

}
}


#endif // Beagle_ENSO_ParameterProperties_hpp
