
/*!
 *  \file   beagle/ENSO/SelectDebTournamentOp.hpp
 *  \brief  Definition of the class SelectDebTournamentOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_SelectDebTournamentOp_hpp
#define Beagle_ENSO_SelectDebTournamentOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/SelectTournamentOp.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class SelectDebTournamentOp beagle/ENSO/SelectDebTournamentOp.hpp "beagle/ENSO/SelectDebTournamentOp.hpp"
 *  \brief Tournament selection operator class with niching as described in
 *      - An efficient constraint handling method for genetic algorithms (Kalyanmoy Deb, 2000, page 11-12),
 *      - Multi-objective Optimization Using Evolutionary Algorithms (Kalyanmoy Deb, 2008, page 159-160)
 *  \ingroup ENSOF
 */
class SelectDebTournamentOp : public Beagle::SelectTournamentOp
{

public:

	//! SelectDebTournamentOp allocator type.
	typedef AllocatorT<SelectDebTournamentOp,Beagle::SelectTournamentOp::Alloc> Alloc;
	//! SelectDebTournamentOp handle type.
	typedef PointerT<SelectDebTournamentOp,Beagle::SelectTournamentOp::Handle> Handle;
	//! SelectDebTournamentOp bag type.
	typedef ContainerT<SelectDebTournamentOp,Beagle::SelectTournamentOp::Bag> Bag;

	explicit SelectDebTournamentOp(std::string inReproProbaName="ec.repro.prob",
                                   std::string inName="ENSO-SelectDebTournamentOp");
	virtual ~SelectDebTournamentOp()
	{ }

	virtual void         registerParams(System& ioSystem);
	virtual unsigned int selectIndividual(Beagle::Individual::Bag& ioPool, Context& ioContext);

protected:

	Float::Handle        mThreshold;  //!< Threshold used in comparing genotype distance.

};

}
}

#endif // Beagle_ENSO_SelectDebTournamentOp_hpp
