
/*!
 *  \file   beagle/ENSO/SelectTournamentOp.hpp
 *  \brief  Definition of the class SelectTournamentOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_SelectTournamentOp_hpp
#define Beagle_ENSO_SelectTournamentOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/SelectTournamentOp.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class SelectTournamentOp beagle/ENSO/SelectTournamentOp.hpp "beagle/ENSO/SelectTournamentOp.hpp"
 *  \brief Tournament selection operator class with niching based on distance groups.
 *  \ingroup ENSOF
 */
class SelectTournamentOp : public Beagle::SelectTournamentOp
{

public:

	//! SelectTournamentOp allocator type.
	typedef AllocatorT<ENSO::SelectTournamentOp,Beagle::SelectTournamentOp::Alloc> Alloc;
	//! SelectTournamentOp handle type.
	typedef PointerT<ENSO::SelectTournamentOp,Beagle::SelectTournamentOp::Handle> Handle;
	//! SelectTournamentOp bag type.
	typedef ContainerT<ENSO::SelectTournamentOp,Beagle::SelectTournamentOp::Bag> Bag;

	explicit SelectTournamentOp(std::string inReproProbaName="ec.repro.prob",
                                std::string inName="ENSO-SelectTournamentOp");
	virtual ~SelectTournamentOp()
	{ }

	virtual void         registerParams(System& ioSystem);
	virtual unsigned int selectIndividual(Beagle::Individual::Bag& ioPool, Context& ioContext);

protected:

	Float::Handle    mMinMaturity;          //!< Minimum maturity for comparing fitness of symmetrically different individuals.
	Float::Handle    mIncMaturity;          //!< Increment in min maturity for comparing fitness of symmetrically different individuals.

};

}
}

#endif // Beagle_ENSO_SelectTournamentOp_hpp
