
/*!
 *  \file   beagle/ENSO/SortingNetworkEvalOp.hpp
 *  \brief  Definition of the type SortingNetworkEvalOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_SortingNetworkEvalOp_hpp
#define Beagle_ENSO_SortingNetworkEvalOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AbstractAllocT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/EvaluationOp.hpp"
#include "beagle/ENSO/EvaluationOp.hpp"
#include "beagle/SN/EvaluationOp.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class SortingNetworkEvalOp beagle/ENSO/SortingNetworkEvalOp.hpp "beagle/ENSO/SortingNetworkEvalOp.hpp"
 *  \brief ENSO evaluation operator for sorting networks.
 *  \ingroup ENSOF
 */
class SortingNetworkEvalOp : public ENSO::EvaluationOp
{

public:

	//! SortingNetworkEvalOp allocator type.
	typedef AllocatorT<SortingNetworkEvalOp,ENSO::EvaluationOp::Alloc>
	Alloc;
	//! SortingNetworkEvalOp handle type.
	typedef PointerT<SortingNetworkEvalOp,ENSO::EvaluationOp::Handle>
	Handle;
	//! SortingNetworkEvalOp bag type.
	typedef ContainerT<SortingNetworkEvalOp,ENSO::EvaluationOp::Bag>
	Bag;

	explicit SortingNetworkEvalOp(Beagle::EvaluationOp::Handle inPhenotypeEvalOp=new SN::EvaluationOp,
                                  const std::string& inName="ENSO-SortingNetworkEvalOp");
	virtual ~SortingNetworkEvalOp()
    { }

	virtual Fitness::Handle evaluate(Beagle::ENSO::Individual& ioIndividual, Beagle::Context& ioContext);

};

}
}


#endif // Beagle_ENSO_SortingNetworkEvalOp_hpp
