
/*!
 *  \file   beagle/ENSO/SortingNetworkInitOp.hpp
 *  \brief  Definition of the type SortingNetworkInitOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_ENSO_SortingNetworkInitOp_hpp
#define Beagle_ENSO_SortingNetworkInitOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/ENSO/InitFullySymmetricOp.hpp"
#include "beagle/NN.hpp"


namespace Beagle
{
namespace ENSO
{

/*!
 *  \class SortingNetworkInitOp beagle/ENSO/SortingNetworkInitOp.hpp "beagle/ENSO/SortingNetworkInitOp.hpp"
 *  \brief ENSO initialization operator for sorting networks.
 *  \ingroup ENSOF
 */
class SortingNetworkInitOp : public ENSO::InitFullySymmetricOp
{

public:

	//! SortingNetworkInitOp allocator type.
	typedef AllocatorT<SortingNetworkInitOp,ENSO::InitFullySymmetricOp::Alloc>
	Alloc;
	//! SortingNetworkInitOp handle type.
	typedef PointerT<SortingNetworkInitOp,ENSO::InitFullySymmetricOp::Handle>
	Handle;
	//! SortingNetworkInitOp bag type.
	typedef ContainerT<SortingNetworkInitOp,ENSO::InitFullySymmetricOp::Bag>
	Bag;

	explicit SortingNetworkInitOp(std::string inReproProbaName="ec.repro.prob",
                                  std::string inName="ENSO-SortingNetworkInitOp");
	virtual ~SortingNetworkInitOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void init(Beagle::System& ioSystem);

	virtual void initTree(ENSO::Tree& outTree, Context& ioContext) const;
	virtual void initParameters(ENSO::Tree& ioTree, Context& ioContext) const;

protected:

    ENSO::MutationComparatorsOp::Handle   mComparatorsOp;   //!< Comparators operator used to initialize genotype trees.

};

}
}


#endif // Beagle_ENSO_SortingNetworkInitOp_hpp
