
/*!
 *  \file   beagle/ENSO/src/SortingNetworkEvalOp.cpp
 *  \brief  Implementation of the class ENSO::SortingNetworkEvalOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#include "beagle/ENSO.hpp"
#include "beagle/SN.hpp"

using namespace Beagle;


/*!
 *  \brief Construct a new ENSO robot evaluation operator.
 *  \param inName Name of the operator.
 */
ENSO::SortingNetworkEvalOp::SortingNetworkEvalOp(Beagle::EvaluationOp::Handle inPhenotypeEvalOp, const std::string& inName) :
		ENSO::EvaluationOp(inPhenotypeEvalOp, inName)
{ }


/*!
 *  \brief Evaluate the fitness of the sorting network encoded by the ENSO individual.
 *  \param ioIndividual Individual to evaluate.
 *  \param ioContext Evolutionary context.
 *  \return Handle to the fitness measure.
 */
Fitness::Handle ENSO::SortingNetworkEvalOp::evaluate(ENSO::Individual& ioIndividual, Context& ioContext)
{
	Beagle_StackTraceBeginM();

	ENSO::Tree& lTree = *ioIndividual[0];
    GA::IntegerVector& lComparators = castObjectT<GA::IntegerVector&>(*(lTree[0].mParameters["comparators"]));
    GA::IntegerVector& lStages = castObjectT<GA::IntegerVector&>(*(lTree[0].mParameters["stages"]));

    SN::Network::Handle lNetwork = new SN::Network;
    lNetwork->setNumInputs(lTree.getNumVertices());

    for (unsigned int i = 0; i < lComparators.size(); i++) {
        lNetwork->push_back(lComparators[i]);
    }
    for (unsigned int i = 0; i < lStages.size(); i++) {
        lNetwork->getStages().push_back(lStages[i]);
    }

    return castHandleT<SN::EvaluationOp>(mPhenotypeEvalOp)->evaluate(*lNetwork, ioContext);

	Beagle_StackTraceEndM("Fitness::Handle ENSO::SortingNetworkEvalOp::evaluate(ENSO::Individual& ioIndividual, Context& ioContext)");
}
