/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2005 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/NN.hpp
 *  \brief  File to include to import all the NN specific Beagle classes.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

/*!
 *  \namespace Beagle::NN
 *  \brief     Namespace of all the classes of the Open Beagle NN framework.
 *  \ingroup NNF
 */

/*!
 *  \defgroup NNF NN Framework
 *  \brief Specialized framework for evolving neural networks.
 *
 *  TODO Fill in a detailed description of the framework.
 *
 */

#ifndef Beagle_NN_hpp
#define Beagle_NN_hpp


// Beagle generic components

#include "beagle/Beagle.hpp"

// NN framework

#include "beagle/NN/BreadthFirstSearch.hpp"
#include "beagle/NN/ChangeFitnessIndexOp.hpp"
#include "beagle/NN/CrossoverModuleOp.hpp"
#include "beagle/NN/Edge.hpp"
#include "beagle/NN/EvaluationOp.hpp"
#include "beagle/NN/FitnessIndexed.hpp"
#include "beagle/NN/GenerationalOp.hpp"
#include "beagle/NN/Graph.hpp"
#include "beagle/NN/GraphState.hpp"
#include "beagle/NN/Individual.hpp"
#include "beagle/NN/InitControllerOp.hpp"
#include "beagle/NN/InitFullyConnectedOp.hpp"
#include "beagle/NN/InitializationOp.hpp"
#include "beagle/NN/MapT.hpp"
#include "beagle/NN/Module.hpp"
#include "beagle/NN/MutationAddEdgeOp.hpp"
#include "beagle/NN/MutationAddModuleOp.hpp"
#include "beagle/NN/MutationAddNodeOp.hpp"
#include "beagle/NN/MutationEdgeWeightOp.hpp"
#include "beagle/NN/MutationInsertModuleOp.hpp"
#include "beagle/NN/MutationMergeModuleOp.hpp"
#include "beagle/NN/MutationNodeBiasOp.hpp"
#include "beagle/NN/MutationNodeSlopeOp.hpp"
#include "beagle/NN/Node.hpp"
#include "beagle/NN/NodeSet.hpp"
#include "beagle/NN/PackageBase.hpp"
#include "beagle/NN/ReplacementModuleOp.hpp"
#include "beagle/NN/SearchReachable.hpp"
#include "beagle/NN/StatsCalcFitnessIndexedOp.hpp"
#include "beagle/NN/TermMaxFitnessOp.hpp"


#endif // Beagle_NN_hpp

