
/*!
 *  \file   beagle/NN/BreadthFirstSearch.hpp
 *  \brief  Definition of the type NN::BreadthFirstSearch.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_BreadthFirstSearch_hpp
#define Beagle_NN_BreadthFirstSearch_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/Allocator.hpp"
#include "beagle/Pointer.hpp"
#include "beagle/Container.hpp"
#include "beagle/NN/Node.hpp"
#include "beagle/NN/NodeSet.hpp"
#include "beagle/NN/Graph.hpp"


namespace Beagle
{
namespace NN
{


/*!
 *  \class BreadthFirstSearch beagle/NN/BreadthFirstSearch.hpp "beagle/NN/BreadthFirstSearch.hpp"
 *  \brief Class for performing breadth-first search of graphs.
 *  \ingroup NNF
 *
 *  Different search problems can be solved by deriving from this class and
 *  overriding the protected virtual functions.
 */
class BreadthFirstSearch
{

public:

	explicit BreadthFirstSearch();
	virtual ~BreadthFirstSearch()
	{ }

	void search(Graph& ioGraph, const NodeSet& inTargetSet);

protected:

	bool mContinue;     //!< Flag indicating if search should continue.

	virtual void eventInitialize(Node& inNode);
	virtual void eventDiscover(Node& inNode);
	virtual void eventBegin(Node& inNode);
	virtual void eventFinish(Node& inNode);

};

}
}


/*!
 *  \brief This method gets called when the node is initialized before search.
 *  \param inNode The node being initialized.
 */
inline void Beagle::NN::BreadthFirstSearch::eventInitialize(Node& inNode)
{ }


/*!
 *  \brief This method gets called when the node is first discovered during search.
 *  \param inNode The node being discovered.
 */
inline void Beagle::NN::BreadthFirstSearch::eventDiscover(Node& inNode)
{ }


/*!
 *  \brief This method gets called when processing of a node has begun.
 *  \param inNode The node whose processing has begun.
 */
inline void Beagle::NN::BreadthFirstSearch::eventBegin(Node& inNode)
{ }


/*!
 *  \brief This method gets called when processing of a node is finished.
 *  \param inNode The node whose processing is finished.
 */
inline void Beagle::NN::BreadthFirstSearch::eventFinish(Node& inNode)
{ }


#endif // Beagle_NN_BreadthFirstSearch_hpp
