
/*!
 *  \file   beagle/NN/ChangeFitnessIndexOp.hpp
 *  \brief  Definition of class ChangeFitnessIndexOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_ChangeFitnessIndexOp_hpp
#define Beagle_NN_ChangeFitnessIndexOp_hpp

#include "beagle/BreederOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class ChangeFitnessIndexOp beagle/NN/ChangeFitnessIndexOp.hpp "beagle/NN/ChangeFitnessIndexOp.hpp"
 *  \brief Operator to change the fitness index of individuals (Fitness class should be FitnessIndexed)
 *  \ingroup NNF
 */
class ChangeFitnessIndexOp : public BreederOp
{

public:

	//! ChangeFitnessIndexOp allocator type.
	typedef AllocatorT<ChangeFitnessIndexOp,BreederOp::Alloc>
	Alloc;
	//! ChangeFitnessIndexOp handle type.
	typedef PointerT<ChangeFitnessIndexOp,BreederOp::Handle>
	Handle;
	//! ChangeFitnessIndexOp bag type.
	typedef ContainerT<ChangeFitnessIndexOp,BreederOp::Bag>
	Bag;

	explicit ChangeFitnessIndexOp(std::string inName="NN-ChangeFitnessIndexOp");
	virtual ~ChangeFitnessIndexOp()
	{ }

	virtual Beagle::Individual::Handle breed(Beagle::Individual::Bag& inBreedingPool,
	        BreederNode::Handle inChild,
	        Beagle::Context& ioContext);
	virtual float getBreedingProba(BreederNode::Handle inChild);
	virtual void  registerParams(System& ioSystem);
	virtual void  operate(Beagle::Deme& ioDeme, Beagle::Context& ioContext);
	virtual bool  changeIndex(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

protected:

	Float::Handle mChangeIndexProba;   //!< Probability with which the index is changed.

};

}
}

#endif // Beagle_NN_ChangeFitnessIndexOp_hpp
