
/*!
 *  \file   beagle/NN/CrossoverModuleOp.hpp
 *  \brief  Definition of the class NN::CrossoverModuleOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_CrossoverModuleOp_hpp
#define Beagle_NN_CrossoverModuleOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/CrossoverOp.hpp"

namespace Beagle
{
namespace NN
{

/*!
 *  \class CrossoverModuleOp beagle/NN/CrossoverModuleOp.hpp "beagle/NN/CrossoverModuleOp.hpp"
 *  \brief Crossover of NN graph modules.
 *  \ingroup NNF
 *
 *  Compatible modules in the two graphs are crossed over randomly.
 */
class CrossoverModuleOp : public CrossoverOp
{

public:

	//! NN::CrossoverModuleOp allocator type.
	typedef AllocatorT<CrossoverModuleOp,CrossoverOp::Alloc>
	Alloc;
	//! NN::CrossoverModuleOp handle type.
	typedef PointerT<CrossoverModuleOp,CrossoverOp::Handle>
	Handle;
	//! NN::CrossoverModuleOp bag type.
	typedef ContainerT<CrossoverModuleOp,CrossoverOp::Bag>
	Bag;

	explicit CrossoverModuleOp(std::string inMatingPbName="nn.cxmod.indpb",
	                           std::string inName="NN-CrossoverModuleOp");
	virtual ~CrossoverModuleOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mate(Beagle::Individual& ioIndiv1, Beagle::Context& ioContext1,
	                  Beagle::Individual& ioIndiv2, Beagle::Context& ioContext2);

protected:

	bool crossover(const Graph& inGraph1, Graph& ioGraph2, Beagle::Context& ioContext);

};

}
}

#endif // Beagle_NN_CrossoverModuleOp_hpp
