
/*!
 *  \file   beagle/NN/Edge.hpp
 *  \brief  Definition of the type NN::Edge.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_Edge_hpp
#define Beagle_NN_Edge_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/Allocator.hpp"
#include "beagle/Pointer.hpp"
#include "beagle/Container.hpp"

namespace Beagle
{
namespace NN
{


/*!
 *  \class Edge beagle/NN/Edge.hpp "beagle/NN/Edge.hpp"
 *  \brief NN genotype graph edge class.
 *  \ingroup NNF
 */
class Edge
{

public:

	explicit Edge(unsigned int inSourceId=0, double inWeight=0.0, bool inRecurrent=false);
	virtual ~Edge()
	{ }

	virtual bool isEqual(const Edge& inRightEdge) const;
	virtual bool isLess(const Edge& inRightEdge) const;
	virtual void read(PACC::XML::ConstIterator inIter);
	virtual void write(PACC::XML::Streamer& ioStreamer, bool inIndent=true, unsigned int inKey=UINT_MAX) const;

	unsigned int getSourceIdentifier() const;
	double       getWeight() const;
	bool         isRecurrent() const;

	void         setWeight(double inWeight);

protected:

	unsigned int mSourceId;       //!< Identifier of the source node
	double       mWeight;         //!< Weight of the edge
	bool         mRecurrent;      //!< Whether the edge is recurrent or not

};

}
}


/*!
 * \return The source node id of the edge.
 */
inline unsigned int Beagle::NN::Edge::getSourceIdentifier() const
{
	Beagle_StackTraceBeginM();
	return mSourceId;
	Beagle_StackTraceEndM("unsigned int NN::Edge::getSourceIdentifier() const");
}


/*!
 * \return The weight of the edge.
 */
inline double Beagle::NN::Edge::getWeight() const
{
	Beagle_StackTraceBeginM();
	return mWeight;
	Beagle_StackTraceEndM("double NN::Edge::getWeight() const");
}


/*!
 * \return True if the edge is recurrent, false otherwise.
 */
inline bool Beagle::NN::Edge::isRecurrent() const
{
	Beagle_StackTraceBeginM();
	return mRecurrent;
	Beagle_StackTraceEndM("bool NN::Edge::isRecurrent() const");
}


/*!
 * \param inWeight The weight of the edge.
 */
inline void Beagle::NN::Edge::setWeight(double inWeight)
{
	Beagle_StackTraceBeginM();
	mWeight = inWeight;
	Beagle_StackTraceEndM("void NN::Edge::setWeight(double inWeight)");
}


#endif // Beagle_NN_Edge_hpp
