
/*!
 *  \file   beagle/NN/EvaluationOp.hpp
 *  \brief  Definition of the class NN::EvaluationOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_EvaluationOp_hpp
#define Beagle_NN_EvaluationOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AbstractAllocT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/EvaluationOp.hpp"
#include "beagle/NN/Individual.hpp"
#include "beagle/NN/FitnessIndexed.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class EvaluationOp beagle/NN/EvaluationOp.hpp "beagle/NN/EvaluationOp.hpp"
 *  \brief Abstract NN evaluation operator class.
 *  \ingroup NNF
 */
class EvaluationOp : public Beagle::EvaluationOp
{

public:

	//! NN::EvaluationOp allocator type.
	typedef AbstractAllocT<EvaluationOp,Beagle::EvaluationOp::Alloc>
	Alloc;
	//! NN::EvaluationOp handle type.
	typedef PointerT<EvaluationOp,Beagle::EvaluationOp::Handle>
	Handle;
	//! NN::EvaluationOp bag type.
	typedef ContainerT<EvaluationOp,Beagle::EvaluationOp::Bag>
	Bag;

	explicit EvaluationOp(const std::string& inName="NN-EvaluationOp");
	virtual ~EvaluationOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual Fitness::Handle evaluate(Beagle::Individual& inIndividual, Beagle::Context& ioContext);
	virtual void activate(NN::Graph& ioGraph, const NN::Graph::Activations& inActivations, Context& ioContext);

	/*!
	 *  \brief Evaluate the fitness of the given NN graph.
	 *  \param ioGraph Current NN graph to evaluate.
	 *  \param ioContext Evolutionary context.
	 *  \return Handle to the fitness value of the NN graph.
	 */
	virtual FitnessIndexed::Handle evaluate(NN::Graph& ioGraph, Context& ioContext) =0;

protected:

	UInt::Handle mNumModules;           //!< Number of modules.
	UInt::Handle mNumModInst;           //!< Number of distinct module instances.
	UInt::Handle mNumModRows;           //!< Number of rows into which modules are arranged.
	UInt::Handle mNumModInputs;         //!< Number of input nodes per module.
	UInt::Handle mNumModOutputs;        //!< Number of output nodes per module.
	UInt::Handle mNumModHelpers;        //!< Number of helper nodes per module.

};

}
}


#endif // Beagle_NN_EvaluationOp_hpp
