
/*!
 *  \file   beagle/NN/GenerationalOp.hpp
 *  \brief  Definition of the class GenerationalOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */


#ifndef Beagle_NN_GenerationalOp_hpp
#define Beagle_NN_GenerationalOp_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/Pointer.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/Allocator.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/AbstractAllocT.hpp"
#include "beagle/Container.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/Operator.hpp"
#include "beagle/GenerationalOp.hpp"
#include "beagle/Float.hpp"
#include "beagle/UInt.hpp"

namespace Beagle
{
namespace NN
{

/*!
 *  \class GenerationalOp beagle/NN/GenerationalOp.hpp "beagle/NN/GenerationalOp.hpp"
 *  \brief Generational replacement strategy operator class.
 *  \ingroup NNF
 *
 *  A generational operator process deme with a breeding tree following a generation by generation
 *  replacement strategy.
 */
class GenerationalOp : public Beagle::GenerationalOp
{

public:

	//! NN::GenerationalOp allocator type.
	typedef AllocatorT<NN::GenerationalOp,Beagle::GenerationalOp::Alloc>
	Alloc;
	//! NN::GenerationalOp handle type.
	typedef PointerT<NN::GenerationalOp,Beagle::GenerationalOp::Handle>
	Handle;
	//! NN::GenerationalOp bag type.
	typedef ContainerT<NN::GenerationalOp,Beagle::GenerationalOp::Bag>
	Bag;

	GenerationalOp(std::string inName="NN-GenerationalOp");
	virtual ~GenerationalOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void operate(Beagle::Deme& ioDeme, Beagle::Context& ioContext);

protected:

	UInt::Handle  mDeme0StartGen;      //!< Generation at which to start evolving deme 0


};

}
}

#endif // Beagle_NN_GenerationalOp_hpp

