
/*!
 *  \file   beagle/NN/Individual.hpp
 *  \brief  Definition of the class NN::Individual.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_Individual_hpp
#define Beagle_NN_Individual_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Individual.hpp"
#include "beagle/NN/Graph.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class Individual beagle/NN/Individual.hpp "beagle/NN/Individual.hpp"
 *  \brief An individual inherits from a uniform bag of genotypes and has a fitness member.
 *  \ingroup NNF
 */
class Individual : public Beagle::Individual
{

public:

	//! NN::Individual allocator type.
	typedef AllocatorT<NN::Individual,Beagle::Individual::Alloc> Alloc;
	//! NN::Individual handle type.
	typedef PointerT<NN::Individual,Beagle::Individual::Handle> Handle;
	//! NN::Individual bag type.
	typedef ContainerT<NN::Individual,Beagle::Individual::Bag> Bag;

	explicit Individual(unsigned int inN=0, NN::Graph::Handle inModel=NULL);

	virtual void                    writeContent(PACC::XML::Streamer& ioStreamer, bool inIndent=true) const;
	virtual const std::string&      getType() const;
	const NN::Graph::Handle&        operator[](unsigned int inN) const;
	NN::Graph::Handle&              operator[](unsigned int inN);

};

}
}


/*!
 *  \brief Constant handle to the nth genotype of the NN::Individual.
 *  \param inN Index of the NN::Graph to get
 *  \return Constant pointer (NN::Graph::Handle) to the genotype at the index N.
 */
inline const Beagle::NN::Graph::Handle& Beagle::NN::Individual::operator[](unsigned int inN) const
{
	Beagle_StackTraceBeginM();
	Beagle_UpperBoundCheckAssertM(inN,size()-1);
	return castHandleT<NN::Graph>(std::vector<Pointer>::operator[](inN));
	Beagle_StackTraceEndM("const NN::Graph::Handle& NN::Individual::operator[](unsigned int inN) const");
}


/*!
 *  \brief Handle to the nth genotype of the NN::Individual.
 *  \param inN Index of the NN::Graph to get.
 *  \return Pointer (NN::Graph::Handle) to the genotype at the index N.
 */
inline Beagle::NN::Graph::Handle& Beagle::NN::Individual::operator[](unsigned int inN)
{
	Beagle_StackTraceBeginM();
	Beagle_UpperBoundCheckAssertM(inN,size()-1);
	return castHandleT<NN::Graph>(std::vector<Pointer>::operator[](inN));
	Beagle_StackTraceEndM("NN::Graph::Handle& NN::Individual::operator[](unsigned int inN)");
}


#endif // Beagle_NN_Individual_hpp
