
/*!
 *  \file   beagle/NN/InitControllerOp.hpp
 *  \brief  Definition of the class NN::InitControllerOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_InitControllerOp_hpp
#define Beagle_NN_InitControllerOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/NN/InitializationOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class InitControllerOp beagle/NN/InitControllerOp.hpp "beagle/NN/InitControllerOp.hpp"
 *  \brief NN graphs are initalized for a robot controller.
 *  \ingroup NNF
 */
class InitControllerOp : public NN::InitializationOp
{

public:

	//! NN::InitControllerOp allocator type.
	typedef AllocatorT<InitControllerOp,NN::InitializationOp::Alloc>
	Alloc;
	//! NN::InitControllerOp handle type.
	typedef PointerT<InitControllerOp,NN::InitializationOp::Handle>
	Handle;
	//! NN::InitControllerOp bag type.
	typedef ContainerT<InitControllerOp,NN::InitializationOp::Bag>
	Bag;

	explicit InitControllerOp(std::string inReproProbaName="ec.repro.prob",
                              std::string inName="NN-InitControllerOp");
	virtual ~InitControllerOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual unsigned int initGraph(NN::Graph& outGraph, Context& ioContext) const;

protected:

	UInt::Handle mNumModules;           //!< Number of modules.
	UInt::Handle mNumModInst;           //!< Number of distinct module instances.
	UInt::Handle mNumModRows;           //!< Number of rows into which modules are arranged.
	UInt::Handle mNumModInputs;         //!< Number of input nodes per module.
	UInt::Handle mNumModOutputs;        //!< Number of output nodes per module.
	UInt::Handle mNumModHelpers;        //!< Number of helper nodes per module.
	Bool::Handle mModOutInst;           //!< Whether to make module outputs separate instances.
	Bool::Handle mModRecurrency;        //!< Whether to use recurrency between outputs and hidden nodes.
	Bool::Handle mModSimpleRecurrency;  //!< Whether to use recurrency between hidden nodes.
	Bool::Handle mModSelfEdge;          //!< Whether to use self loop edges on hidden nodes.
	Bool::Handle mModInterEdge;         //!< Whether to use edges to other hidden nodes.
	UInt::Handle mNumModExcludeRecur;   //!< Number of hidden nodes in a module to exclude from recurrency.

};

}
}

#endif // Beagle_NN_InitControllerOp_hpp
