
/*!
 *  \file   beagle/NN/InitFullyConnectedOp.hpp
 *  \brief  Definition of the class NN::InitFullyConnectedOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_InitFullyConnectedOp_hpp
#define Beagle_NN_InitFullyConnectedOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/NN/InitializationOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class InitFullyConnectedOp beagle/NN/InitFullyConnectedOp.hpp "beagle/NN/InitFullyConnectedOp.hpp"
 *  \brief NN graphs are initalized with input and output nodes fully connected.
 *  \ingroup NNF
 */
class InitFullyConnectedOp : public NN::InitializationOp
{

public:

	//! NN::InitFullyConnectedOp allocator type.
	typedef AllocatorT<InitFullyConnectedOp,NN::InitializationOp::Alloc>
	Alloc;
	//! NN::InitFullyConnectedOp handle type.
	typedef PointerT<InitFullyConnectedOp,NN::InitializationOp::Handle>
	Handle;
	//! NN::InitFullyConnectedOp bag type.
	typedef ContainerT<InitFullyConnectedOp,NN::InitializationOp::Bag>
	Bag;

	explicit InitFullyConnectedOp(std::string inReproProbaName="ec.repro.prob",
	                              std::string inName="NN-InitFullyConnectedOp");
	virtual ~InitFullyConnectedOp()
	{ }

	virtual unsigned int initGraph(NN::Graph& outGraph, Context& ioContext) const;

};

}
}

#endif // Beagle_NN_InitFullyConnectedOp_hpp
