
/*!
 *  \file   beagle/NN/InitializationOp.hpp
 *  \brief  Definition of the class NN::InitializationOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_InitializationOp_hpp
#define Beagle_NN_InitializationOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AbstractAllocT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/Operator.hpp"
#include "beagle/BreederOp.hpp"
#include "beagle/UInt.hpp"
#include "beagle/Individual.hpp"
#include "beagle/NN/Graph.hpp"

namespace Beagle
{
namespace NN
{

/*!
 *  \class InitializationOp beagle/InitializationOp.hpp "beagle/InitializationOp.hpp"
 *  \brief Abstract initialization operator class.
 *  \ingroup NNF
 */
class InitializationOp : public Beagle::InitializationOp
{

public:

	//! InitializationOp allocator type.
	typedef AbstractAllocT<InitializationOp,Beagle::InitializationOp::Alloc>
	Alloc;
	//! InitializationOp handle type.
	typedef PointerT<InitializationOp,Beagle::InitializationOp::Handle>
	Handle;
	//! InitializationOp  bag type.
	typedef ContainerT<InitializationOp,Beagle::InitializationOp::Bag>
	Bag;

	explicit InitializationOp(std::string inReproProbaName="ec.repro.prob",
	                          std::string inName="NN-InitializationOp");
	virtual ~InitializationOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void initIndividual(Beagle::Individual& outIndividual, Beagle::Context& ioContext);

	/*!
	 *  \brief Initialize a genotype.
	 *  \param outGraph Graph to initialize.
	 *  \param ioContext Evolution context.
	 *  \return The size of the initialized genotype.
	 */
	virtual unsigned int initGraph(NN::Graph& outGraph, Context& ioContext) const =0;

protected:

	Double::Handle mMinEdgeWeight;    //!< Minimum initial weight of edges.
	Double::Handle mMaxEdgeWeight;    //!< Maximum initial weight of edges.
	Double::Handle mMinNodeBias;      //!< Minimum initial bias of nodes.
	Double::Handle mMaxNodeBias;      //!< Maximum initial bias of nodes.
	Double::Handle mMinNodeSlope;     //!< Minimum initial slope of nodes.
	Double::Handle mMaxNodeSlope;     //!< Maximum initial slope of nodes.

};

}
}

#endif // Beagle_NN_InitializationOp_hpp
