
/*!
 *  \file   beagle/NN/MutationAddEdgeOp.hpp
 *  \brief  Definition of the type NN::MutationAddEdgeOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_MutationAddEdgeOp_hpp
#define Beagle_NN_MutationAddEdgeOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class MutationAddEdgeOp beagle/NN/MutationAddEdgeOp.hpp "beagle/NN/MutationAddEdgeOp.hpp"
 *  \brief Mutation that adds an edge to the NN genotype graph.
 *  \ingroup NNF
 *
 *  This operator mutates an NN genotype graph by adding an edge between two
 *  randomly selected nodes.  Whether recurrent edges are allowed or not can
 *  be controlled by a parameter.
 */
class MutationAddEdgeOp : public Beagle::MutationOp
{

public:

	//! NN::MutationAddEdgeOp allocator type.
	typedef AllocatorT<MutationAddEdgeOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! NN::MutationAddEdgeOp handle type.
	typedef PointerT<MutationAddEdgeOp,Beagle::MutationOp::Handle>
	Handle;
	//! NN::MutationAddEdgeOp bag type.
	typedef ContainerT<MutationAddEdgeOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationAddEdgeOp(std::string inMutationPbName="nn.mutaddedge.indpb",
	                           std::string inName="NN-MutationAddEdgeOp");
	virtual ~MutationAddEdgeOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

protected:

	Bool::Handle   mMutateRecurrent;  //!< Whether recurrent edge mutations are allowed.
	Double::Handle mMinEdgeWeight;    //!< Minimum initial weight of edges.
	Double::Handle mMaxEdgeWeight;    //!< Maximum initial weight of edges.

};

}
}

#endif // Beagle_NN_MutationAddEdgeOp_hpp
