
/*!
 *  \file   beagle/NN/MutationAddModuleOp.hpp
 *  \brief  Definition of the type NN::MutationAddModuleOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_MutationAddModuleOp_hpp
#define Beagle_NN_MutationAddModuleOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class MutationAddModuleOp beagle/NN/MutationAddModuleOp.hpp "beagle/NN/MutationAddModuleOp.hpp"
 *  \brief Mutation that creates a new module from one of the unused helper nodes.
 *  \ingroup NNF
 *
 *  This operator chooses a random helper node, and if not present in the graph
 *  already, creates a new module with that helper node as module output.
 */
class MutationAddModuleOp : public Beagle::MutationOp
{

public:

	//! NN::MutationAddModuleOp allocator type.
	typedef AllocatorT<MutationAddModuleOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! NN::MutationAddModuleOp handle type.
	typedef PointerT<MutationAddModuleOp,Beagle::MutationOp::Handle>
	Handle;
	//! NN::MutationAddModuleOp bag type.
	typedef ContainerT<MutationAddModuleOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationAddModuleOp(std::string inMutationPbName="nn.mutaddmod.indpb",
	                             std::string inName="NN-MutationAddModuleOp");
	virtual ~MutationAddModuleOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

protected:

	Double::Handle mMinEdgeWeight;    //!< Minimum initial weight of edges.
	Double::Handle mMaxEdgeWeight;    //!< Maximum initial weight of edges.
	Double::Handle mMinNodeBias;      //!< Minimum initial bias of nodes.
	Double::Handle mMaxNodeBias;      //!< Maximum initial bias of nodes.
	Double::Handle mMinNodeSlope;     //!< Minimum initial slope of nodes.
	Double::Handle mMaxNodeSlope;     //!< Maximum initial slope of nodes.

};

}
}

#endif // Beagle_NN_MutationAddModuleOp_hpp
