
/*!
 *  \file   beagle/NN/MutationAddNodeOp.hpp
 *  \brief  Definition of the type NN::MutationAddNodeOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_MutationAddNodeOp_hpp
#define Beagle_NN_MutationAddNodeOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class MutationAddNodeOp beagle/NN/MutationAddNodeOp.hpp "beagle/NN/MutationAddNodeOp.hpp"
 *  \brief Mutation that adds a node to the NN genotype graph.
 *  \ingroup NNF
 *
 *  This operator mutates an NN genotype graph by splitting a randomly chosen edge
 *  from the graph and inserting a new node there.
 */
class MutationAddNodeOp : public Beagle::MutationOp
{

public:

	//! NN::MutationAddNodeOp allocator type.
	typedef AllocatorT<MutationAddNodeOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! NN::MutationAddNodeOp handle type.
	typedef PointerT<MutationAddNodeOp,Beagle::MutationOp::Handle>
	Handle;
	//! NN::MutationAddNodeOp bag type.
	typedef ContainerT<MutationAddNodeOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationAddNodeOp(std::string inMutationPbName="nn.mutaddnode.indpb",
	                           std::string inName="NN-MutationAddNodeOp");
	virtual ~MutationAddNodeOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

protected:

	Double::Handle mMinEdgeWeight;    //!< Minimum initial weight of edges.
	Double::Handle mMaxEdgeWeight;    //!< Maximum initial weight of edges.
	Double::Handle mMinNodeBias;      //!< Minimum initial bias of nodes.
	Double::Handle mMaxNodeBias;      //!< Maximum initial bias of nodes.
	Double::Handle mMinNodeSlope;     //!< Minimum initial slope of nodes.
	Double::Handle mMaxNodeSlope;     //!< Maximum initial slope of nodes.

};

}
}

#endif // Beagle_NN_MutationAddNodeOp_hpp
