
/*!
 *  \file   beagle/NN/MutationEdgeWeightOp.hpp
 *  \brief  Definition of the type NN::MutationEdgeWeightOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_MutationEdgeWeightOp_hpp
#define Beagle_NN_MutationEdgeWeightOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class MutationEdgeWeightOp beagle/NN/MutationEdgeWeightOp.hpp "beagle/NN/MutationEdgeWeightOp.hpp"
 *  \brief Mutation that changes the edge weight of an NN genotype graph.
 *  \ingroup NNF
 */
class MutationEdgeWeightOp : public Beagle::MutationOp
{

public:

	//! NN::MutationEdgeWeightOp allocator type.
	typedef AllocatorT<MutationEdgeWeightOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! NN::MutationEdgeWeightOp handle type.
	typedef PointerT<MutationEdgeWeightOp,Beagle::MutationOp::Handle>
	Handle;
	//! NN::MutationEdgeWeightOp bag type.
	typedef ContainerT<MutationEdgeWeightOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationEdgeWeightOp(std::string inMutationPbName="nn.mutedgewt.indpb",
	                              std::string inMutateGaussSigmaName="nn.mutedgewt.sigma",
	                              std::string inName="NN-MutationEdgeWeightOp");
	virtual ~MutationEdgeWeightOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

protected:

	Float::Handle  mSingleMutationProba;    //!< Probability of a single mutation.
	Float::Handle  mMutateGaussSigma;       //!< Sigma of the Gaussian mutation.
	std::string    mMutateGaussSigmaName;   //!< Name of the Gaussian mutation sigma.

};

}
}

#endif // Beagle_NN_MutationEdgeWeightOp_hpp
