
/*!
 *  \file   beagle/NN/MutationInsertModuleOp.hpp
 *  \brief  Definition of the type NN::MutationInsertModuleOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_MutationInsertModuleOp_hpp
#define Beagle_NN_MutationInsertModuleOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class MutationInsertModuleOp beagle/NN/MutationInsertModuleOp.hpp "beagle/NN/MutationInsertModuleOp.hpp"
 *  \brief This mutation inserts a module into the individual, possibily replacing existing ones.
 *  \ingroup NNF
 *
 *  The module is selected from one of the module demes.
 */
class MutationInsertModuleOp : public Beagle::MutationOp
{

public:

	//! NN::MutationInsertModuleOp allocator type.
	typedef AllocatorT<MutationInsertModuleOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! NN::MutationInsertModuleOp handle type.
	typedef PointerT<MutationInsertModuleOp,Beagle::MutationOp::Handle>
	Handle;
	//! NN::MutationInsertModuleOp bag type.
	typedef ContainerT<MutationInsertModuleOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationInsertModuleOp(std::string inMutationPbName="nn.mutinsmod.indpb",
	                                std::string inName="NN-MutationInsertModuleOp");
	virtual ~MutationInsertModuleOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);
	virtual void init(Beagle::System& ioSystem);
	virtual void readWithSystem(PACC::XML::ConstIterator inIter, Beagle::System& ioSystem);
	virtual void writeContent(PACC::XML::Streamer &ioStreamer, bool inIndent=true) const;

protected:

	SelectionOp::Handle mModSelectionOp;   //!< To select modules from another deme

	Double::Handle mMinEdgeWeight;    //!< Minimum initial weight of edges.
	Double::Handle mMaxEdgeWeight;    //!< Maximum initial weight of edges.

	bool insert(const Graph& inGraph1, Graph& ioGraph2, Beagle::Context& ioContext);

};

}
}

#endif // Beagle_NN_MutationInsertModuleOp_hpp
