
/*!
 *  \file   beagle/NN/MutationMergeModuleOp.hpp
 *  \brief  Definition of the type NN::MutationMergeModuleOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_MutationMergeModuleOp_hpp
#define Beagle_NN_MutationMergeModuleOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class MutationMergeModuleOp beagle/NN/MutationMergeModuleOp.hpp "beagle/NN/MutationMergeModuleOp.hpp"
 *  \brief Mutation that merges two randomly chosen modules together into a single module.
 *  \ingroup NNF
 */
class MutationMergeModuleOp : public Beagle::MutationOp
{

public:

	//! NN::MutationMergeModuleOp allocator type.
	typedef AllocatorT<MutationMergeModuleOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! NN::MutationMergeModuleOp handle type.
	typedef PointerT<MutationMergeModuleOp,Beagle::MutationOp::Handle>
	Handle;
	//! NN::MutationMergeModuleOp bag type.
	typedef ContainerT<MutationMergeModuleOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationMergeModuleOp(std::string inMutationPbName="nn.mutmrgmod.indpb",
	                               std::string inName="NN-MutationMergeModuleOp");
	virtual ~MutationMergeModuleOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

};

}
}

#endif // Beagle_NN_MutationMergeModuleOp_hpp
