
/*!
 *  \file   beagle/NN/MutationNodeBiasOp.hpp
 *  \brief  Definition of the type NN::MutationNodeBiasOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_MutationNodeBiasOp_hpp
#define Beagle_NN_MutationNodeBiasOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class MutationNodeBiasOp beagle/NN/MutationNodeBiasOp.hpp "beagle/NN/MutationNodeBiasOp.hpp"
 *  \brief Mutation that changes the bias of an NN genotype graph node.
 *  \ingroup NNF
 */
class MutationNodeBiasOp : public Beagle::MutationOp
{

public:

	//! NN::MutationNodeBiasOp allocator type.
	typedef AllocatorT<MutationNodeBiasOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! NN::MutationNodeBiasOp handle type.
	typedef PointerT<MutationNodeBiasOp,Beagle::MutationOp::Handle>
	Handle;
	//! NN::MutationNodeBiasOp bag type.
	typedef ContainerT<MutationNodeBiasOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationNodeBiasOp(std::string inMutationPbName="nn.mutnodebias.indpb",
	                            std::string inMutateGaussSigmaName="nn.mutnodebias.sigma",
	                            std::string inName="NN-MutationNodeBiasOp");
	virtual ~MutationNodeBiasOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

protected:

	Float::Handle  mSingleMutationProba;    //!< Probability of a single mutation.
	Float::Handle  mMutateGaussSigma;       //!< Sigma of the Gaussian mutation.
	std::string    mMutateGaussSigmaName;   //!< Name of the Gaussian mutation sigma.

};

}
}

#endif // Beagle_NN_MutationNodeBiasOp_hpp
