
/*!
 *  \file   beagle/NN/MutationNodeSlopeOp.hpp
 *  \brief  Definition of the type NN::MutationNodeSlopeOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_MutationNodeSlopeOp_hpp
#define Beagle_NN_MutationNodeSlopeOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class MutationNodeSlopeOp beagle/NN/MutationNodeSlopeOp.hpp "beagle/NN/MutationNodeSlopeOp.hpp"
 *  \brief Mutation that changes the slope of an NN genotype graph node.
 *  \ingroup NNF
 */
class MutationNodeSlopeOp : public Beagle::MutationOp
{

public:

	//! NN::MutationNodeSlopeOp allocator type.
	typedef AllocatorT<MutationNodeSlopeOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! NN::MutationNodeSlopeOp handle type.
	typedef PointerT<MutationNodeSlopeOp,Beagle::MutationOp::Handle>
	Handle;
	//! NN::MutationNodeSlopeOp bag type.
	typedef ContainerT<MutationNodeSlopeOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationNodeSlopeOp(std::string inMutationPbName="nn.mutnodeslope.indpb",
	                             std::string inMutateGaussSigmaName="nn.mutnodeslope.sigma",
	                             std::string inName="NN-MutationNodeSlopeOp");
	virtual ~MutationNodeSlopeOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

protected:

	Float::Handle  mSingleMutationProba;    //!< Probability of a single mutation.
	Float::Handle  mMutateGaussSigma;       //!< Sigma of the Gaussian mutation.
	std::string    mMutateGaussSigmaName;   //!< Name of the Gaussian mutation sigma.

};

}
}

#endif // Beagle_NN_MutationNodeSlopeOp_hpp
