
/*!
 *  \file   beagle/NN/PackageBase.hpp
 *  \brief  Definition of the class NN::PackageBase.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_PackageBase_hpp
#define Beagle_NN_PackageBase_hpp

#include <map>
#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/System.hpp"
#include "beagle/Evolver.hpp"
#include "beagle/Package.hpp"
#include "beagle/NN/GraphState.hpp"


namespace Beagle
{

namespace NN
{

/*!
 *  \class NN::PackageBase beagle/NN/PackageBase.hpp "beagle/NN/PackageBase.hpp"
 *  \brief Package for adding objects related to NN.
 *  \ingroup NNF
 */
class PackageBase : public Beagle::Package
{

public:

	//! NN::PackageBase allocator type.
	typedef AllocatorT< PackageBase, Beagle::Package::Alloc > Alloc;
	//! NN::PackageBase handle type.
	typedef PointerT< PackageBase, Beagle::Package::Handle > Handle;
	//! NN::PackageBase bag type.
	typedef ContainerT< PackageBase, Beagle::Package::Bag > Bag;

	PackageBase(void);

	virtual void                  configure(Beagle::System& ioSystem);
	virtual Beagle::Package::Bag  listDependencies(void);

protected:

	NN::GraphState::Handle mGraphState;   //!< Global evolution state of the NN graphs.

};

}
}

#endif // Beagle_NN_PackageBase_hpp
