
/*!
 *  \file   beagle/NN/ReplacementModuleOp.hpp
 *  \brief  Definition of the class ReplacementModuleOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */


#ifndef Beagle_NN_ReplacementModuleOp_hpp
#define Beagle_NN_ReplacementModuleOp_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/Pointer.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/Allocator.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/AbstractAllocT.hpp"
#include "beagle/Container.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/Operator.hpp"
#include "beagle/ReplacementStrategyOp.hpp"
#include "beagle/Float.hpp"
#include "beagle/UInt.hpp"

namespace Beagle
{
namespace NN
{

/*!
 *  \class ReplacementModuleOp beagle/NN/ReplacementModuleOp.hpp "beagle/NN/ReplacementModuleOp.hpp"
 *  \brief Replacement strategy operator class for NN graph modules.
 *  \ingroup NNF
 */
class ReplacementModuleOp : public ReplacementStrategyOp
{

public:

	//! ReplacementModuleOp allocator type.
	typedef AllocatorT<ReplacementModuleOp,ReplacementStrategyOp::Alloc>
	Alloc;
	//! ReplacementModuleOp handle type.
	typedef PointerT<ReplacementModuleOp,ReplacementStrategyOp::Handle>
	Handle;
	//! ReplacementModuleOp bag type.
	typedef ContainerT<ReplacementModuleOp,ReplacementStrategyOp::Bag>
	Bag;

	ReplacementModuleOp(std::string inName="NN-ReplacementModuleOp");
	virtual ~ReplacementModuleOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void operate(Beagle::Deme& ioDeme, Beagle::Context& ioContext);

protected:

	UInt::Handle  mElitismKeepSize;      //!< Number of individuals to keep with elitism

	bool replace(const Beagle::Individual& inIndiv1, Beagle::Individual& ioIndiv2,
	             unsigned int inHelperId, Beagle::Context& ioContext);

};

}
}

#endif // Beagle_NN_ReplacementModuleOp_hpp

