
/*!
 *  \file   beagle/NN/SearchReachable.hpp
 *  \brief  Definition of the type NN::SearchReachable.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_SearchReachable_hpp
#define Beagle_NN_SearchReachable_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/Allocator.hpp"
#include "beagle/Pointer.hpp"
#include "beagle/Container.hpp"
#include "beagle/NN/Node.hpp"
#include "beagle/NN/Graph.hpp"
#include "beagle/NN/BreadthFirstSearch.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class SearchReachable beagle/NN/SearchReachable.hpp "beagle/NN/SearchReachable.hpp"
 *  \brief Class for determining reachable nodes using breadth-first search.
 *  \ingroup NNF
 */
class SearchReachable : public BreadthFirstSearch
{

public:

	explicit SearchReachable()
	{ }
	virtual ~SearchReachable()
	{ }

	NodeSet::Handle reachable(Graph& ioGraph, const NodeSet& inTargetSet);
	NodeSet::Handle reachable(Graph& ioGraph, unsigned int inTargetId);
	bool            reachable(Graph& ioGraph, unsigned int inTargetId, unsigned int inSourceId);

	virtual void eventDiscover(Node& inNode);

protected:

	NodeSet::Handle mReachableSet;   //!< The set of reachable nodes.
	unsigned int    mSourceId;       //!< The source node id to search for.

};

}
}


/*!
 *  \brief This method gets called when the node is first discovered during search.
 *  \param inNode The node being discovered.
 */
inline void Beagle::NN::SearchReachable::eventDiscover(Node& inNode)
{
	Beagle_StackTraceBeginM();
	if (mSourceId < UINT_MAX) {
		mContinue = mSourceId != inNode.getIdentifier();
	} else {
		mReachableSet->insert(inNode.getIdentifier());
	}
	Beagle_StackTraceEndM("void NN::SearchReachable::eventDiscover(Node& inNode)");
}


#endif // Beagle_NN_SearchReachable_hpp
