
/*!
 *  \file   beagle/NN/TermMaxFitnessOp.hpp
 *  \brief  Definition of the class TermMaxFitnessOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_NN_TermMaxFitnessOp_hpp
#define Beagle_NN_TermMaxFitnessOp_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/WrapperT.hpp"
#include "beagle/Operator.hpp"
#include "beagle/TerminationOp.hpp"
#include "beagle/Float.hpp"
#include "beagle/Context.hpp"
#include "beagle/System.hpp"


namespace Beagle
{
namespace NN
{

/*!
 *  \class TermMaxFitnessOp beagle/NN/TermMaxFitnessOp.hpp "beagle/NN/TermMaxFitnessOp.hpp"
 *  \brief Maximum fitness value termination criterion operator class.
 *  \ingroup NNF
 */
class TermMaxFitnessOp : public TerminationOp
{

public:

	//! TermMaxFitnessOp allocator type
	typedef AllocatorT<TermMaxFitnessOp,TerminationOp::Alloc>
	Alloc;
	//! TermMaxFitnessOp handle type.
	typedef PointerT<TermMaxFitnessOp,TerminationOp::Handle>
	Handle;
	//! TermMaxFitnessOp bag type.
	typedef ContainerT<TermMaxFitnessOp,TerminationOp::Bag>
	Bag;

	explicit TermMaxFitnessOp(float inMaxFitness=1.0, std::string inName="NN-TermMaxFitnessOp");
	virtual ~TermMaxFitnessOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void readWithMap(PACC::XML::ConstIterator inIter, OperatorMap& inOpMap);
	virtual bool terminate(const Beagle::Deme& inDeme, Beagle::Context& ioContext);
	virtual void write(PACC::XML::Streamer& ioStreamer, bool inIndent=true) const;

protected:

	Float::Handle mMaxFitness;        //!< Maximum fitness of an individual for an evolution.
	float         mMaxFitnessDefault; //!< Default maximum fitness value.

};

}
}

#endif // Beagle_NN_TerminationCheckOp_hpp
