
/*!
 *  \file   beagle/NN/src/GenerationalOp.cpp
 *  \brief  Source code of class GenerationalOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#include "beagle/NN.hpp"

#include <algorithm>

using namespace Beagle;


/*!
 *  \brief Build generational operator.
 *  \param inName Name of the generational operator.
 */
NN::GenerationalOp::GenerationalOp(std::string inName) :
		Beagle::GenerationalOp(inName)
{ }


/*!
 *  \brief Register the parameters of this operator.
 *  \param ioSystem Reference to the evolutionary system.
 */
void NN::GenerationalOp::registerParams(Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();
	Beagle::GenerationalOp::registerParams(ioSystem);
	Register::Description lDescription(
	    "Deme 0 evolution starting generation",
	    "UInt",
	    "0",
	    "The generation number at which to start evolving deme number 0."
	);
	mDeme0StartGen = castHandleT<UInt>(
	                     ioSystem.getRegister().insertEntry("nn.deme0.startgen", new UInt(0), lDescription));
	Beagle_StackTraceEndM("void NN::GenerationalOp::registerParams(System&)");
}


/*!
 *  \brief Apply the operation on a deme in the given context.
 *  \param ioDeme Reference to the deme on which the operation takes place.
 *  \param ioContext Evolutionary context of the operation.
 */
void NN::GenerationalOp::operate(Beagle::Deme& ioDeme, Beagle::Context& ioContext)
{
	Beagle_StackTraceBeginM();

	if (ioContext.getDemeIndex() == 0 && ioContext.getGeneration() < mDeme0StartGen->getWrappedValue()) {
		History::Handle lHistory = castHandleT<History>(ioContext.getSystem().haveComponent("History"));
		for(unsigned int i=0; i<ioDeme.size(); ++i) {
			if (lHistory != NULL) {
                std::vector<HistoryID> lParent;
                HistoryID::Handle lHID = castHandleT<HistoryID>(ioDeme[i]->getMember("HistoryID"));
                if(lHID != NULL) lParent.push_back(*lHID);
				lHistory->allocateNewID(*ioDeme[i]);
				lHistory->trace(ioContext, lParent, ioDeme[i], getName(), "nochange");
			}
		}
	} else {
		Beagle::GenerationalOp::operate(ioDeme, ioContext);
	}

	Beagle_StackTraceEndM("void NN::GenerationalOp::operate(Deme& ioDeme, Context& ioContext)");
}


