
/*!
 *  \file   beagle/NN/src/Individual.cpp
 *  \brief  Source code of class NN::Individual.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#include "beagle/NN.hpp"

using namespace Beagle;


/*!
 *  \brief Contruct a NN individual.
 *  \param inN Initial number of graphs in vivarium.
 *  \param inModel Initial value of graph handles.
 */
NN::Individual::Individual(unsigned int inN, NN::Graph::Handle inModel) :
		Beagle::Individual(inN,inModel)
{ }


/*!
 *  \brief Get exact type of the individual.
 *  \return Exact type of the individual.
 */
const std::string& NN::Individual::getType() const
{
	Beagle_StackTraceBeginM();
	const static std::string lType("NN-Individual");
	return lType;
	Beagle_StackTraceEndM("const std::string& NN::Individual::getType() const");
}


/*!
 *  \brief Write content of an individual into a XML streamer.
 *  \param ioStreamer XML streamer to write the individual into.
 *  \param inIndent Whether XML output should be indented.
 *
 *  Override the base class implementation to cause indentation
 *  of the various items being written.
 */
void NN::Individual::writeContent(PACC::XML::Streamer& ioStreamer, bool inIndent) const
{
	Beagle_StackTraceBeginM();
	// Write members of the individual
	writeMembers(ioStreamer, true);
	// Write fitness
	writeFitness(ioStreamer, true);
	// Write genotypes
	writeGenotypes(ioStreamer, true);
	Beagle_StackTraceEndM("void NN::Individual::writeContent(PACC::XML::Streamer&,bool) const");
}


