
/*!
 *  \file   beagle/SN/EvaluationOp.hpp
 *  \brief  Definition of the class SN::EvaluationOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_SN_EvaluationOp_hpp
#define Beagle_SN_EvaluationOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AbstractAllocT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/EvaluationOp.hpp"
#include "beagle/SN/Individual.hpp"
#include "beagle/SN/MutationAddComparatorOp.hpp"


namespace Beagle
{
namespace SN
{

/*!
 *  \class EvaluationOp beagle/SN/EvaluationOp.hpp "beagle/SN/EvaluationOp.hpp"
 *  \brief Sorting network evaluation operator class.
 *  \ingroup SNF
 */
class EvaluationOp : public Beagle::EvaluationOp
{

public:

	//! SN::EvaluationOp allocator type.
	typedef AllocatorT<EvaluationOp,Beagle::EvaluationOp::Alloc>
	Alloc;
	//! SN::EvaluationOp handle type.
	typedef PointerT<EvaluationOp,Beagle::EvaluationOp::Handle>
	Handle;
	//! SN::EvaluationOp bag type.
	typedef ContainerT<EvaluationOp,Beagle::EvaluationOp::Bag>
	Bag;

	explicit EvaluationOp(const std::string& inName="SN-EvaluationOp");
	virtual ~EvaluationOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
    virtual void init(Beagle::System& ioSystem);
	virtual Fitness::Handle evaluate(Beagle::Individual& inIndividual, Beagle::Context& ioContext);
	virtual Fitness::Handle evaluate(SN::Network& ioNetwork, Beagle::Context& ioContext);

protected:

	UInt::Handle mNumInputs;                            //!< Number of inputs of the sorting network.
	Bool::Handle mTrace;                                //!< Whether to trace the operation of the network.
	Bool::Handle mComparatorFitness;                    //!< Whether to use number of comparators as fitness.
	Bool::Handle mUseMutationOp;                        //!< Whether to use mutation operator to add comparators.
    MutationAddComparatorOp::Handle  mMutationOp;       //!< Mutation operator to use for adding comparators.
	Float::Handle mApplyRouletteProb;                   //!< Probability to choose comparator for mutation using a roulette wheel.

};

}
}


#endif // Beagle_SN_EvaluationOp_hpp
