
/*!
 *  \file   beagle/SN/Individual.hpp
 *  \brief  Definition of the class SN::Individual.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_SN_Individual_hpp
#define Beagle_SN_Individual_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Individual.hpp"
#include "beagle/SN/Network.hpp"


namespace Beagle
{
namespace SN
{

/*!
 *  \class Individual beagle/SN/Individual.hpp "beagle/SN/Individual.hpp"
 *  \brief An individual inherits from a uniform bag of genotypes and has a fitness member.
 *  \ingroup SNF
 */
class Individual : public Beagle::Individual
{

public:

	//! SN::Individual allocator type.
	typedef AllocatorT<SN::Individual,Beagle::Individual::Alloc> Alloc;
	//! SN::Individual handle type.
	typedef PointerT<SN::Individual,Beagle::Individual::Handle> Handle;
	//! SN::Individual bag type.
	typedef ContainerT<SN::Individual,Beagle::Individual::Bag> Bag;

	explicit Individual(unsigned int inN=0, SN::Network::Handle inModel=NULL);

	virtual void                    writeContent(PACC::XML::Streamer& ioStreamer, bool inIndent=true) const;
	virtual const std::string&      getType() const;
	const SN::Network::Handle&        operator[](unsigned int inN) const;
	SN::Network::Handle&              operator[](unsigned int inN);

};

}
}


/*!
 *  \brief Constant handle to the nth genotype of the SN::Individual.
 *  \param inN Index of the SN::Network to get
 *  \return Constant pointer (SN::Network::Handle) to the genotype at the index N.
 */
inline const Beagle::SN::Network::Handle& Beagle::SN::Individual::operator[](unsigned int inN) const
{
	Beagle_StackTraceBeginM();
	Beagle_UpperBoundCheckAssertM(inN,size()-1);
	return castHandleT<SN::Network>(std::vector<Pointer>::operator[](inN));
	Beagle_StackTraceEndM("const SN::Network::Handle& SN::Individual::operator[](unsigned int inN) const");
}


/*!
 *  \brief Handle to the nth genotype of the SN::Individual.
 *  \param inN Index of the SN::Network to get.
 *  \return Pointer (SN::Network::Handle) to the genotype at the index N.
 */
inline Beagle::SN::Network::Handle& Beagle::SN::Individual::operator[](unsigned int inN)
{
	Beagle_StackTraceBeginM();
	Beagle_UpperBoundCheckAssertM(inN,size()-1);
	return castHandleT<SN::Network>(std::vector<Pointer>::operator[](inN));
	Beagle_StackTraceEndM("SN::Network::Handle& SN::Individual::operator[](unsigned int inN)");
}


#endif // Beagle_SN_Individual_hpp
