
/*!
 *  \file   beagle/SN/InitializationOp.hpp
 *  \brief  Definition of the class SN::InitializationOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_SN_InitializationOp_hpp
#define Beagle_SN_InitializationOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AbstractAllocT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/Operator.hpp"
#include "beagle/BreederOp.hpp"
#include "beagle/UInt.hpp"
#include "beagle/Individual.hpp"
#include "beagle/SN/Network.hpp"


namespace Beagle
{
namespace SN
{

/*!
 *  \class InitializationOp beagle/InitializationOp.hpp "beagle/InitializationOp.hpp"
 *  \brief Abstract initialization operator class.
 *  \ingroup SNF
 */
class InitializationOp : public Beagle::InitializationOp
{

public:

	//! InitializationOp allocator type.
	typedef AllocatorT<SN::InitializationOp,Beagle::InitializationOp::Alloc>
	Alloc;
	//! InitializationOp handle type.
	typedef PointerT<SN::InitializationOp,Beagle::InitializationOp::Handle>
	Handle;
	//! InitializationOp  bag type.
	typedef ContainerT<SN::InitializationOp,Beagle::InitializationOp::Bag>
	Bag;

	explicit InitializationOp(std::string inReproProbaName="ec.repro.prob",
	                          std::string inName="SN-InitializationOp");
	virtual ~InitializationOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual void initIndividual(Beagle::Individual& outIndividual, Beagle::Context& ioContext);

protected:

	UInt::Handle mNumInputs;                //!< Number of inputs of the sorting network.
	Bool::Handle mInitFuncs;                //!< Whether to initialize the monotone Boolean functions.
	Bool::Handle mHeuristicSearch;          //!< Whether to perform heuristic search for a valid sorting network.
	UInt::Handle mSearchDepthBound;         //!< The depth bound for heuristic search.
	UIntArray::Handle mNondomMethods;       //!< Heuristic methods used to compute non-dominating set of candidate comparators.
	UIntArray::Handle mFilterMethods;       //!< Heuristic methods used to filter the non-dominating set of candidate comparators.
	Bool::Handle mRandomNumAllLines;        //!< Whether to randomly choose the number of lines used for heuristics in eAutoAllLinesUtility.

    void initNetwork(SN::Network& outNetwork, Beagle::Context& ioContext) const;
    void heuristicSearch(SN::Network& ioNetwork, Beagle::Context& ioContext) const;
    bool DFSVisit(SN::Network& ioNetwork, unsigned int inJoin, unsigned int inMeet, unsigned int inDepthBound) const;

};

}
}


#endif // Beagle_SN_InitializationOp_hpp
