
/*!
 *  \file   beagle/SN/MutationAddComparatorOp.hpp
 *  \brief  Definition of the type SN::MutationAddComparatorOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_SN_MutationAddComparatorOp_hpp
#define Beagle_SN_MutationAddComparatorOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/MutationOp.hpp"
#include "beagle/SN/Network.hpp"


namespace Beagle
{
namespace SN
{

/*!
 *  \class MutationAddComparatorOp beagle/SN/MutationAddComparatorOp.hpp "beagle/SN/MutationAddComparatorOp.hpp"
 *  \brief Mutation that adds a comparator to the sorting network.
 *  \ingroup SNF
 */
class MutationAddComparatorOp : public Beagle::MutationOp
{

public:

	//! SN::MutationAddComparatorOp allocator type.
	typedef AllocatorT<MutationAddComparatorOp,Beagle::MutationOp::Alloc>
	Alloc;
	//! SN::MutationAddComparatorOp handle type.
	typedef PointerT<MutationAddComparatorOp,Beagle::MutationOp::Handle>
	Handle;
	//! SN::MutationAddComparatorOp bag type.
	typedef ContainerT<MutationAddComparatorOp,Beagle::MutationOp::Bag>
	Bag;

	explicit MutationAddComparatorOp(std::string inMutationPbName="sn.mutaddcomp.indpb",
                                     std::string inName="SN-MutationAddComparatorOp");
	virtual ~MutationAddComparatorOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
    virtual void init(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);
	virtual bool mutate(SN::Network& ioNetwork, Beagle::Context& ioContext);

    bool removeSymmetricComparator(const SN::Network& inNetwork, __gnu_cxx::slist<unsigned int>& ioAsymmetricComparators, unsigned int inJoin, unsigned int inMeet) const;

protected:

	Float::Handle mMakeSymmetricProb;           //!< Probability to add comparators to make network symmetric.
	Bool::Handle mApplyLookAheadFilter;         //!< Whether to filter comparators using look-ahead heuristics.
	Float::Handle mApplyMaxMatchingFilterProb;  //!< Probability to filter comparators using maximum cardinality matching.
	Float::Handle mApplyRouletteProb;           //!< Probability to choose comparator for mutation using a roulette wheel.
	Bool::Handle mUseComparatorTriplets;        //!< Whether using triplets of comparators instead of singles is allowed.
	UIntArray::Handle mFilterGroups;            //!< Groups of filter heuristic methods to select comparators.
	FloatArray::Handle mFilterGroupProbs;       //!< Probabilities of the filter groups.

    std::vector< std::vector<unsigned int> >    mFilterGroupVectors;  //!< Vector of filter groups indexed by group index.
    RouletteT<unsigned int>                     mFilterGroupRoulette; //!< Normalized probabilities of filter groups.

};

}
}

#endif // Beagle_SN_MutationAddComparatorOp_hpp
