
/*!
 *  \file   beagle/SN/MutationTruncateConstructOp.hpp
 *  \brief  Definition of the type SN::MutationTruncateConstructOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_SN_MutationTruncateConstructOp_hpp
#define Beagle_SN_MutationTruncateConstructOp_hpp

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "MutationAddComparatorOp.hpp"


namespace Beagle
{
namespace SN
{

/*!
 *  \class MutationTruncateConstructOp beagle/SN/MutationTruncateConstructOp.hpp "beagle/SN/MutationTruncateConstructOp.hpp"
 *  \brief Mutation that truncates a given network and then adds comparators to construct a valid network.
 *  \ingroup SNF
 */
class MutationTruncateConstructOp : public MutationAddComparatorOp
{

public:

	//! SN::MutationTruncateConstructOp allocator type.
	typedef AllocatorT<MutationTruncateConstructOp,MutationAddComparatorOp::Alloc>
	Alloc;
	//! SN::MutationTruncateConstructOp handle type.
	typedef PointerT<MutationTruncateConstructOp,MutationAddComparatorOp::Handle>
	Handle;
	//! SN::MutationTruncateConstructOp bag type.
	typedef ContainerT<MutationTruncateConstructOp,MutationAddComparatorOp::Bag>
	Bag;

	explicit MutationTruncateConstructOp(std::string inMutationPbName="sn.muttruncons.indpb",
                                         std::string inName="SN-MutationTruncateConstructOp");
	virtual ~MutationTruncateConstructOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool mutate(Beagle::Individual& ioIndividual, Beagle::Context& ioContext);

protected:

	Bool::Handle mConstructValid;            //!< Whether to construct a valid network after truncating it.

};

}
}

#endif // Beagle_SN_MutationTruncateConstructOp_hpp
