
/*!
 *  \file   beagle/SN/PackageBase.hpp
 *  \brief  Definition of the class SN::PackageBase.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_SN_PackageBase_hpp
#define Beagle_SN_PackageBase_hpp

#include <map>
#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/System.hpp"
#include "beagle/Evolver.hpp"
#include "beagle/Package.hpp"


namespace Beagle
{

namespace SN
{

/*!
 *  \class SN::PackageBase beagle/SN/PackageBase.hpp "beagle/SN/PackageBase.hpp"
 *  \brief Package for adding objects related to SN.
 *  \ingroup SNF
 */
class PackageBase : public Beagle::Package
{

public:

	//! SN::PackageBase allocator type.
	typedef AllocatorT< PackageBase, Beagle::Package::Alloc > Alloc;
	//! SN::PackageBase handle type.
	typedef PointerT< PackageBase, Beagle::Package::Handle > Handle;
	//! SN::PackageBase bag type.
	typedef ContainerT< PackageBase, Beagle::Package::Bag > Bag;

	PackageBase(void);

	virtual void                  configure(Beagle::System& ioSystem);
	virtual Beagle::Package::Bag  listDependencies(void);

};

}
}

#endif // Beagle_SN_PackageBase_hpp
