
/*!
 *  \file   beagle/SN/TermSortedOp.hpp
 *  \brief  Definition of the class TermSortedOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef Beagle_SN_TermSortedOp_hpp
#define Beagle_SN_TermSortedOp_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/WrapperT.hpp"
#include "beagle/Operator.hpp"
#include "beagle/TerminationOp.hpp"
#include "beagle/Context.hpp"
#include "beagle/System.hpp"


namespace Beagle
{
namespace SN
{

/*!
 *  \class TermSortedOp beagle/SN/TermSortedOp.hpp "beagle/SN/TermSortedOp.hpp"
 *  \brief Termination criterion operator class that terminates when the network
 *      sorts all inputs correctly.
 *  \ingroup SNF
 */
class TermSortedOp : public TerminationOp
{

public:

	//! TermSortedOp allocator type
	typedef AllocatorT<TermSortedOp,TerminationOp::Alloc>
	Alloc;
	//! TermSortedOp handle type.
	typedef PointerT<TermSortedOp,TerminationOp::Handle>
	Handle;
	//! TermSortedOp bag type.
	typedef ContainerT<TermSortedOp,TerminationOp::Bag>
	Bag;

	explicit TermSortedOp(std::string inName="SN-TermSortedOp");
	virtual ~TermSortedOp()
	{ }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual bool terminate(const Beagle::Deme& inDeme, Beagle::Context& ioContext);

protected:

	UInt::Handle mNumInputs;                //!< Number of inputs of the sorting network.

};

}
}

#endif // Beagle_SN_TermSortedOp_hpp
