
/*!
 *  \file   beagle/SN/src/Individual.cpp
 *  \brief  Source code of class SN::Individual.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#include "beagle/SN.hpp"

using namespace Beagle;


/*!
 *  \brief Contruct an SN individual.
 *  \param inN Initial number of networks in vivarium.
 *  \param inModel Initial value of network handles.
 */
SN::Individual::Individual(unsigned int inN, SN::Network::Handle inModel) :
		Beagle::Individual(inN,inModel)
{ }


/*!
 *  \brief Get exact type of the individual.
 *  \return Exact type of the individual.
 */
const std::string& SN::Individual::getType() const
{
	Beagle_StackTraceBeginM();
	const static std::string lType("SN-Individual");
	return lType;
	Beagle_StackTraceEndM("const std::string& SN::Individual::getType() const");
}


/*!
 *  \brief Write content of an individual into a XML streamer.
 *  \param ioStreamer XML streamer to write the individual into.
 *  \param inIndent Whether XML output should be indented.
 *
 *  Override the base class implementation to cause indentation
 *  of the various items being written.
 */
void SN::Individual::writeContent(PACC::XML::Streamer& ioStreamer, bool inIndent) const
{
	Beagle_StackTraceBeginM();
	// Write members of the individual
	writeMembers(ioStreamer, true);
	// Write fitness
	writeFitness(ioStreamer, true);
	// Write genotypes
	writeGenotypes(ioStreamer, true);
	Beagle_StackTraceEndM("void SN::Individual::writeContent(PACC::XML::Streamer&,bool) const");
}


