
/*!
 *  \file   beagle/SN/src/PackageBase.cpp
 *  \brief  Source code of class SN::PackageBase.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#include "beagle/Beagle.hpp"

using namespace Beagle;


/*!
 *  \brief Construct package with basic SN objects.
 */
SN::PackageBase::PackageBase() :
		Beagle::Package("SN-PackageBase")
{ }


/*!
 *  \brief Configure system by adding necessary components into it.
 *  \param ioSystem System to configure.
 */
void SN::PackageBase::configure(Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();
	Factory& lFactory = ioSystem.getFactory();

	// Add available basic types to the factory
	lFactory.insertAllocator("Beagle::SN::AlgoDummy", new SN::AlgoDummy::Alloc);
	lFactory.insertAllocator("Beagle::SN::EstimateReconstructOp", new SN::EstimateReconstructOp::Alloc);
	lFactory.insertAllocator("Beagle::SN::Individual", new SN::Individual::Alloc);
	lFactory.insertAllocator("Beagle::SN::InitializationOp", new SN::InitializationOp::Alloc);
	lFactory.insertAllocator("Beagle::SN::MutationAddComparatorOp", new SN::MutationAddComparatorOp::Alloc);
	lFactory.insertAllocator("Beagle::SN::MutationTruncateConstructOp", new SN::MutationTruncateConstructOp::Alloc);
	lFactory.insertAllocator("Beagle::SN::Network", new SN::Network::Alloc);
	lFactory.insertAllocator("Beagle::SN::PackageBase", new SN::PackageBase::Alloc);
	lFactory.insertAllocator("Beagle::SN::TermSortedOp", new SN::TermSortedOp::Alloc);

	// Set aliases
	lFactory.aliasAllocator("Beagle::SN::AlgoDummy", "SN-AlgoDummy");
	lFactory.aliasAllocator("Beagle::SN::EstimateReconstructOp", "SN-EstimateReconstructOp");
	lFactory.aliasAllocator("Beagle::SN::Individual", "SN-Individual");
	lFactory.aliasAllocator("Beagle::SN::InitializationOp", "SN-InitializationOp");
	lFactory.aliasAllocator("Beagle::SN::Network", "SN-Network");
	lFactory.aliasAllocator("Beagle::SN::MutationAddComparatorOp", "SN-MutationAddComparatorOp");
	lFactory.aliasAllocator("Beagle::SN::MutationTruncateConstructOp", "SN-MutationTruncateConstructOp");
	lFactory.aliasAllocator("Beagle::SN::PackageBase", "SN-PackageBase");
	lFactory.aliasAllocator("Beagle::SN::TermSortedOp", "SN-TermSortedOp");

	// Set some basic concept-type associations in factory
	lFactory.setConcept("Algorithm", "SN-AlgoDummy");
	lFactory.setConcept("Individual", "SN-Individual");
	lFactory.setConcept("InitializationOp", "SN-InitializationOp");
	lFactory.setConcept("Genotype", "SN-Network");
	lFactory.setConcept("MutationOp", "SN-MutationAddComparatorOp");

	// Set some default parameters
	ioSystem.getRegister().setEntryDefaultValue("ec.pop.size",
	        new UIntArray(1,500),
	        "500");
	ioSystem.getRegister().setEntryDefaultValue("ec.sel.tournsize",
	        new UInt(2),
	        "2");

	Beagle_StackTraceEndM("void SN::PackageBase::configure(System&)");
}


/*!
 *  \brief List packages on which current package depends.
 *  \return Bag with the packages instanciation on which current package depends.
 *
 *  SN::PackageBase depends on Beagle::PackageBase.
 */
Beagle::Package::Bag SN::PackageBase::listDependencies(void)
{
	Beagle_StackTraceBeginM();
	Package::Bag lDependencies;
	lDependencies.push_back(new Beagle::PackageBase());
	return lDependencies;
	Beagle_StackTraceEndM("Beagle::Package::Bag SN::PackageBase::listDependencies(void)");
}
