/*
 *  Co-evolutionary Symbolic Regression (coev_symbreg):
 *  Mixed real-valued GA - GP co-evolution example with Open BEAGLE
 *
 *  Copyright (C) 2003-2006 by Christian Gagne and Jiachuan Wang
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   CoevSymbregMain.cpp
 *  \brief  Implementation of the main routine of co-evolutionary symbolic regression.
 *  \author Jiachuan Wang <jiacwang@ecs.umass.edu>
 *  \author Christian Gagne <cgagne@gmail.com>
 *  $Revision: 1.14 $
 *  $Date: 2007/08/10 20:32:22 $
 */

#include "beagle/Coev.hpp"
#include "CoSymEvalOp.hpp"
#include "TrainSetEvalOp.hpp"
#include "TrainSetThread.hpp"
#include "SymGPEvalOp.hpp"
#include "SymGPThread.hpp"

#include <cmath>
#include <cstdlib>
#include <iostream>
#include <stdexcept>
#include <vector>
#include <numeric>

using namespace std;
using namespace Beagle;


/*!
 *  \brief Main routine for the co-evolutionary symbolic regression example.
 *  \param argc Number of arguments on the command-line.
 *  \param argv Arguments on the command-line.
 *  \return Return value of the program.
 *  \ingroup CoSym
 */
int main(int argc, char** argv)
{
	try {
		// Creates and starts first thread/population for co-evolution.
		SymGPThread lThread1("symgp-thread.conf");
		lThread1.run();
		// Creates and starts second thread/population for co-evolution.
		TrainSetThread lThread2("trainset-thread.conf");
		lThread2.run();
		// Wait for the threads to finish.
		lThread1.wait();
		lThread2.wait();
	} catch(std::exception& inException) {
		std::cerr << "Standard exception catched:" << std::endl << std::flush;
		std::cerr << inException.what() << std::endl << std::flush;
		return 1;
	}
}
