/*
 *  Iterated Prisoner's Dilemma (ipd):
 *  Co-evolution GA example
 *
 *  Copyright (C) 2003-2007 by Christian Gagne and Marc Parizeau
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   IPDMain.cpp
 *  \brief  Implementation of the main routine for the iterated prisoner's dilemma problem.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.12 $
 *  $Date: 2007/08/10 20:32:23 $
 */

#include "beagle/Coev.hpp"
#include "IPDThread.hpp"

#include <cstdlib>
#include <iostream>
#include <stdexcept>
#include <vector>
#include <numeric>

using namespace std;
using namespace Beagle;


/*!
 *  \brief Main routine for the iteration prisoner's dilemma problem.
 *  \param argc Number of arguments on the command-line.
 *  \param argv Arguments on the command-line.
 *  \return Return value of the program.
 *  \ingroup IPD
 */
int main(int argc, char** argv)
{
	try {

		// Starting and first thread/population for co-evolution.
		std::cout << "Starting first thread for co-evolution with configuration file '";
		std::cout << "ipd-thread1.conf'" << std::endl << std::flush;
		IPDThread lThread1("ipd-thread1.conf");
		lThread1.run();
		
		// Starting and second thread/population for co-evolution.
		std::cout << "Starting second thread for co-evolution with configuration file '";
		std::cout << "ipd-thread2.conf'" << std::endl << std::flush;
		IPDThread lThread2("ipd-thread2.conf");
		lThread2.run();

		// Waiting for the threads to finish.
		std::cout << "Waiting for the threads to finish" << std::endl << std::flush;
		lThread1.wait();
		lThread2.wait();
		std::cout << "Threads finished, exiting" << std::endl << std::flush;

	} catch(std::exception& inException) {
		std::cerr << "Standard exception catched:" << std::endl << std::flush;
		std::cerr << inException.what() << std::endl << std::flush;
		return 1;
	}
}
