/*
 *  Iterated Prisoner's Dilemma (ipd):
 *  Co-evolution GA example
 *
 *  Copyright (C) 2003-2007 by Christian Gagne and Marc Parizeau
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   IPDThread.hpp
 *  \brief  Definition of the type IPDThread.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.13 $
 *  $Date: 2007/08/10 20:32:23 $
 */

#ifndef IPDThread_hpp
#define IPDThread_hpp

#include "beagle/Coev.hpp"
#include "PACC/Threading.hpp"
#include <vector>


/*!
 *  \class IPDThread IPDThread.hpp "IPDThread.hpp"
 *  \brief Co-evolution thread class for the iterated prisoner's dilemma.
 *  \ingroup IPD
 */
class IPDThread : public PACC::Threading::Thread
{

public:
	IPDThread(std::string inConfigFilename="ipd.conf");
	~IPDThread();

protected:

	virtual void main();

	std::string mConfigFilename;   //!< Configuration filename

};

#endif // IPDThread_hpp

