#include <iostream>
#include <string>
#include "beagle/Beagle.hpp"
#include "beagle/Distrib/Island.hpp"
#include "PACC/Socket.hpp"
#include "Test.hpp"


using namespace std;
using namespace Beagle;
using namespace Beagle::Distrib;

/*!
 *  \brief Receive messages via Communications system component.
 *
 *  Create Communications system component and wait for messages to
 *  arrive.  Once messages have been receive, log them via
 *  Communications::logAllMessages().
 */
int main(int argc, char** argv)
{
	try {
		// Create System
		System::Handle lSystem = new System;

		// Register "port.num"
		UInt::Handle lPortNum;
		Register::Description lDescription(
		    "Port to listen on",
		    "UInt",
		    "4500",
		    "The TCP/IP port number on which to listen."
		);
		lPortNum = castHandleT<UInt>(
		               lSystem->getRegister().insertEntry("port.num", new UInt(4500), lDescription));
		Beagle_NonNullPointerAssertM( lPortNum );

		// Create and add Communications system component
		Island::CommunicationsTCPIP::Handle lComms = new Island::CommunicationsTCPIP(lPortNum->getWrappedValue());
		lSystem->addComponent(lComms);

		// Initialize
		lSystem->initialize(argc,argv);
		lSystem->initComponents();

		// Create Context
		Context lContext;
		lContext.setSystemHandle(lSystem);

		// Wait for messages to be received
		cout << "\n\nPress enter to finish receiving messages.\n" << endl;
		char c;
		cin >> c;

		// Get and log messages
		lComms->getMessages(lContext);
		lComms->logAllMessages(lContext);
	}
	Beagle_Test_CatchEverythingM;
}
