#include <iostream>

#include "beagle/Beagle.hpp"
#include "beagle/Distrib/Island.hpp"
#include "Test.hpp"

#include "PACC/Socket.hpp"

using namespace std;
using namespace Beagle;
using namespace Beagle::Distrib;

/*!
 *  \brief Tests the TransportReceiveOp operator by creating a small
 *  population and applying the operator.
 */
int main(int argc, char** argv)
{
	try {
		Beagle_StackTraceBeginM();
		// Initialize BeagleTest
		Test::BeagleTest lBeagleTest;
		System::Handle lSystem = lBeagleTest.requestSystem();
		Context::Handle lContext = lBeagleTest.requestContext();
		lBeagleTest.requestVivarium();

		// Create and install Comms component
		Island::CommunicationsTCPIP::Handle lComms = new Island::CommunicationsTCPIP(4500);
		lSystem->addComponent( lComms );

		// Create a CommunicationsOp
		Island::CommunicationsOp::Handle lCommsOp =
		    new Island::CommunicationsOp( lComms->getName() );
		// Create a transport receive operator
		Island::TransportReceiveOp::Handle lTransportReceiveOp
		= new Island::TransportReceiveOp(lComms->getName());

		// Initialize
		lCommsOp->registerParams(*lSystem);
		lTransportReceiveOp->registerParams(*lSystem);
		lBeagleTest.initialize(argc,argv);
		lCommsOp->init(*lSystem);
		lTransportReceiveOp->init(*lSystem);

		// Wait for individuals to be sent
		Beagle_LogBasicM(
		    lSystem->getLogger(),
		    "main", "main()",
		    "Sleeping for 10 seconds while waiting for messages to arrive"
		);
		PACC::Threading::Thread::sleep(10.);

		// Test the operator
		Deme::Handle lDeme = lBeagleTest.getDeme();
		lCommsOp->operate(*lDeme, *lContext);
		lTransportReceiveOp->operate(*lDeme, *lContext);
		Beagle_StackTraceEndM("int main(int, char**)");
	}
	Beagle_Test_CatchEverythingM;
}

