#include <iostream>
#include <string>
#include "beagle/Beagle.hpp"
#include "beagle/Distrib/Island.hpp"
#include "PACC/Socket.hpp"
#include "Test.hpp"


using namespace std;
using namespace Beagle;
using namespace Beagle::Distrib;

/*!
 *  \brief Tests the TransportSendOp operator by creating a small
 *  population and applying the operator.
 */
int main(int argc, char** argv)
{
	try {
		// Initialize BeagleTest
		Test::BeagleTest lBeagleTest;
		System::Handle lSystem = lBeagleTest.requestSystem();
		Island::Communications::Handle lComms = new Island::CommunicationsTCPIP;
		lSystem->addComponent( lComms );
		Context::Handle lContext = lBeagleTest.requestContext();
		lBeagleTest.requestVivarium();
		lBeagleTest.initialize(argc,argv);

		// Create a transport send operator
		Island::TransportSendOp::Handle lTransportSendOp =
		    new Island::TransportSendOp(lComms->getName());

		// Read configuration (normally done by an Evolver)
		std::string lFilename("testTransportSendOp.conf");
		Beagle_LogBasicM(
		    lSystem->getLogger(),
		    "main", "main",
		    std::string("Opening file '")+lFilename+"'"
		);
		PACC::XML::Document lDoc;
		lDoc.parse(lFilename);
		PACC::XML::ConstIterator lRootNode = lDoc.getFirstDataTag();
		if (!lRootNode) {
			Beagle_LogBasicM(
			    lSystem->getLogger(),
			    "main", "main",
			    std::string("Failed to obtain a 'first data tag' from file '")+
			    lFilename+"'"
			);
		} else {
			PACC::XML::ConstFinder lFinder( lRootNode );
			PACC::XML::ConstIterator lNode
			= lFinder.find(std::string("//")+lTransportSendOp->getName());
			if (!lNode) {
				Beagle_LogBasicM(
				    lSystem->getLogger(),
				    "main", "main",
				    std::string("Failed to obtain '")+lTransportSendOp->getName()+
				    "' node from file '"+lFilename+"'"
				);
			} else {
				lTransportSendOp->readWithSystem( lNode, *lSystem );
			}
		}

		// Initialize operator
		lTransportSendOp->registerParams(*lSystem);
		lTransportSendOp->init(*lSystem);


		// Test the operator
		Deme::Handle lDeme = lBeagleTest.getDeme();
		lTransportSendOp->operate(*lDeme, *lContext);
	}
	Beagle_Test_CatchEverythingM;
}


