#include <iostream>
#include <cstdlib>

// Compute checksum of hex file.
// Adapted from billyzelsnack's post at
// http://robosavvy.com/forum/viewtopic.php?p=6293
unsigned char computeChecksum(FILE* fp)
{
    unsigned char totalChecksum=0;

    while(!feof(fp)) {
        unsigned char lineChecksum=0;
   
        int numPairs;
        int addressHi;
        int addressLo;
        int recordType;
        if(fscanf(fp,":%02x%02x%02x%02x",&numPairs,&addressHi,&addressLo,&recordType)!=4) {
            throw "error reading hex file";
        }
        lineChecksum+=numPairs;
        lineChecksum+=addressHi;
        lineChecksum+=addressLo;
        lineChecksum+=recordType;
      
        //printf("%d %d %d %d\n", numPairs, addressHi, addressLo, recordType);
      
        for( int ii=0; ii<numPairs; ii++ ) {
            int value;
            if(fscanf(fp,"%02x",&value)!=1) {
                throw "error reading value from hex file";
            }
            if(recordType==0) {
                //fprintf(fpout,"%c",value);
            
                //unsigned char buffer[1]={(unsigned char)(value)};
                //if(serial_write(1,buffer)!=1) {
                //    return false;
                //}

                totalChecksum+=value;
            }
            lineChecksum+=value;
        }
        lineChecksum=(~lineChecksum)+1;
      
        int checksum;
        if(fscanf(fp,"%02x\n",&checksum)!=1) {
            throw "error reading checksum from hex file";
        }
      
        //printf("%02x %02x",checksum, lineChecksum);
      
        if(checksum!=lineChecksum) {
            throw "error: bad line checksum";
        }
    }
   
    //printf("%02x\n",totalChecksum);
         
    return totalChecksum;
} 


int main(int argc, char* argv[]) {
    try {
        if (argc < 2) {
            throw "error: specify hex file as argument";
        }
        std::cout << computeChecksum(fopen(argv[1], "r"));
    }
    catch (char const* msg) {
        std::cout << msg << std::endl;
    }
    return 0;
}


