/*
 * CM-2+ Walking Tables Control Program
 * Date : May-Jun 2010
 * Author : Vinod Valsalam
 *
 * Based on the Machine Metabolism Control Program
 * by Jeremy Blum and Franz Nigl.
 */

#include "dynamixel.h"

/*
SIGNAL() UART0 Rx Interrupt - write data to buffer
*/
SIGNAL(SIG_UART0_RECV)
{
	gbpRxInterruptBuffer[(gbRxBufferWritePointer++)] = RXD0_DATA;
}

ISR(TIMER1_COMPA_vect)
{
	sei();	// allow other interrupts (for reading UART!)///
	time += 0.005;///
}


void PortInitialize(void)
{
	DDRA = DDRB = DDRC = DDRD = DDRE = DDRF = 0;  //Set all port to input direction first.
	PORTB = PORTC = PORTD = PORTE = PORTF = PORTG = 0x00; //PortData initialize to 0
	cbi(SFIOR,2); //All Port Pull Up ready
	DDRE |= _BV(BIT_RS485_DIRECTION0)|_BV(BIT_RS485_DIRECTION1); //set output the bit RS485direction

// For CM-5
	DDRD |= _BV(BIT_ZIGBEE_RESET)|_BV(BIT_ENABLE_RXD_LINK_PC)|_BV(BIT_ENABLE_RXD_LINK_ZIGBEE);

	PORTD &= ~_BV(BIT_LINK_PLUGIN); // no pull up
	PORTD |= _BV(BIT_ZIGBEE_RESET);
	PORTD |= _BV(BIT_ENABLE_RXD_LINK_PC);
	PORTD |= _BV(BIT_ENABLE_RXD_LINK_ZIGBEE);

// For CM-2PLUS

	DDRB |= _BV(BIT_ENABLE_RS485)|_BV(BIT_ENABLE_HDBRT);

	PORTB &= ~_BV(BIT_ENABLE_HDBRT);
	PORTB |= _BV(BIT_ENABLE_RS485);
}

/*
SerialInitialize() set Serial Port to initial state.
Vide Mega128 Data sheet about Setting bit of register.
SerialInitialize() needs port, Baud rate, Interrupt value.

*/
void SerialInitialize(byte bPort, byte bBaudrate, byte bInterrupt)
{
	if(bPort == SERIAL_PORT0)
	{
		UBRR0H = 0; UBRR0L = bBaudrate;
		UCSR0A = 0x02;  UCSR0B = 0x18;
		if(bInterrupt&RX_INTERRUPT) sbi(UCSR0B,7); // RxD interrupt enable
		UCSR0C = 0x06; UDR0 = 0xFF;
		sbi(UCSR0A,6);//SET_TXD0_FINISH; // Note. set 1, then 0 is read
	}
	else if(bPort == SERIAL_PORT1)
	{
		UBRR1H = 0; UBRR1L = bBaudrate;
		UCSR1A = 0x02;  UCSR1B = 0x18;
		if(bInterrupt&RX_INTERRUPT) sbi(UCSR1B,7); // RxD interrupt enable
		UCSR1C = 0x06; UDR1 = 0xFF;
		sbi(UCSR1A,6);//SET_TXD1_FINISH; // Note. set 1, then 0 is read
	}
}


void Initialize(void)
{
	//INITIALIZE DEVICE SETTINGS
	PortInitialize(); //Port In/Out Direction Definition
	RS485_RXD; //Set RS485 Direction to Input State.
	SerialInitialize(SERIAL_PORT0,1,RX_INTERRUPT);//RS485 Initializing(RxInterrupt)
	SerialInitialize(SERIAL_PORT1,DEFAULT_BAUD_RATE,0); //RS232 Initializing(None Interrupt)
	gbRxBufferReadPointer = gbRxBufferWritePointer = 0; //RS485 RxBuffer Clearing.
	TCCR1B |= (1 << WGM12); // Configure timer 1 for CTC mode
	TIMSK |= (1 << OCIE1A); // Enable CTC interrupt
	sei();  //Enable Interrupt -- Compiler Function
	OCR1A   = 80;
	TCCR1B |= ((1 << CS10) | (1 << CS12)); // Set up timer with prescale 1024
	time = 0;
}

/*
TxPacket() send data to RS485.
TxPacket() needs 3 parameter; ID of Dynamixel, Instruction byte, Length of parameters.
TxPacket() return length of Return packet from Dynamixel.
*/
byte TxPacket(byte bID, byte bInstruction, byte bParameterLength)
{
	byte bCount,bCheckSum,bPacketLength;

	gbpTxBuffer[0] = 0xff;
	gbpTxBuffer[1] = 0xff;
	gbpTxBuffer[2] = bID;

// if ID is over than 100, CM-2PLUS will communicate with AX Series.
// Else CM-2PLUS will communicate with DX/RX Series.
	if( (bID > 99) && (bID !=254) )	(ENABLE_HDBRT);
	else					(ENABLE_RS485);

	gbpTxBuffer[3] = bParameterLength+2; //Length(Paramter,Instruction,Checksum)
	gbpTxBuffer[4] = bInstruction;
	for(bCount = 0; bCount < bParameterLength; bCount++)
	{
		gbpTxBuffer[bCount+5] = gbpParameter[bCount];
	}
	bCheckSum = 0;
	bPacketLength = bParameterLength+4+2;
	for(bCount = 2; bCount < bPacketLength-1; bCount++) //except 0xff,checksum
	{
		bCheckSum += gbpTxBuffer[bCount];
	}
	gbpTxBuffer[bCount] = ~bCheckSum; //Writing Checksum with Bit Inversion

	RS485_TXD;
	for(bCount = 0; bCount < bPacketLength; bCount++)
	{
		sbi(UCSR0A,6);//SET_TXD0_FINISH;
		TxD80(gbpTxBuffer[bCount]);
	}
	while(!CHECK_TXD0_FINISH); //Wait until TXD Shift register empty
	RS485_RXD;
	return(bPacketLength);
}

/*
RxPacket() read data from buffer.
RxPacket() need a Parameter; Total length of Return Packet.
RxPacket() return Length of Return Packet.
*/
byte RxPacket(byte bRxPacketLength)
{
#define RX_TIMEOUT_COUNT2   3000L
#define RX_TIMEOUT_COUNT1  (RX_TIMEOUT_COUNT2*10L)
	unsigned long ulCounter;
	byte bCount, bLength, bChecksum;
	byte bTimeout;
	bTimeout = 0;
	for(bCount = 0; bCount < bRxPacketLength; bCount++)
	{
		ulCounter = 0;
		while(gbRxBufferReadPointer == gbRxBufferWritePointer)
		{
			if(ulCounter++ > RX_TIMEOUT_COUNT1)
			{
				bTimeout = 1;
				break;
			}
		}

		if(bTimeout) break;
		gbpRxBuffer[bCount] = gbpRxInterruptBuffer[gbRxBufferReadPointer++];
	}
	bLength = bCount;
	bChecksum = 0;
	if(gbpTxBuffer[2] != BROADCASTING_ID)
	{
		if(bTimeout && bRxPacketLength != 255)
		{
			TxDString("[TxID ");TxD32Dec(gbpTxBuffer[2]);TxDString(" Error: RxD Timeout]\r\n");
			CLEAR_BUFFER;
		}

		if(bLength > 3) //checking is available.
		{
			if(gbpRxBuffer[0] != 0xff || gbpRxBuffer[1] != 0xff )
			{
				TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: Wrong Header]\r\n");
				CLEAR_BUFFER;
				return 0;
			}
			if(gbpRxBuffer[2] != gbpTxBuffer[2] )
			{
				TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: TxID != RxID]\r\n");
				CLEAR_BUFFER;
				return 0;
			}
			if(gbpRxBuffer[3] != bLength-4)
			{
				TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: Wrong Length]\r\n");
				CLEAR_BUFFER;
				return 0;
			}
			if(gbpRxBuffer[4] != 0)
			{
				if((gbpRxBuffer[4] & 1) == 1) {TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: input voltage out of rage]\r\n");}
				if((gbpRxBuffer[4] & 2) == 2) {TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: goal position out of rage]\r\n");}
				if((gbpRxBuffer[4] & 4) == 4) {TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: overheating]\r\n");}
				if((gbpRxBuffer[4] & 8) == 8) {TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: instruction out of rage]\r\n");}
				if((gbpRxBuffer[4] & 16) == 16) {TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: bad instruction packet checksum]\r\n");}
				if((gbpRxBuffer[4] & 32) == 32) {TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: torque overload]\r\n");}
				if((gbpRxBuffer[4] & 64) == 64) {TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: bad instruction]\r\n");}
				CLEAR_BUFFER;
				return 0;
			}
			for(bCount = 2; bCount < bLength; bCount++) bChecksum += gbpRxBuffer[bCount];
			if(bChecksum != 0xff)
			{
				
				TxDString("[RxID ");TxD32Dec(gbpRxBuffer[2]);TxDString(" Error: Wrong CheckSum]\r\n");
				CLEAR_BUFFER;
				return 0;
			}
		}
	}
	return bLength;
}


/*
TxD8Hex() print data seperatly.
ex> 0x1a -> '1' 'a'.
*/
void TxD8Hex(byte bSentData)
{
	byte bTmp;

	bTmp =((byte)(bSentData>>4)&0x0f) + (byte)'0';
	if(bTmp > '9') bTmp += 7;
	TxD8(bTmp);
	bTmp =(byte)(bSentData & 0x0f) + (byte)'0';
	if(bTmp > '9') bTmp += 7;
	TxD8(bTmp);
}

/*
TxD80() send data to USART 0.
*/
void TxD80(byte bTxdData)
{
	while(!TXD0_READY);
	TXD0_DATA = bTxdData;
}

/*
TXD81() send data to USART 1.
*/
void TxD81(byte bTxdData)
{
	while(!TXD1_READY);
	TXD1_DATA = bTxdData;
}

/*
TXD32Dex() change data to decimal number system
*/
void TxD32Dec(long lLong)
{
	byte bCount, bPrinted;
	long lTmp,lDigit;
	bPrinted = 0;
	if(lLong < 0)
	{
		lLong = -lLong;
		TxD8('-');
	}
	lDigit = 1000000000L;
	for(bCount = 0; bCount < 9; bCount++)
	{
		lTmp = (byte)(lLong/lDigit);
		if(lTmp)
		{
			TxD8(((byte)lTmp)+'0');
			bPrinted = 1;
		}
		else if(bPrinted) TxD8(((byte)lTmp)+'0');
		lLong -= ((long)lTmp)*lDigit;
		lDigit = lDigit/10;
	}
	lTmp = (byte)(lLong/lDigit);
	/*if(lTmp)*/ TxD8(((byte)lTmp)+'0');
}

/*
TxDString() prints data in ASCII code.
*/
void TxDString(char *bData)
{
	while(*bData)
	{
		TxD8(*bData++);
	}
}

/*
RxD81() read data from UART1.
RxD81() return Read data.
*/
byte RxD81(void)
{
	while(!RXD1_READY);
	return(RXD1_DATA);
}


/*
PrintBuffer() print data in Hex code.
PrintBuffer() needs two parameter; name of Pointer(gbpTxBuffer, gbpRxBuffer)
*/
void PrintBuffer(byte *bpPrintBuffer, byte bLength)
{
	byte bCount;
	for(bCount = 0; bCount < bLength; bCount++)
	{
		TxD8Hex(bpPrintBuffer[bCount]);
		TxD8(' ');
	}
	TxDString("(LEN:");TxD8Hex(bLength);TxD8(')');
}

/*
Print value of Baud Rate.
*/
void PrintBaudrate(void)
{
	TxDString("\r\n RS232:");TxD32Dec((16000000L/8L)/((long)UBRR1L+1L) );  TxDString(" BPS,");
	TxDString("  RS485:");TxD32Dec((16000000L/8L)/((long)UBRR0L+1L) );  TxDString(" BPS");
}


/*
Read one byte from the given address of the servo.
*/
uint8_t Read8(byte servoID, byte address)
{
  byte bTxPacketLength,bRxPacketLength;
  gbpParameter[0] = address;
  gbpParameter[1] = 1; //Read Length
  bTxPacketLength = TxPacket(servoID,INST_READ,2);
  bRxPacketLength = RxPacket(DEFAULT_RETURN_PACKET_SIZE+gbpParameter[1]);
  if (bRxPacketLength == DEFAULT_RETURN_PACKET_SIZE+gbpParameter[1])
  {
    return gbpRxBuffer[5];
  }
  return 0xff;
}


/*
Read two bytes starting at the given address of the servo.
*/
uint16_t Read16(byte servoID, byte address)
{
  byte bTxPacketLength,bRxPacketLength;
  gbpParameter[0] = address;
  gbpParameter[1] = 2; //Read Length
  bTxPacketLength = TxPacket(servoID,INST_READ,2);
  bRxPacketLength = RxPacket(DEFAULT_RETURN_PACKET_SIZE+gbpParameter[1]);
  if (bRxPacketLength == DEFAULT_RETURN_PACKET_SIZE+gbpParameter[1])
  {
    return (gbpRxBuffer[6]<<8) + gbpRxBuffer[5];
  }
  return 0xffff;
}


/*
Write one byte of data to the given address of the servo.
*/
void Write8(byte servoID, byte address, uint8_t data)
{
  byte bTxPacketLength,bRxPacketLength;
  gbpParameter[0] = address;
  gbpParameter[1] = data;
  bTxPacketLength = TxPacket(servoID,INST_WRITE,2);
  bRxPacketLength = RxPacket(DEFAULT_RETURN_PACKET_SIZE);  // receive the status packet
} 


/*
Write two bytes of data starting at the given address of the servo.
*/
void Write16(byte servoID, byte address, uint16_t data)
{
  byte bTxPacketLength,bRxPacketLength;
  gbpParameter[0] = address;
  gbpParameter[1] = data & 0xff;
  gbpParameter[2] = data >> 8;
  bTxPacketLength = TxPacket(servoID,INST_WRITE,3);
  bRxPacketLength = RxPacket(DEFAULT_RETURN_PACKET_SIZE);  // receive the status packet
}


void SetGoalPositionWithSpeed(byte servoID,uint16_t pos,uint16_t speed)
{
  byte ph,pl,sh,sl,bTxPacketLength,bRxPacketLength;

  ph = pos >> 8;
  pl = pos & 0xff;
  sh = speed >> 8;
  sl = speed & 0xff;
 
  gbpParameter[0] = P_GOAL_POSITION_L; //Address of Firmware Version
  gbpParameter[1] = pl; //Writing Data P_GOAL_POSITION_L
  gbpParameter[2] = ph; //Writing Data P_GOAL_POSITION_H
  gbpParameter[3] = sl; //Writing Data P_GOAL_SPEED_L
  gbpParameter[4] = sh; //Writing Data P_GOAL_SPEED_H
  bTxPacketLength = TxPacket(servoID,INST_WRITE,5);
  bRxPacketLength = RxPacket(DEFAULT_RETURN_PACKET_SIZE);
} 


// Max possible delay is 6.5535 seconds; see the following page for more info:
// http://www.nongnu.org/avr-libc/user-manual/group__util__delay.html
// For longer delays, use the DelaySec() function.
void DelayMilliSec(double millisec)
{ 
	_delay_ms(millisec);
}


void DelaySec(unsigned int sec)
{ 
 int i;
 for (i=0; i<sec; i++) _delay_ms(1000);
}
