
/*!
 *  \file   netutils.h
 *  \author Vinod Valsalam
 */

#ifndef netutils_h
#define netutils_h

#include "network.h"
extern Network gNetwork;


/*!
 *  \brief Return the current activation of the given node.
 */
inline double getActivation(uint8_t inNodeId)
{
    return gNetwork.nodes[inNodeId]->activations[gNetwork.activationIndex];
}

/*!
 *  \brief Return the previous activation of the given node.
 */
inline double getPreviousActivation(uint8_t inNodeId)
{
    return gNetwork.nodes[inNodeId]->activations[gNetwork.activationIndex^1];
}

/*!
 *  \brief Set the current activation of the given node.
 */
inline void setActivation(uint8_t inNodeId, double inActivation)
{
    gNetwork.nodes[inNodeId]->activations[gNetwork.activationIndex] = inActivation;
}

/*!
 *  \brief Propagate activation in the network by one link.
 *  \param inActivations Array containing the activations of input nodes.
 */
void activateOnce(const double* inActivations);

/*!
 *  \brief Activate the network until all nodes are active.
 *  \param inActivations Array containing the activations of input nodes.
 */
void activateNetwork(const double* inActivations);

/*!
 *  \brief Refresh the network to start fresh activations.
 */
void refreshNetwork(void);

/*!
 *  \brief Return the output activations of the network.
 */
void getNetworkOutputs(double* outActivations);


#endif // netutils_h
