
/*!
 *  \file   GuiRobotEvalOp.hpp
 *  \brief  Definition of the type GuiRobotEvalOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef GuiRobotEvalOp_hpp
#define GuiRobotEvalOp_hpp

#include <QObject>
#include <QTimer>
#include <string>
#include <vector>

#include "beagle/NN.hpp"
#include "explorer/NN.hpp"
#include "VizRobotEvalOp.hpp"

namespace Beagle
{
namespace NN
{


/*!
 *  \class GuiRobotEvalOp GuiRobotEvalOp.hpp "GuiRobotEvalOp.hpp"
 *  \brief Double pole balancing evaluation operator for neural networks (Markov version)
 *  \ingroup NNF
 */
class GuiRobotEvalOp : public QObject, public VizRobotEvalOp
{

	Q_OBJECT

public:

	//! GuiRobotEvalOp allocator type.
	typedef Explorer::AllocatorT<GuiRobotEvalOp,VizRobotEvalOp::Alloc>
	Alloc;
	//! GuiRobotEvalOp handle type.
	typedef PointerT<GuiRobotEvalOp,VizRobotEvalOp::Handle>
	Handle;
	//! GuiRobotEvalOp bag type.
	typedef ContainerT<GuiRobotEvalOp,VizRobotEvalOp::Bag>
	Bag;

	explicit GuiRobotEvalOp(const std::string& inName="NN-GuiRobotEvalOp");
	virtual ~GuiRobotEvalOp()
    { }

	virtual FitnessIndexed::Handle evaluate(NN::Graph& ioGraph, Context& ioContext);
	virtual void init(Beagle::System& ioSystem);

protected slots:

    void renderEventLoop();
    void enterEventLoop();
    void exitEventLoop();

protected:

    Explorer::NN::Interaction::Handle   mInteraction;       //!< Object that enables interacting with the GUI.
    QTimer*                             mRenderTimer;       //!< Timer to periodically call render inside break points.

	virtual void updateStep(opal::real inTime,
                            opal::real inZ, opal::real inXTarget, opal::real inXActual,
                            const NN::Graph& inGraph, Context& ioContext);

};

}
}


#endif // GuiRobotEvalOp_hpp
