
/*!
 *  \file   PrintRobotEvalOp.hpp
 *  \brief  Definition of the type PrintRobotEvalOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef PrintRobotEvalOp_hpp
#define PrintRobotEvalOp_hpp

#include "PACC/Util.hpp"
#include "beagle/NN.hpp"
#include <string>
#include <vector>
#include "RobotEvalOp.hpp"

namespace Beagle
{
namespace NN
{


/*!
 *  \class PrintRobotEvalOp PrintRobotEvalOp.hpp "PrintRobotEvalOp.hpp"
 *  \brief Convert neural network controller to version for printed robot and evaluate.
 *  \ingroup NNF
 */
class PrintRobotEvalOp : public RobotEvalOp
{

public:

	//! PrintRobotEvalOp allocator type.
	typedef AllocatorT<PrintRobotEvalOp,RobotEvalOp::Alloc>
	Alloc;
	//! PrintRobotEvalOp handle type.
	typedef PointerT<PrintRobotEvalOp,RobotEvalOp::Handle>
	Handle;
	//! PrintRobotEvalOp bag type.
	typedef ContainerT<PrintRobotEvalOp,RobotEvalOp::Bag>
	Bag;

	explicit PrintRobotEvalOp(const std::string& inName="NN-PrintRobotEvalOp");
	virtual ~PrintRobotEvalOp()
    { }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual FitnessIndexed::Handle evaluate(NN::Graph& ioGraph, Context& ioContext);

protected:

	Bool::Handle        mConvertNetwork;      //!< Whether to convert or evaluate neural network.

    void convertNetwork(const NN::Graph& ioGraph, const Beagle::Context& inContext);

};

}
}


#endif // PrintRobotEvalOp_hpp
