
/*!
 *  \file   VizRobotEvalOp.hpp
 *  \brief  Definition of the type VizRobotEvalOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef VizRobotEvalOp_hpp
#define VizRobotEvalOp_hpp

#include "PACC/Util.hpp"
#include "beagle/NN.hpp"
#include <string>
#include <vector>
#include "RobotEvalOp.hpp"
#include "RobotApp.h"

namespace Beagle
{
namespace NN
{


/*!
 *  \class VizRobotEvalOp VizRobotEvalOp.hpp "VizRobotEvalOp.hpp"
 *  \brief Double pole balancing evaluation operator for neural networks (Markov version)
 *  \ingroup NNF
 */
class VizRobotEvalOp : public RobotEvalOp
{

public:

	//! VizRobotEvalOp allocator type.
	typedef AllocatorT<VizRobotEvalOp,RobotEvalOp::Alloc>
	Alloc;
	//! VizRobotEvalOp handle type.
	typedef PointerT<VizRobotEvalOp,RobotEvalOp::Handle>
	Handle;
	//! VizRobotEvalOp bag type.
	typedef ContainerT<VizRobotEvalOp,RobotEvalOp::Bag>
	Bag;

	explicit VizRobotEvalOp(const std::string& inName="NN-VizRobotEvalOp");
	virtual ~VizRobotEvalOp()
    { }

	virtual void registerParams(Beagle::System& ioSystem);
	virtual FitnessIndexed::Handle evaluate(NN::Graph& ioGraph, Context& ioContext);
	virtual void init(Beagle::System& ioSystem);

protected:

    static RobotApp*    mRobotApp;      //!< The opal-ogre robot visualization application (a singleton).
    PACC::Timer         mTimer;         //!< Timer used for making visualization run in real time.
    double              mPrvTime;		//!< Previously recorded timer value.
	double				mCurTime;		//!< Current recording of timer value.
	double				mAccTime;		//!< Buffer containing produced/consumed time.
	Bool::Handle        mRealTime;      //!< Whether to run the simulation in real time.
	Bool::Handle        mShowFitness;   //!< Whether to show fitness in the visualization.

    virtual void createAgent(opal::real inX, opal::real inY, opal::real inZ, opal::real inScale);
    virtual void removeAgent();
    virtual void createEnviron(Context& ioContext);
    virtual void removeEnviron();
    virtual void simulateAgent(opal::real inDt);

};

}
}


#endif // VizRobotEvalOp_hpp
