
/*!
 *  \file   GuiEvaluationOp.cpp
 *  \brief  Implementation of the class GuiEvaluationOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#include "GuiEvaluationOp.hpp"

using namespace Beagle;


/*!
 *  \brief Construct a new robot evaluation operator.
 *  \param inName Name of the operator.
 */
SN::GuiEvaluationOp::GuiEvaluationOp(const std::string& inName) :
		SN::EvaluationOp(inName),
        mInteraction(NULL)
{ }


/*!
 *  \brief Override the evaluate method to allow interaction with the GUI.
 *  \param ioNetwork Network to evaluate.
 *  \param ioContext Evolutionary context.
 *  \return Handle to the fitness measure.
 */
Fitness::Handle SN::GuiEvaluationOp::evaluate(SN::Network& ioNetwork, Context& ioContext)
{
	Beagle_StackTraceBeginM();
  
    mInteraction->update(ioNetwork, ioContext);
    return SN::EvaluationOp::evaluate(ioNetwork, ioContext);

	Beagle_StackTraceEndM("Fitness::Handle SN::GuiEvaluationOp::evaluate(SN::Network& ioNetwork, Context& ioContext)");
}


/*!
 * \brief Initialize the operator.
 * \param ioSystem System to use.
 */
void SN::GuiEvaluationOp::init(Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();

	if (isInitialized())
		return;

	SN::EvaluationOp::init(ioSystem);

    // Create an SN interaction object to interact with the GUI.
    mInteraction = new Explorer::SN::Interaction;

	Beagle_StackTraceEndM("void SN::GuiEvaluationOp::init(System& ioSystem)");
}
