
/*!
 *  \file   GuiEvaluationOp.hpp
 *  \brief  Definition of the type GuiEvaluationOp.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#ifndef GuiEvaluationOp_hpp
#define GuiEvaluationOp_hpp

#include <QObject>
#include <string>
#include <vector>

#include "beagle/SN.hpp"
#include "explorer/SN.hpp"

namespace Beagle
{
namespace SN
{


/*!
 *  \class GuiEvaluationOp GuiEvaluationOp.hpp "GuiEvaluationOp.hpp"
 *  \brief Sorting network evaluation operator with Explorer GUI.
 *  \ingroup SNF
 */
class GuiEvaluationOp : public QObject, public EvaluationOp
{

	Q_OBJECT

public:

	//! GuiEvaluationOp allocator type.
	typedef Explorer::AllocatorT<GuiEvaluationOp,EvaluationOp::Alloc>
	Alloc;
	//! GuiEvaluationOp handle type.
	typedef PointerT<GuiEvaluationOp,EvaluationOp::Handle>
	Handle;
	//! GuiEvaluationOp bag type.
	typedef ContainerT<GuiEvaluationOp,EvaluationOp::Bag>
	Bag;

	explicit GuiEvaluationOp(const std::string& inName="SN-GuiEvaluationOp");
	virtual ~GuiEvaluationOp()
    { }

	virtual Fitness::Handle evaluate(SN::Network& ioNetwork, Context& ioContext);
	virtual void init(Beagle::System& ioSystem);

protected:

    Explorer::SN::Interaction::Handle   mInteraction;       //!< Object that enables interacting with the GUI.

};

}
}


#endif // GuiEvaluationOp_hpp
