
/*!
 *  \file   main.cpp
 *  \brief  Main routine for the sorting network problem.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#include "explorer/SN.hpp"
#include "GuiEvaluationOp.hpp"

#define _USE_MATH_DEFINES
#include <cmath>
#include <cstdlib>
#include <iostream>
#include <stdexcept>
#include <vector>
#include <numeric>


int main(int argc, char *argv[])
{
	Explorer::Application lApp(argc, argv);
    Q_INIT_RESOURCE(explorer);

	try {
		// Build a system with Beagle and Explorer SN packages
		Beagle::System::Handle lSystem = new Beagle::System;
        lSystem->addPackage(new Beagle::SN::PackageBase);
        lSystem->addPackage(new Explorer::SN::PackageBase);

        // Add optional history component
		//lSystem->addComponent(new Beagle::History);

        // Add evaluation operator allocator
        lSystem->setEvaluationOp("SN-GuiEvaluationOp", new Beagle::SN::GuiEvaluationOp::Alloc);

        // Setup a Qt main window.
		Explorer::MainWindow* lMainWindow = new Explorer::MainWindow(lSystem);
		lMainWindow->show();

	} catch (Beagle::Exception& inException) {
		inException.terminate(std::cerr);
	} catch (std::exception& inException) {
		std::cerr << "Standard exception caught:" << std::endl << std::flush;
		std::cerr << inException.what() << std::endl << std::flush;
		abort();
	} catch (...) {
        std::cerr << "Unknown exception caught!" << std::endl << std::flush;
		abort();
	}

	return lApp.exec();
}

