
/*!
 *  \file   main.cpp
 *  \brief  Main routine for the sorting network problem.
 *  \author Vinod Valsalam
 *  $Revision: $
 *  $Date: $
 */

#include "beagle/SN.hpp"

#define _USE_MATH_DEFINES
#include <cmath>
#include <cstdlib>
#include <iostream>
#include <stdexcept>
#include <vector>
#include <numeric>

using namespace std;
using namespace Beagle;


int main(int argc, char *argv[])
{
	try {
		// Build a system with SN package
		System::Handle lSystem = new System;
        lSystem->addPackage(new SN::PackageBase);

        // Add optional history component
        //lSystem->addComponent(new History);

        // Add evaluation operator allocator
        lSystem->setEvaluationOp("SN-EvaluationOp", new SN::EvaluationOp::Alloc);

        // Initialize the evolver
        Evolver::Handle lEvolver = new Evolver;
        lEvolver->initialize(lSystem, argc, argv);

        // Create population
        Vivarium::Handle lVivarium = new Vivarium;

        // Launch evolution
        lEvolver->evolve(lVivarium, lSystem);

	} catch(Exception& inException) {
		inException.terminate();
	} catch(exception& inException) {
		cerr << "Standard exception caught:" << endl;
		cerr << inException.what() << endl << flush;
		return 1;
	} catch(...) {
		cerr << "Unknown exception caught!" << endl << flush;
		return 1;
	}
	return 0;
}

