
#include "explorer/GP.hpp"


int main(int argc, char *argv[])
{
	Explorer::Application lApp(argc, argv);

	try {
        // Build primitives
		Beagle::GP::PrimitiveSet::Handle lSet = new Beagle::GP::PrimitiveSet;
		lSet->insert(new Beagle::GP::Add);
		lSet->insert(new Beagle::GP::Subtract);
		lSet->insert(new Beagle::GP::Multiply);
		lSet->insert(new Beagle::GP::Divide);
		lSet->insert(new Beagle::GP::Sin);
		lSet->insert(new Beagle::GP::Cos);
		lSet->insert(new Beagle::GP::Exp);
		lSet->insert(new Beagle::GP::Log);
		lSet->insert(new Beagle::GP::TokenT<Beagle::Double>("X"));
		lSet->insert(new Beagle::GP::EphemeralDouble);

        // Build a system with Beagle and Explorer GP packages
        Beagle::System::Handle lSystem = new Beagle::System;
        lSystem->addPackage(new Beagle::GP::PackageBase(lSet));
        lSystem->addPackage(new Explorer::GP::PackageBase);

        // Add optional history component
		lSystem->addComponent(new Beagle::History);

        // Setup a Qt main window.
		Explorer::MainWindow* lMainWindow = new Explorer::MainWindow(lSystem);
		lMainWindow->show();

	} catch (Beagle::Exception& inException) {
		inException.terminate(std::cerr);
	} catch (std::exception& inException) {
		std::cerr << "Standard exception caught:" << std::endl << std::flush;
		std::cerr << inException.what() << std::endl << std::flush;
		abort();
	} catch (...) {
        std::cerr << "Unknown exception caught!" << std::endl << std::flush;
		abort();
	}

	return lApp.exec();
}

