
#ifndef Explorer_ENSO_InteractionData_hpp
#define Explorer_ENSO_InteractionData_hpp

#include <QObject>
#include <QCloseEvent>
#include <QWidget>
#include <QLabel>
#include <QSpinBox>
#include <QCheckBox>
#include <QVector>

#include <beagle/ENSO.hpp>

#include "explorer/ENSO/TreeWidget.hpp"
#include "explorer/ENSO/GraphWidget.hpp"


namespace Explorer
{
namespace ENSO
{

/*!
 *  \brief This class is a singleton and contains the data for the ENSO module
 *  that enables evolution to interact with the GUI.
 */
class InteractionData : public QObject
{

	Q_OBJECT

public:

    static InteractionData& instance();

	void setup(Beagle::System::Handle ioSystem);
	void setupStatusIndicators(QWidget* inParent, QVector< QVector<QWidget*> >& outStatusIndicators);
	void setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators);

	void update(const Beagle::ENSO::Individual& inIndividual, const Beagle::Context& inContext);

	void closeEvent(QCloseEvent *event);

private:

	InteractionData();
    InteractionData(const InteractionData&)
    { }
    InteractionData& operator=(const InteractionData&)
    { }
 	~InteractionData()
    { }

    QVector<QCheckBox*> mWidgetCheckBoxes;
    TreeWidget*         mTreeWidget;         //!< Contains graphical display of the genotype tree.
    GraphWidget*        mGraphWidget;        //!< Contains graphical display of the phenotype graph.

    Beagle::ENSO::Individual::Handle mIndividual;   //!< The current individual being evaluated.

};

}
}

#endif // Explorer_ENSO_InteractionData_hpp

