
#ifndef Explorer_ENSO_Node_hpp
#define Explorer_ENSO_Node_hpp

#include <beagle/Beagle.hpp>

#include "explorer/DisplayObject.hpp"

namespace Explorer
{
namespace ENSO
{

class Node : public DisplayObject
{

public:

	//! ENSO::Node allocator type.
	typedef Beagle::AllocatorT<ENSO::Node,DisplayObject::Alloc> Alloc;
	//! ENSO::Node handle type.
	typedef Beagle::PointerT<ENSO::Node,DisplayObject::Handle> Handle;
	//! ENSO::Node bag type.
	typedef Beagle::ContainerT<ENSO::Node,DisplayObject::Bag> Bag;

	Node(const Beagle::ENSO::Node& inNode, unsigned int inIndex, const Beagle::ENSO::Tree& inTree);
	virtual ~Node()
	{ }

	virtual void        reset();

	virtual int         getDisplaySize() const;
	virtual std::string getDisplayName(int inIndex) const;
	virtual float       getDisplayValue(int inIndex) const;
	virtual float       getMinDisplayValue(int inIndex) const;
	virtual float       getMaxDisplayValue(int inIndex) const;

	virtual std::string getToolTip() const;

    const Beagle::ENSO::Node& getBeagleNode() const;

private:

	static std::vector<std::string> smDisplayNames;      //!< Names of the display values.
	static std::vector<float>       smMinDisplayValues;  //!< The minimum of the display values.
	static std::vector<float>       smMaxDisplayValues;  //!< The maximum of the display values.
	std::vector<float>              mDisplayValues;      //!< Values to be displayed.
    std::string                     mToolTip;            //!< Tool tip of the node.

    static std::vector<unsigned int>    smLeaves;        //!< Indices of leaf nodes of genotype tree.
    const Beagle::ENSO::Node&           mNode;           //!< Reference to the Beagle::ENSO Node.

};

}
}


inline int Explorer::ENSO::Node::getDisplaySize() const
{
	Beagle_StackTraceBeginM();

	return Explorer::DisplayObject::getDisplaySize() + smDisplayNames.size();

	Beagle_StackTraceEndM("int ENSO::Node::getDisplaySize() const");
}


inline std::string Explorer::ENSO::Node::getDisplayName(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getDisplayName(inIndex);
	else
		return smDisplayNames[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("std::string ENSO::Node::getDisplayName(int inIndex) const");
}


inline float Explorer::ENSO::Node::getDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getDisplayValue(inIndex);
	else
		return mDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float ENSO::Node::getDisplayValue(int inIndex) const");
}


inline float Explorer::ENSO::Node::getMinDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getMinDisplayValue(inIndex);
	else
		return smMinDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float ENSO::Node::getMinDisplayValue(int inIndex) const");
}


inline float Explorer::ENSO::Node::getMaxDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getMaxDisplayValue(inIndex);
	else
		return smMaxDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float ENSO::Node::getMaxDisplayValue(int inIndex) const");
}


inline std::string Explorer::ENSO::Node::getToolTip() const
{
	Beagle_StackTraceBeginM();

	return mToolTip;

	Beagle_StackTraceEndM("std::string ENSO::Node::getToolTip() const");
}


inline const Beagle::ENSO::Node& Explorer::ENSO::Node::getBeagleNode() const
{
	Beagle_StackTraceBeginM();

	return mNode;

	Beagle_StackTraceEndM("const Beagle::ENSO::Node& ENSO::Node::getBeagleNode() const");
}


#endif // Explorer_ENSO_Node_hpp
