
#ifndef Explorer_ENSO_Vertex_hpp
#define Explorer_ENSO_Vertex_hpp

#include <beagle/Beagle.hpp>

#include "explorer/DisplayObject.hpp"

namespace Explorer
{
namespace ENSO
{

class Vertex : public DisplayObject
{

public:

	//! ENSO::Vertex allocator type.
	typedef Beagle::AllocatorT<ENSO::Vertex,DisplayObject::Alloc> Alloc;
	//! ENSO::Vertex handle type.
	typedef Beagle::PointerT<ENSO::Vertex,DisplayObject::Handle> Handle;
	//! ENSO::Vertex bag type.
	typedef Beagle::ContainerT<ENSO::Vertex,DisplayObject::Bag> Bag;

	Vertex(const Beagle::ENSO::Node& inLeafNode, const Beagle::ENSO::Parameters::Handle inParameters, const Beagle::ENSO::Tree& inTree);
	virtual ~Vertex()
	{ }

	virtual void        reset();

	virtual int         getDisplaySize() const;
	virtual std::string getDisplayName(int inIndex) const;
	virtual float       getDisplayValue(int inIndex) const;
	virtual float       getMinDisplayValue(int inIndex) const;
	virtual float       getMaxDisplayValue(int inIndex) const;

	virtual std::string getToolTip() const;

    const Beagle::ENSO::Node&       getLeafNode() const;
    const Beagle::ENSO::Parameters::Handle getParameters() const;

private:

	static std::vector<std::string> smDisplayNames;      //!< Names of the display values.
	static std::vector<float>       smMinDisplayValues;  //!< The minimum of the display values.
	static std::vector<float>       smMaxDisplayValues;  //!< The maximum of the display values.
	std::vector<float>              mDisplayValues;      //!< Values to be displayed.
    std::string                     mToolTip;            //!< Tool tip of the node.

    static std::vector<unsigned int>    smLeaves;        //!< Indices of leaf nodes of genotype tree.
    const Beagle::ENSO::Node&           mLeafNode;       //!< Reference to the Beagle::ENSO leaf Node.
    const Beagle::ENSO::Parameters::Handle     mParameters;     //!< Handle to the Beagle::ENSO Parameters.

};

}
}


inline int Explorer::ENSO::Vertex::getDisplaySize() const
{
	Beagle_StackTraceBeginM();

	return Explorer::DisplayObject::getDisplaySize() + smDisplayNames.size();

	Beagle_StackTraceEndM("int ENSO::Vertex::getDisplaySize() const");
}


inline std::string Explorer::ENSO::Vertex::getDisplayName(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getDisplayName(inIndex);
	else
		return smDisplayNames[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("std::string ENSO::Vertex::getDisplayName(int inIndex) const");
}


inline float Explorer::ENSO::Vertex::getDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getDisplayValue(inIndex);
	else
		return mDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float ENSO::Vertex::getDisplayValue(int inIndex) const");
}


inline float Explorer::ENSO::Vertex::getMinDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getMinDisplayValue(inIndex);
	else
		return smMinDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float ENSO::Vertex::getMinDisplayValue(int inIndex) const");
}


inline float Explorer::ENSO::Vertex::getMaxDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getMaxDisplayValue(inIndex);
	else
		return smMaxDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float ENSO::Vertex::getMaxDisplayValue(int inIndex) const");
}


inline std::string Explorer::ENSO::Vertex::getToolTip() const
{
	Beagle_StackTraceBeginM();

	return mToolTip;

	Beagle_StackTraceEndM("std::string ENSO::Vertex::getToolTip() const");
}


inline const Beagle::ENSO::Node& Explorer::ENSO::Vertex::getLeafNode() const
{
	Beagle_StackTraceBeginM();

	return mLeafNode;

	Beagle_StackTraceEndM("const Beagle::ENSO::Node& ENSO::Vertex::getLeafNode() const");
}


inline const Beagle::ENSO::Parameters::Handle Explorer::ENSO::Vertex::getParameters() const
{
	Beagle_StackTraceBeginM();

	return mParameters;

	Beagle_StackTraceEndM("const Beagle::ENSO::Parameters::Handle ENSO::Vertex::getParameters() const");
}


#endif // Explorer_ENSO_Vertex_hpp
