
#include "beagle/GA.hpp"
#include "explorer/ENSO.hpp"

using namespace Explorer;


ENSO::GraphVertex::GraphVertex(Vertex::Handle inVertex, unsigned int inVertexId) :
		mVertex(inVertex),
        mVertexId(inVertexId)
{ }


void ENSO::GraphVertex::paint(QPainter *ioPainter, const QStyleOptionGraphicsItem *inOption, QWidget *inWidget)
{
	Beagle_StackTraceBeginM();

	Q_UNUSED(inWidget);

	DisplayScene* lScene = qobject_cast<DisplayScene*>(scene());
	if (lScene == 0)
		return;
	qreal lBoundingSize = lScene->getNodeBoundingSize();

	static const qreal lOuterPenW = 2;

	QColor lOuterColor = getOuterColor();
	ioPainter->setPen(QPen(Qt::black, lOuterPenW));
	ioPainter->setBrush(lOuterColor);

	static const qreal lOuterRectW = lBoundingSize - lOuterPenW;
	static const qreal lOuterRectX = lOuterPenW/2;
	static const QRectF lOuterRect(lOuterRectX, lOuterRectX, lOuterRectW, lOuterRectW);
	ioPainter->drawEllipse(lOuterRect);
    ioPainter->drawText(lOuterRect, Qt::AlignCenter, QString::number(mVertexId));

    // Set tool tip of the node.
    QString lToolTip;
    if (lOuterColor != QColor(Qt::magenta)) {
        const DisplayObject::Handle lDisplayObject = getDisplayObject();

        const Beagle::ENSO::Parameters::Handle lParameters = Beagle::castHandleT<Vertex>(lDisplayObject)->getParameters();
        unsigned int lTreeIndex = (*Beagle::castHandleT<Beagle::GA::IntegerVector>(lParameters->find("index")->second))[0];
        lToolTip += "Tree Index = " + QString::number(lTreeIndex);
        lToolTip += "\n";

        const Beagle::ENSO::Node& lLeafNode = mVertex->getLeafNode();
        std::ostringstream lOSS;
        for(unsigned int i=0; i<lLeafNode.mColor.size(); i++) {
            lOSS << (lLeafNode.mColor[i] ? '1' : '0');
        }
        lToolTip += "Color = " + QString::fromStdString(lOSS.str());
    }
    setToolTip(lToolTip);

	Beagle_StackTraceEndM("void ENSO::GraphVertex::paint(QPainter *ioPainter, const QStyleOptionGraphicsItem *inOption, QWidget *inWidget)");
}
