
#include "explorer/ENSO.hpp"

using namespace Explorer;


std::vector<std::string> ENSO::Individual::smDisplayNames;
std::vector<float>       ENSO::Individual::smMinDisplayValues;
std::vector<float>       ENSO::Individual::smMaxDisplayValues;


ENSO::Individual::Individual() :
		mDisplayValues(0)
{}


/*!
 *  \brief Reset information.
 */
void ENSO::Individual::reset()
{
	Beagle_StackTraceBeginM();

	Explorer::Individual::reset();

	smDisplayNames.clear();
	smMinDisplayValues.clear();
	smMaxDisplayValues.clear();
	mDisplayValues.clear();

	Beagle_StackTraceEndM("void ENSO::Individual::reset()");
}


/*!
 *  \brief Setup the member variables with data from the provided Beagle Individual.
 */
void ENSO::Individual::setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();

	Explorer::Individual::setup(inIndividual, ioSystem);

	if (inIndividual != NULL) {
		Beagle::ENSO::Individual::Handle lIndividual = Beagle::castHandleT<Beagle::ENSO::Individual>(inIndividual);

		// Initialize static vector values.
		if (smDisplayNames.size() == 0) {
			smDisplayNames.push_back("Lattice Index");
			smMinDisplayValues.push_back(FLT_MAX);
			smMaxDisplayValues.push_back(-FLT_MAX);
		}

		float lLatticeIndex = (*lIndividual)[0]->getLatticeIndex();
		mDisplayValues.push_back(lLatticeIndex);

		// Keep track of the min and max display values.
		if (lLatticeIndex < smMinDisplayValues[0])
			smMinDisplayValues[0] = lLatticeIndex;
		if (lLatticeIndex > smMaxDisplayValues[0])
			smMaxDisplayValues[0] = lLatticeIndex;
	}

	Beagle_StackTraceEndM("void ENSO::Individual::setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem)");
}
