
#include "explorer/ENSO.hpp"

using namespace Explorer;


ENSO::InteractionData&  ENSO::NNInteraction::mENSOInteractionData = ENSO::InteractionData::instance();


/*!
 *  \brief Construct an interaction object.
 */
ENSO::NNInteraction::NNInteraction() :
        Explorer::NN::Interaction()
{ }


/*!
 *  \brief Setup the interaction object.
 */
void ENSO::NNInteraction::setup(Beagle::System::Handle ioSystem)
{
	Beagle_StackTraceBeginM();
    mInteractionData.setup(ioSystem);
    mENSOInteractionData.setup(ioSystem);
    mNNInteractionData.setup(ioSystem);
	Beagle_StackTraceEndM("void ENSO::NNInteraction::setup(Beagle::System::Handle ioSystem)");
}


/*!
 *  \brief Create and setup the evolution status indicators.
 *  \param inParent The parent widget of the new status indicator widgets.
 *  \param outStatusIndicators The new status indicator widgets are returned in this array.
 */
void ENSO::NNInteraction::setupStatusIndicators(QWidget* inParent, QVector< QVector<QWidget*> >& outStatusIndicators)
{
	Beagle_StackTraceBeginM();
    mInteractionData.setupStatusIndicators(inParent, outStatusIndicators);
    mENSOInteractionData.setupStatusIndicators(inParent, outStatusIndicators);
    mNNInteractionData.setupStatusIndicators(inParent, outStatusIndicators);
	Beagle_StackTraceEndM("void ENSO::NNInteraction::setupStatusIndicators(QWidget* inParent, QVector< QVector<QWidget*> >& outStatusIndicators)");
}


/*!
 *  \brief Create and setup the display window indicators.
 *  \param inParent The parent widget of the new display indicator widgets.
 *  \param outDisplayIndicators The new display indicator widgets are returned in this array.
 */
void ENSO::NNInteraction::setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators)
{	Beagle_StackTraceBeginM();
    mInteractionData.setupDisplayIndicators(inParent, outDisplayIndicators);
    mENSOInteractionData.setupDisplayIndicators(inParent, outDisplayIndicators);
    mNNInteractionData.setupDisplayIndicators(inParent, outDisplayIndicators);
	Beagle_StackTraceEndM("void ENSO::NNInteraction::setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators)");
}


/*!
 *  \brief Controls GUI updates and entering Qt's event loop.
 *  \param inIndividual The graph of the individual being evaluated.
 *  \param inContext The context of evolution.
 */
void ENSO::NNInteraction::update(const Beagle::ENSO::Individual& inIndividual, const Beagle::Context& inContext)
{
	Beagle_StackTraceBeginM();
    mENSOInteractionData.update(inIndividual, inContext);
	Beagle_StackTraceEndM("void ENSO::NNInteraction::update(const Beagle::ENSO::Individual& inIndividual, const Beagle::Context& inContext)");
}


/*!
 *  \brief Process the close event.
 */
void ENSO::NNInteraction::closeEvent(QCloseEvent *event)
{
	Beagle_StackTraceBeginM();
    mInteractionData.closeEvent(event);
    mENSOInteractionData.closeEvent(event);
    mNNInteractionData.closeEvent(event);
	Beagle_StackTraceEndM("void ENSO::NNInteraction::closeEvent(QCloseEvent *event)");
}
