
#include "explorer/ENSO.hpp"

using namespace Explorer;


/*!
 *  \brief Construct package with basic Explorer ENSO objects.
 */
ENSO::PackageBase::PackageBase() :
		Beagle::Package("Explorer-ENSO-PackageBase")
{ }


/*!
 *  \brief Configure system by adding necessary components into it.
 *  \param ioSystem System to configure.
 */
void ENSO::PackageBase::configure(Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();
	Beagle::Factory& lFactory = ioSystem.getFactory();

	// Add available basic types to the factory
	lFactory.insertAllocator("Explorer::ENSO::Individual", new Explorer::ENSO::Individual::Alloc);
	lFactory.insertAllocator("Explorer::ENSO::NNInteraction", new Explorer::ENSO::NNInteraction::Alloc);

	// Set aliases
	lFactory.aliasAllocator("Explorer::ENSO::Individual", "Explorer-ENSO-Individual");
	lFactory.aliasAllocator("Explorer::ENSO::NNInteraction", "ENSO-NNInteraction");

	// Set some basic concept-type associations in factory
	lFactory.setConcept("Explorer-Fitness", "Explorer-FitnessSimple");
	lFactory.setConcept("Explorer-Individual", "Explorer-ENSO-Individual");
	lFactory.setConcept("Interaction", "ENSO-NNInteraction");

	Beagle_StackTraceEndM("void ENSO::PackageBase::configure(System&)");
}


/*!
 *  \brief List packages on which current package depends.
 *  \return Bag with the packages instanciation on which current package depends.
 *
 *  Explorer::PackageBase depends on Beagle::PackageBase.
 */
Beagle::Package::Bag ENSO::PackageBase::listDependencies(void)
{
	Beagle_StackTraceBeginM();
	Package::Bag lDependencies;
	lDependencies.push_back(new Explorer::PackageBase());
	return lDependencies;
	Beagle_StackTraceEndM("Beagle::Package::Bag ENSO::PackageBase::listDependencies(void)");
}
