
#include "explorer/ENSO.hpp"

using namespace Explorer;


ENSO::TreeNode::TreeNode(Node::Handle inNode, unsigned int inIndex) :
		mNode(inNode),
        mIndex(inIndex)
{ }


void ENSO::TreeNode::paint(QPainter *ioPainter, const QStyleOptionGraphicsItem *inOption, QWidget *inWidget)
{
	Beagle_StackTraceBeginM();

	Q_UNUSED(inWidget);

	DisplayScene* lScene = qobject_cast<DisplayScene*>(scene());
	if (lScene == 0)
		return;
	qreal lBoundingSize = lScene->getNodeBoundingSize();

	static const qreal lOuterPenW = 2;

	QColor lOuterColor = getOuterColor();
	ioPainter->setPen(QPen(Qt::black, lOuterPenW));
	ioPainter->setBrush(lOuterColor);

	static const qreal lOuterRectW = lBoundingSize - lOuterPenW;
	static const qreal lOuterRectX = lOuterPenW/2;
	static const QRectF lOuterRect(lOuterRectX, lOuterRectX, lOuterRectW, lOuterRectW);
	ioPainter->drawRect(lOuterRect);
    ioPainter->drawText(lOuterRect, Qt::AlignCenter, QString::number(mIndex));

    // Set tool tip of the node.
    QString lToolTip;
    if (lOuterColor != QColor(Qt::magenta)) {
        const DisplayObject::Handle lDisplayObject = getDisplayObject();
        int lDisplaySize = lDisplayObject->getDisplaySize();
        for (int i = 1; i < lDisplaySize; ++i) {
            lToolTip += QString::fromStdString(lDisplayObject->getDisplayName(i)) + " = " + QString::number(lDisplayObject->getDisplayValue(i));
            lToolTip += "\n";
        }

        const Beagle::ENSO::Node& lBeagleNode = mNode->getBeagleNode();
        std::ostringstream lOSS;
        for(unsigned int i=0; i<lBeagleNode.mColor.size(); i++) {
            lOSS << (lBeagleNode.mColor[i] ? '1' : '0');
        }
        lToolTip += "Color = " + QString::fromStdString(lOSS.str());
    }
    setToolTip(lToolTip);

	Beagle_StackTraceEndM("void ENSO::TreeNode::paint(QPainter *ioPainter, const QStyleOptionGraphicsItem *inOption, QWidget *inWidget)");
}
