
#include "beagle/GA.hpp"
#include "explorer/ENSO.hpp"

using namespace Explorer;


std::vector<std::string>    ENSO::Vertex::smDisplayNames;
std::vector<float>          ENSO::Vertex::smMinDisplayValues;
std::vector<float>          ENSO::Vertex::smMaxDisplayValues;
std::vector<unsigned int>   ENSO::Vertex::smLeaves;


ENSO::Vertex::Vertex(const Beagle::ENSO::Node& inLeafNode, const Beagle::ENSO::Parameters::Handle inParameters, const Beagle::ENSO::Tree& inTree) :
		mDisplayValues(0),
        mLeafNode(inLeafNode),
        mParameters(inParameters)
{
	Beagle_StackTraceBeginM();

    // Initialize static vector values.
    if (smDisplayNames.size() == 0) {
        smDisplayNames.push_back("Leaf Index");

        // Min and Max of leaf index.
        inTree.getLeaves(smLeaves);
        smMinDisplayValues.push_back(0.0);
        smMaxDisplayValues.push_back((float)(smLeaves.size()-1));
    }

    unsigned int lTreeIndex = (*Beagle::castHandleT<Beagle::GA::IntegerVector>(mParameters->find("index")->second))[0];

    // The index of node in the smLeaves vector; undefined if not a leaf.
    unsigned int lLeafIndex = UINT_MAX;
    if (inTree[lTreeIndex].mChildren.size() == 0) {
        for (unsigned int i = 0; i < smLeaves.size() && lLeafIndex == UINT_MAX; i++) {
            if (smLeaves[i] == lTreeIndex) lLeafIndex = i;
        }
    }

    mDisplayValues.push_back(lLeafIndex == UINT_MAX ? FLT_MAX : (float)lLeafIndex);

	Beagle_StackTraceEndM("ENSO::Vertex::Vertex(const Beagle::ENSO::Node& inLeafNode, const Beagle::ENSO::Parameters::Handle inParameters, const Beagle::ENSO::Tree& inTree)");
}


/*!
 *  \brief Reset information.
 */
void ENSO::Vertex::reset()
{
	Beagle_StackTraceBeginM();

	smDisplayNames.clear();
	smMinDisplayValues.clear();
	smMaxDisplayValues.clear();
	mDisplayValues.clear();

	Beagle_StackTraceEndM("void ENSO::Vertex::reset()");
}
