
#include "explorer/GP.hpp"

using namespace Explorer;


std::vector<std::string> GP::Individual::smDisplayNames;
std::vector<float>       GP::Individual::smMinDisplayValues;
std::vector<float>       GP::Individual::smMaxDisplayValues;


GP::Individual::Individual() :
		mDisplayValues(0)
{}


/*!
 *  \brief Reset information.
 */
void GP::Individual::reset()
{
	Beagle_StackTraceBeginM();

	Explorer::Individual::reset();

	smDisplayNames.clear();
	smMinDisplayValues.clear();
	smMaxDisplayValues.clear();
	mDisplayValues.clear();

	Beagle_StackTraceEndM("void GP::Individual::reset()");
}


/*!
 *  \brief Setup the member variables with data from the provided Beagle Individual.
 */
void GP::Individual::setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();

	Explorer::Individual::setup(inIndividual, ioSystem);

	if (inIndividual != NULL) {
		Beagle::GP::Individual::Handle lIndividual = Beagle::castHandleT<Beagle::GP::Individual>(inIndividual);

		// Initialize static vector values.
		if (smDisplayNames.size() == 0) {
			smDisplayNames.push_back("Tree Depth");
			smDisplayNames.push_back("Tree Size");
			smMinDisplayValues.push_back(FLT_MAX);
			smMinDisplayValues.push_back(FLT_MAX);
			smMaxDisplayValues.push_back(-FLT_MAX);
			smMaxDisplayValues.push_back(-FLT_MAX);
		}

		float lTreeDepth = lIndividual->getMaxTreeDepth();
		mDisplayValues.push_back(lTreeDepth);

		float lTreeSize = lIndividual->getTotalNodes();
		mDisplayValues.push_back(lTreeSize);

		// Keep track of the min and max display values.
		if (lTreeDepth < smMinDisplayValues[0])
			smMinDisplayValues[0] = lTreeDepth;
		if (lTreeDepth > smMaxDisplayValues[0])
			smMaxDisplayValues[0] = lTreeDepth;

		if (lTreeSize < smMinDisplayValues[1])
			smMinDisplayValues[1] = lTreeSize;
		if (lTreeSize > smMaxDisplayValues[1])
			smMaxDisplayValues[1] = lTreeSize;
	}

	Beagle_StackTraceEndM("void GP::Individual::setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem)");
}
