
#include "explorer/GP.hpp"

using namespace Explorer;


/*!
 *  \brief Construct package with basic Explorer GP objects.
 */
GP::PackageBase::PackageBase() :
		Beagle::Package("Explorer-GP-PackageBase")
{ }


/*!
 *  \brief Configure system by adding necessary components into it.
 *  \param ioSystem System to configure.
 */
void GP::PackageBase::configure(Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();
    Beagle::Factory& lFactory = ioSystem.getFactory();

	// Add available basic types to the factory
	lFactory.insertAllocator("Explorer::GP::Individual", new Explorer::GP::Individual::Alloc);

	// Set aliases
	lFactory.aliasAllocator("Explorer::GP::Individual", "Explorer-GP-Individual");

	// Set some basic concept-type associations in factory
	lFactory.setConcept("Explorer-Individual", "Explorer-GP-Individual");

	Beagle_StackTraceEndM("void GP::PackageBase::configure(System&)");
}


/*!
 *  \brief List packages on which current package depends.
 *  \return Bag with the packages instanciation on which current package depends.
 *
 *  Explorer::PackageBase depends on Beagle::PackageBase.
 */
Beagle::Package::Bag GP::PackageBase::listDependencies(void)
{
	Beagle_StackTraceBeginM();
	Package::Bag lDependencies;
	lDependencies.push_back(new Explorer::PackageBase());
	return lDependencies;
	Beagle_StackTraceEndM("Beagle::Package::Bag GP::PackageBase::listDependencies(void)");
}
