
#ifndef Explorer_NN_FitnessIndexed_hpp
#define Explorer_NN_FitnessIndexed_hpp

#include <beagle/NN.hpp>

#include "explorer/Fitness.hpp"

namespace Explorer
{
namespace NN
{

class FitnessIndexed : public Fitness
{

public:

	//! FitnessIndexed allocator type.
	typedef Beagle::AllocatorT<FitnessIndexed,Fitness::Alloc>
	Alloc;
	//! FitnessIndexed handle type.
	typedef Beagle::PointerT<FitnessIndexed,Fitness::Handle>
	Handle;
	//! FitnessIndexed bag type.
	typedef Beagle::ContainerT<FitnessIndexed,Fitness::Bag>
	Bag;


	FitnessIndexed();
	virtual ~FitnessIndexed()
	{ }

	virtual void        reset();
	virtual void        setup(Beagle::Fitness::Handle inFitness, Beagle::System& ioSystem);

	virtual int         getDisplaySize() const;
	virtual std::string getDisplayName(int inIndex) const;
	virtual float       getDisplayValue(int inIndex) const;
	virtual float       getMinDisplayValue(int inIndex) const;
	virtual float       getMaxDisplayValue(int inIndex) const;

private:

	static std::vector<std::string> smDisplayNames;      //!< Names of the display values.
	static std::vector<float>       smMinDisplayValues;  //!< The minimum of the display values.
	static std::vector<float>       smMaxDisplayValues;  //!< The maximum of the display values.
	std::vector<float>              mDisplayValues;      //!< Values to be displayed.

};

}
}


inline int Explorer::NN::FitnessIndexed::getDisplaySize() const
{
	Beagle_StackTraceBeginM();

	return Fitness::getDisplaySize() + smDisplayNames.size();

	Beagle_StackTraceEndM("int NN::FitnessIndexed::getDisplaySize() const");
}


inline std::string Explorer::NN::FitnessIndexed::getDisplayName(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Fitness::getDisplaySize())
		return Fitness::getDisplayName(inIndex);
	else
		return smDisplayNames[inIndex - Fitness::getDisplaySize()];

	Beagle_StackTraceEndM("std::string NN::FitnessIndexed::getDisplayName(int inIndex) const");
}


inline float Explorer::NN::FitnessIndexed::getDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Fitness::getDisplaySize())
		return Fitness::getDisplayValue(inIndex);
	else
		return mDisplayValues[inIndex - Fitness::getDisplaySize()];

	Beagle_StackTraceEndM("float NN::FitnessIndexed::getDisplayValue(int inIndex) const");
}


inline float Explorer::NN::FitnessIndexed::getMinDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Fitness::getDisplaySize())
		return Fitness::getMinDisplayValue(inIndex);
	else
		return smMinDisplayValues[inIndex - Fitness::getDisplaySize()];

	Beagle_StackTraceEndM("float NN::FitnessIndexed::getMinDisplayValue(int inIndex) const");
}


inline float Explorer::NN::FitnessIndexed::getMaxDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Fitness::getDisplaySize())
		return Fitness::getMaxDisplayValue(inIndex);
	else
		return smMaxDisplayValues[inIndex - Fitness::getDisplaySize()];

	Beagle_StackTraceEndM("float NN::FitnessIndexed::getMaxDisplayValue(int inIndex) const");
}


#endif // Explorer_NN_FitnessIndexed_hpp
