
#ifndef Explorer_NN_Individual_hpp
#define Explorer_NN_Individual_hpp

#ifdef BEAGLE_HAVE_EXT_HASH_MAP
#include <ext/hash_map>
#else // BEAGLE_HAVE_EXT_HASH_MAP
#include <map>
#endif // BEAGLE_HAVE_EXT_HASH_MAP

#include <beagle/NN.hpp>

#include "explorer/Individual.hpp"
#include "explorer/Fitness.hpp"

namespace Explorer
{
namespace NN
{

class Individual : public Explorer::Individual
{

public:

	//! NN::Individual allocator type.
	typedef Beagle::AllocatorT<NN::Individual,Explorer::Individual::Alloc> Alloc;
	//! NN::Individual handle type.
	typedef Beagle::PointerT<NN::Individual,Explorer::Individual::Handle> Handle;
	//! NN::Individual bag type.
	typedef Beagle::ContainerT<NN::Individual,Explorer::Individual::Bag> Bag;

	Individual();
	virtual ~Individual()
	{ }

	virtual void        reset();
	virtual void        setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem);

	virtual int         getDisplaySize() const;
	virtual std::string getDisplayName(int inIndex) const;
	virtual float       getDisplayValue(int inIndex) const;
	virtual float       getMinDisplayValue(int inIndex) const;
	virtual float       getMaxDisplayValue(int inIndex) const;

private:

	//! Map to associate modules with sequential numerical values for display.
#ifdef BEAGLE_HAVE_EXT_HASH_MAP
	typedef BEAGLE_EXT_HASH_MAP_NAMESPACE::hash_map<unsigned int, unsigned int> ModuleMap;
#else // BEAGLE_HAVE_EXT_HASH_MAP
	typedef std::map<unsigned int, unsigned int> ModuleMap;
#endif // BEAGLE_HAVE_EXT_HASH_MAP

	static ModuleMap    smModuleMap;         //!< Module map.
	static std::string  smDisplayName;       //!< Name of the display value.
	static float        smMinDisplayValue;   //!< The minimum of the display value.
	static float        smMaxDisplayValue;   //!< The maximum of the display value.
	float               mDisplayValue;       //!< Value to be displayed.

};

}
}


inline int Explorer::NN::Individual::getDisplaySize() const
{
	Beagle_StackTraceBeginM();

	return Explorer::Individual::getDisplaySize() + 1;

	Beagle_StackTraceEndM("int NN::Individual::getDisplaySize() const");
}


inline std::string Explorer::NN::Individual::getDisplayName(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::Individual::getDisplaySize())
		return Explorer::Individual::getDisplayName(inIndex);
	else
		return smDisplayName;

	Beagle_StackTraceEndM("std::string NN::Individual::getDisplayName(int inIndex) const");
}


inline float Explorer::NN::Individual::getDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::Individual::getDisplaySize())
		return Explorer::Individual::getDisplayValue(inIndex);
	else
		return mDisplayValue;

	Beagle_StackTraceEndM("float NN::Individual::getDisplayValue(int inIndex) const");
}


inline float Explorer::NN::Individual::getMinDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::Individual::getDisplaySize())
		return Explorer::Individual::getMinDisplayValue(inIndex);
	else
		return smMinDisplayValue;

	Beagle_StackTraceEndM("float NN::Individual::getMinDisplayValue(int inIndex) const");
}


inline float Explorer::NN::Individual::getMaxDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::Individual::getDisplaySize())
		return Explorer::Individual::getMaxDisplayValue(inIndex);
	else
		return smMaxDisplayValue;

	Beagle_StackTraceEndM("float NN::Individual::getMaxDisplayValue(int inIndex) const");
}


#endif // Explorer_NN_Individual_hpp
