
#ifndef Explorer_NN_InteractionData_hpp
#define Explorer_NN_InteractionData_hpp

#include <QObject>
#include <QCloseEvent>
#include <QWidget>
#include <QLabel>
#include <QSpinBox>
#include <QCheckBox>
#include <QVector>

#include <beagle/NN.hpp>

#include "explorer/NN/NetworkWidget.hpp"
#include "explorer/NN/Plot.hpp"


namespace Explorer
{
namespace NN
{

/*!
 *  \brief This class is a singleton and contains the data for the NN module
 *  that enables evolution to interact with the GUI.
 */
class InteractionData : public QObject
{

	Q_OBJECT

public:

    static InteractionData& instance();

	void setup(Beagle::System::Handle ioSystem);
	void setupStatusIndicators(QWidget* inParent, QVector< QVector<QWidget*> >& outStatusIndicators);
	void setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators);

	void update(const Beagle::NN::Graph& inGraph, const Beagle::Context& inContext);
    bool testBreak(const Beagle::NN::Graph& inGraph);

	void closeEvent(QCloseEvent *event);

private:

	InteractionData();
    InteractionData(const InteractionData&)
    { }
    InteractionData& operator=(const InteractionData&)
    { }
 	~InteractionData()
    { }

    QLabel*             mLabelActivationCount;
    QLabel*             mCurrentActivationCount;
    QSpinBox*           mBreakActivationCount;
    QVector<QCheckBox*> mWidgetCheckBoxes;

    NetworkWidget*      mNetworkWidget;         //!< Contains graphical display of the network.
    QVector<Plot*>      mPlotWidgets;           //!< Contains the data plots.

};

}
}

#endif // Explorer_NN_InteractionData_hpp

