
#ifndef Explorer_NN_Node_hpp
#define Explorer_NN_Node_hpp

#include <beagle/Beagle.hpp>

#include "explorer/DisplayObject.hpp"

namespace Explorer
{
namespace NN
{

class Node : public DisplayObject
{

public:

	//! NN::Node allocator type.
	typedef Beagle::AllocatorT<NN::Node,DisplayObject::Alloc> Alloc;
	//! NN::Node handle type.
	typedef Beagle::PointerT<NN::Node,DisplayObject::Handle> Handle;
	//! NN::Node bag type.
	typedef Beagle::ContainerT<NN::Node,DisplayObject::Bag> Bag;

	Node(const Beagle::NN::Node& inNode, const Beagle::NN::Graph& inGraph);
	virtual ~Node()
	{ }

	virtual void        reset();
	virtual void        update();

	virtual int         getDisplaySize() const;
	virtual std::string getDisplayName(int inIndex) const;
	virtual float       getDisplayValue(int inIndex) const;
	virtual float       getMinDisplayValue(int inIndex) const;
	virtual float       getMaxDisplayValue(int inIndex) const;

	virtual std::string getToolTip() const;

    const Beagle::NN::Node& getBeagleNode() const;

private:

	static std::vector<std::string> smDisplayNames;      //!< Names of the display values.
	static std::vector<float>       smMinDisplayValues;  //!< The minimum of the display values.
	static std::vector<float>       smMaxDisplayValues;  //!< The maximum of the display values.
	std::vector<float>              mDisplayValues;      //!< Values to be displayed.
    std::string                     mToolTip;            //!< Tool tip of the node.

    const Beagle::NN::Node&         mNode;               //!< Reference to the Beagle::NN Node.
    const Beagle::NN::Graph&        mGraph;              //!< Reference to the Beagle::NN Graph.
};

}
}


inline int Explorer::NN::Node::getDisplaySize() const
{
	Beagle_StackTraceBeginM();

	return Explorer::DisplayObject::getDisplaySize() + smDisplayNames.size();

	Beagle_StackTraceEndM("int NN::Node::getDisplaySize() const");
}


inline std::string Explorer::NN::Node::getDisplayName(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getDisplayName(inIndex);
	else
		return smDisplayNames[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("std::string NN::Node::getDisplayName(int inIndex) const");
}


inline float Explorer::NN::Node::getDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getDisplayValue(inIndex);
	else
		return mDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float NN::Node::getDisplayValue(int inIndex) const");
}


inline float Explorer::NN::Node::getMinDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getMinDisplayValue(inIndex);
	else
		return smMinDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float NN::Node::getMinDisplayValue(int inIndex) const");
}


inline float Explorer::NN::Node::getMaxDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getMaxDisplayValue(inIndex);
	else
		return smMaxDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float NN::Node::getMaxDisplayValue(int inIndex) const");
}


inline std::string Explorer::NN::Node::getToolTip() const
{
	Beagle_StackTraceBeginM();

	return mToolTip;

	Beagle_StackTraceEndM("std::string NN::Node::getToolTip() const");
}


inline const Beagle::NN::Node& Explorer::NN::Node::getBeagleNode() const
{
	Beagle_StackTraceBeginM();

	return mNode;

	Beagle_StackTraceEndM("const Beagle::NN::Node& NN::Node::getBeagleNode() const");
}


#endif // Explorer_NN_Node_hpp
