
#ifndef Explorer_NN_PlotData_hpp
#define Explorer_NN_PlotData_hpp

#include "beagle/NN.hpp"
#include <QVector>
#include <QString>


namespace Explorer
{
namespace NN
{

class PlotData : public Beagle::Component
{

public:

	//! NN::PlotData allocator type.
	typedef Beagle::AllocatorT<PlotData,Beagle::Component::Alloc>
	Alloc;
	//! NN::PlotData handle type.
	typedef Beagle::PointerT<PlotData,Beagle::Component::Handle>
	Handle;
	//! NN::PlotData bag type.
	typedef Beagle::ContainerT<PlotData,Beagle::Component::Bag>
	Bag;

	PlotData(void);
	virtual ~PlotData()
	{ }

    //! The data for one plot
    struct Data {
        QString                         mTitle;         //!< The title of the plot

        QString                         mLabelAxisX;   	//!< The label of the X axis
        QVector<double>                 mX;         	//!< The X coordinates
        double                          mMinX;      	//!< The min value of the X coordinate
        double                          mMaxX;      	//!< The max value of the X coordinate
                                                    	 
        QString                         mLabelAxisY;   	//!< The label of the Y axis
        QVector<QString>                mLabelCurve;    //!< The label of each curve
        QVector< QVector<double> >      mY;         	//!< The Y coordinates of each curve
        double                          mMinY;      	//!< The min value of the Y coordinate
        double                          mMaxY;      	//!< The max value of the Y coordinate                                                    	 
    };

    QVector<Data>& getDataVector();

protected:

    QVector<Data>                   mPlotData;          //!< Data for all the plots

};

}
}


/*!
 *  \return The vector of plot data stored in the component
 */
inline QVector<Explorer::NN::PlotData::Data>& Explorer::NN::PlotData::getDataVector()
{
	Beagle_StackTraceBeginM();
	return mPlotData;
	Beagle_StackTraceEndM("QVector<NN::PlotData::Data>& NN::PlotData::getDataVector()");
}


#endif // Explorer_NN_PlotData_hpp
