
#include "explorer/NN.hpp"

using namespace Explorer;


std::vector<std::string> NN::FitnessIndexed::smDisplayNames;
std::vector<float>       NN::FitnessIndexed::smMinDisplayValues;
std::vector<float>       NN::FitnessIndexed::smMaxDisplayValues;


NN::FitnessIndexed::FitnessIndexed() :
		Fitness(),
		mDisplayValues(0)
{}


/*!
 *  \brief Reset information.
 */
void NN::FitnessIndexed::reset()
{
	Beagle_StackTraceBeginM();

	Fitness::reset();

	smDisplayNames.clear();
	smMinDisplayValues.clear();
	smMaxDisplayValues.clear();
	mDisplayValues.clear();

	Beagle_StackTraceEndM("void NN::FitnessIndexed::reset()");
}


/*!
 *  \brief Setup the object with the given Beagle fitness.
 */
void NN::FitnessIndexed::setup(Beagle::Fitness::Handle inFitness, Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();

	Fitness::setup(inFitness, ioSystem);

	Beagle::NN::FitnessIndexed::Handle lFitness = Beagle::castHandleT<Beagle::NN::FitnessIndexed>(inFitness);

	for (unsigned int i = 0; i < lFitness->size(); ++i) {
		// Initialize static vector values.
		if (smDisplayNames.size() == i) {
			smDisplayNames.push_back("Fitness Indexed " + Beagle::uint2str(i));
			smMinDisplayValues.push_back(FLT_MAX);
			smMaxDisplayValues.push_back(-FLT_MAX);
		}

		if (lFitness->isValid() && lFitness->getValue(i) > -FLT_MAX) {
			float lDisplayValue = lFitness->getValue(i);
			mDisplayValues.push_back(lDisplayValue);

			// Keep track of the min and max display values.
			if (lDisplayValue < smMinDisplayValues[i])
				smMinDisplayValues[i] = lDisplayValue;
			if (lDisplayValue > smMaxDisplayValues[i])
				smMaxDisplayValues[i] = lDisplayValue;
		} else {
			mDisplayValues.push_back(-FLT_MAX);
		}
	}

	Beagle_StackTraceEndM("void NN::FitnessIndexed::setup(Beagle::Fitness::Handle inFitness, Beagle::System& ioSystem)");
}
