
#include "explorer/NN.hpp"

using namespace Explorer;


NN::InteractionData&    NN::Interaction::mNNInteractionData = NN::InteractionData::instance();


/*!
 *  \brief Construct an interaction object.
 */
NN::Interaction::Interaction() :
        Explorer::Interaction()
{ }


/*!
 *  \brief Setup the interaction object.
 */
void NN::Interaction::setup(Beagle::System::Handle ioSystem)
{
	Beagle_StackTraceBeginM();
    mInteractionData.setup(ioSystem);
    mNNInteractionData.setup(ioSystem);
	Beagle_StackTraceEndM("void NN::Interaction::setup(Beagle::System::Handle ioSystem)");
}


/*!
 *  \brief Create and setup the evolution status indicators.
 *  \param inParent The parent widget of the new status indicator widgets.
 *  \param outStatusIndicators The new status indicator widgets are returned in this array.
 */
void NN::Interaction::setupStatusIndicators(QWidget* inParent, QVector< QVector<QWidget*> >& outStatusIndicators)
{
	Beagle_StackTraceBeginM();
    mInteractionData.setupStatusIndicators(inParent, outStatusIndicators);
    mNNInteractionData.setupStatusIndicators(inParent, outStatusIndicators);
	Beagle_StackTraceEndM("void NN::Interaction::setupStatusIndicators(QWidget* inParent, QVector< QVector<QWidget*> >& outStatusIndicators)");
}


/*!
 *  \brief Create and setup the display window indicators.
 *  \param inParent The parent widget of the new display indicator widgets.
 *  \param outDisplayIndicators The new display indicator widgets are returned in this array.
 */
void NN::Interaction::setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators)
{	Beagle_StackTraceBeginM();
    mInteractionData.setupDisplayIndicators(inParent, outDisplayIndicators);
    mNNInteractionData.setupDisplayIndicators(inParent, outDisplayIndicators);
	Beagle_StackTraceEndM("void NN::Interaction::setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators)");
}


/*!
 *  \brief Controls GUI updates and entering Qt's event loop.
 *  \param inGraph The graph of the individual being evaluated.
 *  \param inContext The context of evolution.
 */
void NN::Interaction::update(const Beagle::NN::Graph& inGraph, const Beagle::Context& inContext)
{
	Beagle_StackTraceBeginM();
    if (mInteractionData.testPauseOrPastBreak(inContext) ||
        (mInteractionData.testAtBreak(inContext) && mNNInteractionData.testBreak(inGraph))) {
        mInteractionData.update(inContext);
        mNNInteractionData.update(inGraph, inContext);
        mInteractionData.enterEventLoop();
    }
    else if (mInteractionData.testStatusUpdateTime() || inGraph.getActivationCount() == 0) {
        mInteractionData.update(inContext);
        mNNInteractionData.update(inGraph, inContext);
    }
	Beagle_StackTraceEndM("void NN::Interaction::update(const Beagle::NN::Graph& inGraph, const Beagle::Context& inContext)");
}


/*!
 *  \brief Process the close event.
 */
void NN::Interaction::closeEvent(QCloseEvent *event)
{
	Beagle_StackTraceBeginM();
    mInteractionData.closeEvent(event);
    mNNInteractionData.closeEvent(event);
	Beagle_StackTraceEndM("void NN::Interaction::closeEvent(QCloseEvent *event)");
}
