
#include "explorer/NN.hpp"

using namespace Explorer;


NN::NetworkNode::NetworkNode(Node::Handle inNode) :
		mNode(inNode),
        mIdentifier(inNode->getBeagleNode().getIdentifier())
{ }


void NN::NetworkNode::paint(QPainter *ioPainter, const QStyleOptionGraphicsItem *inOption, QWidget *inWidget)
{
	Beagle_StackTraceBeginM();

	Q_UNUSED(inWidget);

	DisplayScene* lScene = qobject_cast<DisplayScene*>(scene());
	if (lScene == 0)
		return;
	qreal lBoundingSize = lScene->getNodeBoundingSize();

	static const qreal lOuterPenW = 2;

	QColor lOuterColor = getOuterColor();
	ioPainter->setPen(QPen(Qt::black, lOuterPenW));
	ioPainter->setBrush(lOuterColor);

	static const qreal lOuterRectW = lBoundingSize - lOuterPenW;
	static const qreal lOuterRectX = lOuterPenW/2;
	static const QRectF lOuterRect(lOuterRectX, lOuterRectX, lOuterRectW, lOuterRectW);
	ioPainter->drawEllipse(lOuterRect);
    ioPainter->drawText(lOuterRect, Qt::AlignCenter, QString::number(mIdentifier));

    // Set tool tip of the node.
    QString lToolTip;
    if (lOuterColor != QColor(Qt::magenta)) {
        const DisplayObject::Handle lDisplayObject = getDisplayObject();
        int lDisplaySize = lDisplayObject->getDisplaySize();
        for (int i = 1; i < lDisplaySize; ++i) {
            lToolTip += QString::fromStdString(lDisplayObject->getDisplayName(i)) + " = " + QString::number(lDisplayObject->getDisplayValue(i));
            if (i < lDisplaySize - 1) lToolTip += "\n";
        }
    }
    setToolTip(lToolTip);

	Beagle_StackTraceEndM("void NN::NetworkNode::paint(QPainter *ioPainter, const QStyleOptionGraphicsItem *inOption, QWidget *inWidget)");
}
