
#include "explorer/NN.hpp"

using namespace Explorer;


std::vector<std::string> NN::Node::smDisplayNames;
std::vector<float>       NN::Node::smMinDisplayValues;
std::vector<float>       NN::Node::smMaxDisplayValues;


NN::Node::Node(const Beagle::NN::Node& inNode, const Beagle::NN::Graph& inGraph) :
		mDisplayValues(0),
        mNode(inNode),
        mGraph(inGraph)
{
	Beagle_StackTraceBeginM();

    // Initialize static vector values.
    if (smDisplayNames.size() == 0) {
        smDisplayNames.push_back("Bias");
        smDisplayNames.push_back("Slope");
        smDisplayNames.push_back("Modules");
        smDisplayNames.push_back("Activation");

        // Initialize Min and Max display values to be updated later.
        for (unsigned int i = 0; i < smDisplayNames.size()-1; i++) {
            smMinDisplayValues.push_back(FLT_MAX);
            smMaxDisplayValues.push_back(-FLT_MAX);
        }

        // Min and Max values of activation do not change and are set here.
        smMinDisplayValues.push_back(-1.0);
        smMaxDisplayValues.push_back(1.0);
    }

    float lModule = inNode.getModuleKey();
    if (inNode.getModuleKey() == UINT_MAX) lModule = -1;

    mDisplayValues.push_back(inNode.getBias());
    mDisplayValues.push_back(inNode.getSlope());
    mDisplayValues.push_back(lModule);
    mDisplayValues.push_back(inGraph.getActivation(inNode.getIdentifier()));

    // Keep track of the min and max display values except activation.
    for (unsigned int i = 0; i < smDisplayNames.size()-1; i++) {
        if (mDisplayValues[i] < smMinDisplayValues[i])
            smMinDisplayValues[i] = mDisplayValues[i];
        if (mDisplayValues[i] > smMaxDisplayValues[i])
            smMaxDisplayValues[i] = mDisplayValues[i];
    }

	Beagle_StackTraceEndM("NN::Node::Node(const Beagle::NN::Node& inNode, const Beagle::NN::Graph& inGraph)");
}


/*!
 *  \brief Reset information.
 */
void NN::Node::reset()
{
	Beagle_StackTraceBeginM();

	smDisplayNames.clear();
	smMinDisplayValues.clear();
	smMaxDisplayValues.clear();
	mDisplayValues.clear();

	Beagle_StackTraceEndM("void NN::Node::reset()");
}


/*!
 *  \brief Update information in the display values.
 */
void NN::Node::update()
{
	Beagle_StackTraceBeginM();

    mDisplayValues[smDisplayNames.size()-1] = mGraph.getActivation(mNode.getIdentifier());

	Beagle_StackTraceEndM("void NN::Node::update()");
}
