
#include "explorer/NN.hpp"

using namespace Explorer;


/*!
 *  \brief Construct package with basic Explorer NN objects.
 */
NN::PackageBase::PackageBase() :
		Beagle::Package("Explorer-NN-PackageBase")
{ }


/*!
 *  \brief Configure system by adding necessary components into it.
 *  \param ioSystem System to configure.
 */
void NN::PackageBase::configure(Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();
	Beagle::Factory& lFactory = ioSystem.getFactory();

	// Add available basic types to the factory
	lFactory.insertAllocator("Explorer::NN::PlotData", new Explorer::NN::PlotData::Alloc);
	lFactory.insertAllocator("Explorer::NN::FitnessIndexed", new Explorer::NN::FitnessIndexed::Alloc);
	lFactory.insertAllocator("Explorer::NN::Individual", new Explorer::NN::Individual::Alloc);
	lFactory.insertAllocator("Explorer::NN::Interaction", new Explorer::NN::Interaction::Alloc);

	// Set aliases
	lFactory.aliasAllocator("Explorer::NN::PlotData", "NN-PlotData");
	lFactory.aliasAllocator("Explorer::NN::FitnessIndexed", "Explorer-NN-FitnessIndexed");
	lFactory.aliasAllocator("Explorer::NN::Individual", "Explorer-NN-Individual");
	lFactory.aliasAllocator("Explorer::NN::Interaction", "NN-Interaction");

	// Set some basic concept-type associations in factory
	lFactory.setConcept("Explorer-Fitness", "Explorer-NN-FitnessIndexed");
	lFactory.setConcept("Explorer-Individual", "Explorer-NN-Individual");
	lFactory.setConcept("Interaction", "NN-Interaction");

	Beagle_StackTraceEndM("void NN::PackageBase::configure(System&)");
}


/*!
 *  \brief List packages on which current package depends.
 *  \return Bag with the packages instanciation on which current package depends.
 *
 *  Explorer::PackageBase depends on Beagle::PackageBase.
 */
Beagle::Package::Bag NN::PackageBase::listDependencies(void)
{
	Beagle_StackTraceBeginM();
	Package::Bag lDependencies;
	lDependencies.push_back(new Explorer::PackageBase());
	return lDependencies;
	Beagle_StackTraceEndM("Beagle::Package::Bag NN::PackageBase::listDependencies(void)");
}
