
#include "explorer/NN.hpp"
#include <qwt_plot_layout.h>
#include <qwt_legend.h>
#include <qwt_legend_item.h>
#include <qwt_color_map.h>
#include <qwt_plot_curve.h>
#include <qwt_legend_item.h>

using namespace Explorer;


NN::Plot::Plot(const NN::PlotData::Data& inData, QWidget *inParent) :
        QwtPlot(inParent)
{
	Beagle_StackTraceBeginM();

    plotLayout()->setAlignCanvasToScales(true);

    QwtLegend* lLegend = new QwtLegend;
    lLegend->setItemMode(QwtLegend::CheckableItem);
    insertLegend(lLegend, QwtPlot::RightLegend);

    setAxisTitle(QwtPlot::xBottom, inData.mLabelAxisX);
    setAxisScale(QwtPlot::xBottom, inData.mMinX, inData.mMaxX);

    setAxisTitle(QwtPlot::yLeft, inData.mLabelAxisY);
    setAxisScale(QwtPlot::yLeft, inData.mMinY, inData.mMaxY);

    QwtLinearColorMap lColorMap(Qt::red, Qt::blue);
    for (unsigned int i = 0; i < inData.mLabelCurve.size(); i++) {
        QwtPlotCurve* lCurve = new QwtPlotCurve(inData.mLabelCurve[i]);
        lCurve->setPen(lColorMap.color(QwtDoubleInterval(0, inData.mLabelCurve.size()-1), i));
        lCurve->attach(this);
        showCurve(lCurve, false);
        mCurves.push_back(lCurve);
    }

    connect(this, SIGNAL(legendChecked(QwtPlotItem *, bool)), SLOT(showCurve(QwtPlotItem *, bool)));

	Beagle_StackTraceEndM("NN::Plot::Plot(const NN::PlotData::Data& inData, QWidget *inParent):");
}


void NN::Plot::showCurve(QwtPlotItem *inItem, bool inOn)
{
	Beagle_StackTraceBeginM();

    inItem->setVisible(inOn);
    QWidget *w = legend()->find(inItem);
    if ( w && w->inherits("QwtLegendItem") )
        ((QwtLegendItem *)w)->setChecked(inOn);
    
    replot();

	Beagle_StackTraceEndM("void NN::Plot::showCurve(QwtPlotItem *inItem, bool inOn)");
}


void NN::Plot::updateView(const NN::PlotData::Data& inData)
{
	Beagle_StackTraceBeginM();

    for (unsigned int i = 0; i < inData.mLabelCurve.size(); i++) {
        mCurves[i]->setRawData(inData.mX.data(), inData.mY[i].data(), inData.mX.size());
    }

    replot();

	Beagle_StackTraceEndM("void NN::Plot::updateView(const NN::PlotData::Data& inData)");
}


void NN::Plot::closeEvent(QCloseEvent *inEvent)
{
	Beagle_StackTraceBeginM();

    emit closed();
    QwtPlot::closeEvent(inEvent);
    
	Beagle_StackTraceEndM("void NN::Plot::closeEvent(QCloseEvent *inEvent)");
}
