
#ifndef Explorer_SN_Function_hpp
#define Explorer_SN_Function_hpp

#include <beagle/Beagle.hpp>

#include "explorer/DisplayObject.hpp"

namespace Explorer
{
namespace SN
{

class Function : public DisplayObject
{

public:

	//! SN::Function allocator type.
	typedef Beagle::AllocatorT<SN::Function,DisplayObject::Alloc> Alloc;
	//! SN::Function handle type.
	typedef Beagle::PointerT<SN::Function,DisplayObject::Handle> Handle;
	//! SN::Function bag type.
	typedef Beagle::ContainerT<SN::Function,DisplayObject::Bag> Bag;

	Function(const Beagle::SN::Function& inFunction);
	virtual ~Function()
	{ }

	virtual void        reset();

	virtual int         getDisplaySize() const;
	virtual std::string getDisplayName(int inIndex) const;
	virtual float       getDisplayValue(int inIndex) const;
	virtual float       getMinDisplayValue(int inIndex) const;
	virtual float       getMaxDisplayValue(int inIndex) const;

	virtual std::string getToolTip() const;

private:

	static std::vector<std::string> smDisplayNames;      //!< Names of the display values.
	static std::vector<float>       smMinDisplayValues;  //!< The minimum of the display values.
	static std::vector<float>       smMaxDisplayValues;  //!< The maximum of the display values.
	std::vector<float>              mDisplayValues;      //!< Values to be displayed.
    std::string                     mToolTip;            //!< Tool tip of the node.

};

}
}


inline int Explorer::SN::Function::getDisplaySize() const
{
	Beagle_StackTraceBeginM();

	return Explorer::DisplayObject::getDisplaySize() + smDisplayNames.size();

	Beagle_StackTraceEndM("int SN::Function::getDisplaySize() const");
}


inline std::string Explorer::SN::Function::getDisplayName(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getDisplayName(inIndex);
	else
		return smDisplayNames[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("std::string SN::Function::getDisplayName(int inIndex) const");
}


inline float Explorer::SN::Function::getDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getDisplayValue(inIndex);
	else
		return mDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float SN::Function::getDisplayValue(int inIndex) const");
}


inline float Explorer::SN::Function::getMinDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getMinDisplayValue(inIndex);
	else
		return smMinDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float SN::Function::getMinDisplayValue(int inIndex) const");
}


inline float Explorer::SN::Function::getMaxDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::DisplayObject::getDisplaySize())
		return Explorer::DisplayObject::getMaxDisplayValue(inIndex);
	else
		return smMaxDisplayValues[inIndex - Explorer::DisplayObject::getDisplaySize()];

	Beagle_StackTraceEndM("float SN::Function::getMaxDisplayValue(int inIndex) const");
}


inline std::string Explorer::SN::Function::getToolTip() const
{
	Beagle_StackTraceBeginM();

	return mToolTip;

	Beagle_StackTraceEndM("std::string SN::Function::getToolTip() const");
}


#endif // Explorer_SN_Function_hpp
