
#ifndef Explorer_SN_Individual_hpp
#define Explorer_SN_Individual_hpp

#include <beagle/SN.hpp>

#include "explorer/Individual.hpp"
#include "explorer/Fitness.hpp"

namespace Explorer
{
namespace SN
{

class Individual : public Explorer::Individual
{

public:

	//! SN::Individual allocator type.
	typedef Beagle::AllocatorT<SN::Individual,Explorer::Individual::Alloc> Alloc;
	//! SN::Individual handle type.
	typedef Beagle::PointerT<SN::Individual,Explorer::Individual::Handle> Handle;
	//! SN::Individual bag type.
	typedef Beagle::ContainerT<SN::Individual,Explorer::Individual::Bag> Bag;

	Individual();
	virtual ~Individual()
	{ }

	virtual void        reset();
	virtual void        setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem);

	virtual int         getDisplaySize() const;
	virtual std::string getDisplayName(int inIndex) const;
	virtual float       getDisplayValue(int inIndex) const;
	virtual float       getMinDisplayValue(int inIndex) const;
	virtual float       getMaxDisplayValue(int inIndex) const;

private:

	static std::vector<std::string> smDisplayNames;      //!< Names of the display values.
	static std::vector<float>       smMinDisplayValues;  //!< The minimum of the display values.
	static std::vector<float>       smMaxDisplayValues;  //!< The maximum of the display values.
	std::vector<float>              mDisplayValues;      //!< Values to be displayed.

};

}
}


inline int Explorer::SN::Individual::getDisplaySize() const
{
	Beagle_StackTraceBeginM();

	return Explorer::Individual::getDisplaySize() + smDisplayNames.size();

	Beagle_StackTraceEndM("int SN::Individual::getDisplaySize() const");
}


inline std::string Explorer::SN::Individual::getDisplayName(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::Individual::getDisplaySize())
		return Explorer::Individual::getDisplayName(inIndex);
	else
		return smDisplayNames[inIndex - Explorer::Individual::getDisplaySize()];

	Beagle_StackTraceEndM("std::string SN::Individual::getDisplayName(int inIndex) const");
}


inline float Explorer::SN::Individual::getDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::Individual::getDisplaySize())
		return Explorer::Individual::getDisplayValue(inIndex);
	else
		return mDisplayValues[inIndex - Explorer::Individual::getDisplaySize()];

	Beagle_StackTraceEndM("float SN::Individual::getDisplayValue(int inIndex) const");
}


inline float Explorer::SN::Individual::getMinDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::Individual::getDisplaySize())
		return Explorer::Individual::getMinDisplayValue(inIndex);
	else
		return smMinDisplayValues[inIndex - Explorer::Individual::getDisplaySize()];

	Beagle_StackTraceEndM("float SN::Individual::getMinDisplayValue(int inIndex) const");
}


inline float Explorer::SN::Individual::getMaxDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Explorer::Individual::getDisplaySize())
		return Explorer::Individual::getMaxDisplayValue(inIndex);
	else
		return smMaxDisplayValues[inIndex - Explorer::Individual::getDisplaySize()];

	Beagle_StackTraceEndM("float SN::Individual::getMaxDisplayValue(int inIndex) const");
}


#endif // Explorer_SN_Individual_hpp
