
#ifndef Explorer_SN_InteractionData_hpp
#define Explorer_SN_InteractionData_hpp

#include <QObject>
#include <QCloseEvent>
#include <QWidget>
#include <QLabel>
#include <QSpinBox>
#include <QCheckBox>
#include <QVector>

#include <beagle/SN.hpp>

#include "explorer/SN/NetworkWidget.hpp"


namespace Explorer
{
namespace SN
{

/*!
 *  \brief This class is a singleton and contains the data for the SN module
 *  that enables evolution to interact with the GUI.
 */
class InteractionData : public QObject
{

	Q_OBJECT

public:

    static InteractionData& instance();

	void setup(Beagle::System::Handle ioSystem);
	void setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators);

	void update(const Beagle::SN::Network& inNetwork, const Beagle::Context& inContext);

	void closeEvent(QCloseEvent *event);

private:

	InteractionData();
    InteractionData(const InteractionData&)
    { }
    InteractionData& operator=(const InteractionData&)
    { }
 	~InteractionData()
    { }

    QVector<QCheckBox*> mWidgetCheckBoxes;
    NetworkWidget*      mNetworkWidget;         //!< Contains graphical display of the network.

};

}
}

#endif // Explorer_SN_InteractionData_hpp

