
#include "explorer/SN.hpp"

using namespace Explorer;


std::vector<std::string> SN::Function::smDisplayNames;
std::vector<float>       SN::Function::smMinDisplayValues;
std::vector<float>       SN::Function::smMaxDisplayValues;


SN::Function::Function(const Beagle::SN::Function& inFunction) :
		mDisplayValues(0)
{
	Beagle_StackTraceBeginM();

    const unsigned int lNumLevels = Beagle::SN::Function::getNumLevels();

    // Initialize static vector values.
    if (smDisplayNames.size() == 0) {
        // Displaying all levels is a special case that has to be treated separately
        // in the paint() method of FunctionNode.
        smDisplayNames.push_back("All Levels");
        smMinDisplayValues.push_back(0);
        smMaxDisplayValues.push_back(lNumLevels-1);

        // Don't display the first and the last level, since they don't carry any
        // information.
        for (unsigned int i = 1; i < lNumLevels-1; i++) {
            smDisplayNames.push_back("Level " + Beagle::uint2str(i));
            smMinDisplayValues.push_back(0);
            smMaxDisplayValues.push_back(Beagle::SN::Function::getMaxSetsInLevel(i));
        }
    }

    // Any value greater than max display value (e.g. lNumLevels) is invalid.
    mDisplayValues.push_back(lNumLevels);  // Value for "All Levels"
    for (unsigned int i = 1; i < lNumLevels-1; i++) {
        mDisplayValues.push_back(inFunction.getNumSetsInLevel(i));
        mToolTip += "Level " + Beagle::uint2str(i) + " = " + Beagle::uint2str(inFunction.getNumSetsInLevel(i));
        if (i < lNumLevels - 2) mToolTip += "\n";
    }

	Beagle_StackTraceEndM("SN::Function::Function(const Beagle::SN::Function& inFunction)");
}


/*!
 *  \brief Reset information.
 */
void SN::Function::reset()
{
	Beagle_StackTraceBeginM();

	smDisplayNames.clear();
	smMinDisplayValues.clear();
	smMaxDisplayValues.clear();
	mDisplayValues.clear();

	Beagle_StackTraceEndM("void SN::Function::reset()");
}
