
#include "explorer/SN.hpp"

using namespace Explorer;


SN::FunctionNode::FunctionNode(Function::Handle inFunction) :
		mFunction(inFunction)
{ }


void SN::FunctionNode::paint(QPainter *ioPainter, const QStyleOptionGraphicsItem *inOption, QWidget *inWidget)
{
	Beagle_StackTraceBeginM();

	Q_UNUSED(inWidget);

	static const qreal lOuterPenW = 0.2;

	DisplayScene* lScene = qobject_cast<DisplayScene*>(scene());
	if (lScene == 0)
		return;

	static const qreal lBoundingSize = lScene->getNodeBoundingSize();
    static const qreal lOuterRectW = lBoundingSize - lOuterPenW;
    static const qreal lOuterRectX = lOuterPenW/2;
    static const QRectF lOuterRect(lOuterRectX, lOuterRectX, lOuterRectW, lOuterRectW);

	int lNodeHueIndex = lScene->getNodeHueIndex();
	int lNodeSatIndex = lScene->getNodeSatIndex();
	int lNodeValIndex = lScene->getNodeValIndex();

    // Don't draw anything if all indices refer to "None"
	if (lNodeHueIndex == 0 && lNodeSatIndex == 0 && lNodeValIndex == 0) {
    }
	else {
        // Notify QGraphicsScene if there is a change in geometry.
        // Actually, no need for this because the geometry change is triggered
        // only by any change in the node color indexes, which invalidates
        // the scene in Explorer::DisplaySettings.  Also, this function call
        // produces a huge increase in CPU usage.
        //prepareGeometryChange();

        const DisplayObject::Handle lDisplayObject = getDisplayObject();
        int lDisplaySize = lDisplayObject->getDisplaySize();

        // If any of the indices refer to "All Lines", then draw an array of boxes,
        // one box for each line.  The number of rows in which the boxes are drawn
        // depends on how many of the indexes is 1.
        if (lNodeHueIndex <= 1 && lNodeSatIndex <= 1 && lNodeValIndex <= 1) {
            int lNumRows = lNodeHueIndex + lNodeSatIndex + lNodeValIndex;
            int lNumInputs = lDisplaySize-2;
            int lNumCols = lNumInputs/lNumRows + ((lNumInputs%lNumRows > 0) ? 1 : 0);
            QFont lFont = ioPainter->font();
            lFont.setPointSizeF(1);
            ioPainter->setFont(lFont);
            for (int i = 2; i < lDisplaySize; ++i) {
                int j = i-2;
                int lRow = j/lNumCols;
                int lCol = j%lNumCols;
                QColor lOuterColor = (lDisplayObject->getDisplayValue(i) > lDisplayObject->getMaxDisplayValue(i)) ? QColor(Qt::transparent) : getOuterColor(i, 0, 0);
                ioPainter->setPen(QPen(Qt::black, lOuterPenW));
                ioPainter->setBrush(lOuterColor);
                ioPainter->save();
                ioPainter->translate(lCol*lOuterRectW, lRow*lOuterRectW);
                ioPainter->drawRect(lOuterRect);
                //ioPainter->drawText(lOuterRect, Qt::AlignCenter, QString::number(j));
                ioPainter->restore();
            }

            // Update the bounding rectangle.
            mBoundingRect = QRectF(0, 0, lNumCols*lOuterRectW + lOuterPenW, lNumRows*lOuterRectW + lOuterPenW);
        }
        else { // Otherwise, draw only one box.
            QColor lOuterColor = ((lDisplayObject->getDisplayValue(lNodeHueIndex) > lDisplayObject->getMaxDisplayValue(lNodeHueIndex)) ||
                                  (lDisplayObject->getDisplayValue(lNodeSatIndex) > lDisplayObject->getMaxDisplayValue(lNodeSatIndex)) ||
                                  (lDisplayObject->getDisplayValue(lNodeValIndex) > lDisplayObject->getMaxDisplayValue(lNodeValIndex))) ? QColor(Qt::transparent) : getOuterColor();
            ioPainter->setPen(QPen(Qt::black, lOuterPenW));
            ioPainter->setBrush(lOuterColor);
            ioPainter->drawRect(lOuterRect);

            // Update the bounding rectangle.
            mBoundingRect = QRectF(0, 0, lOuterRectW + lOuterPenW, lOuterRectW + lOuterPenW);
        }
    }

	Beagle_StackTraceEndM("void SN::FunctionNode::paint(QPainter *ioPainter, const QStyleOptionGraphicsItem *inOption, QWidget *inWidget)");
}
