
#include "explorer/SN.hpp"

using namespace Explorer;


std::vector<std::string> SN::Individual::smDisplayNames;
std::vector<float>       SN::Individual::smMinDisplayValues;
std::vector<float>       SN::Individual::smMaxDisplayValues;


SN::Individual::Individual() :
		mDisplayValues(0)
{}


/*!
 *  \brief Reset information.
 */
void SN::Individual::reset()
{
	Beagle_StackTraceBeginM();

	Explorer::Individual::reset();

	smDisplayNames.clear();
	smMinDisplayValues.clear();
	smMaxDisplayValues.clear();
	mDisplayValues.clear();

	Beagle_StackTraceEndM("void SN::Individual::reset()");
}


/*!
 *  \brief Setup the member variables with data from the provided Beagle Individual.
 */
void SN::Individual::setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();

	Explorer::Individual::setup(inIndividual, ioSystem);

	if (inIndividual != NULL) {
		Beagle::SN::Individual::Handle lIndividual = Beagle::castHandleT<Beagle::SN::Individual>(inIndividual);

		// Initialize static vector values.
		if (smDisplayNames.size() == 0) {
			smDisplayNames.push_back("Size");
			smMinDisplayValues.push_back(FLT_MAX);
			smMaxDisplayValues.push_back(-FLT_MAX);
		}

		float lSize = (*lIndividual)[0]->getNumComparators();
		mDisplayValues.push_back(lSize);

		// Keep track of the min and max display values.
		if (lSize < smMinDisplayValues[0])
			smMinDisplayValues[0] = lSize;
		if (lSize > smMaxDisplayValues[0])
			smMaxDisplayValues[0] = lSize;
	}

	Beagle_StackTraceEndM("void SN::Individual::setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem)");
}
