
#include "explorer/SN.hpp"

using namespace Explorer;


SN::InteractionData&    SN::Interaction::mSNInteractionData = SN::InteractionData::instance();


/*!
 *  \brief Construct an interaction object.
 */
SN::Interaction::Interaction() :
        Explorer::Interaction()
{ }


/*!
 *  \brief Setup the interaction object.
 */
void SN::Interaction::setup(Beagle::System::Handle ioSystem)
{
	Beagle_StackTraceBeginM();
    mInteractionData.setup(ioSystem);
    mSNInteractionData.setup(ioSystem);
	Beagle_StackTraceEndM("void SN::Interaction::setup(Beagle::System::Handle ioSystem)");
}


/*!
 *  \brief Create and setup the display window indicators.
 *  \param inParent The parent widget of the new display indicator widgets.
 *  \param outDisplayIndicators The new display indicator widgets are returned in this array.
 */
void SN::Interaction::setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators)
{	Beagle_StackTraceBeginM();
    mInteractionData.setupDisplayIndicators(inParent, outDisplayIndicators);
    mSNInteractionData.setupDisplayIndicators(inParent, outDisplayIndicators);
	Beagle_StackTraceEndM("void SN::Interaction::setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators)");
}


/*!
 *  \brief Controls GUI updates and entering Qt's event loop.
 *  \param inNetwork The graph of the individual being evaluated.
 *  \param inContext The context of evolution.
 */
void SN::Interaction::update(const Beagle::SN::Network& inNetwork, const Beagle::Context& inContext)
{
	Beagle_StackTraceBeginM();
    if (mInteractionData.testPauseOrPastBreak(inContext) || mInteractionData.testAtBreak(inContext)) {
        mInteractionData.update(inContext);
        mSNInteractionData.update(inNetwork, inContext);
        mInteractionData.enterEventLoop();
    }
    else {
        mInteractionData.update(inContext);
        mSNInteractionData.update(inNetwork, inContext);
    }
	Beagle_StackTraceEndM("void SN::Interaction::update(const Beagle::SN::Network& inNetwork, const Beagle::Context& inContext)");
}


/*!
 *  \brief Process the close event.
 */
void SN::Interaction::closeEvent(QCloseEvent *event)
{
	Beagle_StackTraceBeginM();
    mInteractionData.closeEvent(event);
    mSNInteractionData.closeEvent(event);
	Beagle_StackTraceEndM("void SN::Interaction::closeEvent(QCloseEvent *event)");
}
