
#include "explorer/SN.hpp"

using namespace Explorer;


/*!
 *  \brief Returns the static singleton instance of the InteractionData class.
 */
SN::InteractionData::InteractionData& SN::InteractionData::instance()
{
	Beagle_StackTraceBeginM();
    static SN::InteractionData lInteractionData;
    return lInteractionData;
	Beagle_StackTraceEndM("SN::InteractionData::InteractionData& SN::InteractionData::instance()");
}


/*!
 *  \brief Construct an interaction object.
 */
SN::InteractionData::InteractionData() :
        mNetworkWidget(0)
{ }


/*!
 *  \brief Setup the interaction object for this class.
 */
void SN::InteractionData::setup(Beagle::System::Handle ioSystem)
{
	Beagle_StackTraceBeginM();

    // Create and setup the NetworkWidget.  Since we want it to be a separate window,
    // we don't make it a child of another widget.
    mNetworkWidget = new NetworkWidget;
    mNetworkWidget->setWindowTitle("Sorting Network");

	Beagle_StackTraceEndM("void SN::InteractionData::setup(Beagle::System::Handle ioSystem)");
}


/*!
 *  \brief Create and setup the display window indicators for this class.
 *  \param inParent The parent widget of the new display indicator widgets.
 *  \param outDisplayIndicators The new display indicator widgets are returned in this array.
 */
void SN::InteractionData::setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators)
{
	Beagle_StackTraceBeginM();

    // Widget for the primal network
    mWidgetCheckBoxes.push_back(new QCheckBox(inParent));
    mWidgetCheckBoxes.back()->setObjectName(QString::fromUtf8("mWidgetCheckBoxes0"));
    mWidgetCheckBoxes.back()->setText(QApplication::translate("EvolutionTab", mNetworkWidget->windowTitle().toStdString().c_str(), 0, QApplication::UnicodeUTF8));
    mWidgetCheckBoxes.back()->setChecked(true);
	connect(mWidgetCheckBoxes.back(), SIGNAL(toggled(bool)), mNetworkWidget, SLOT(setVisible(bool)));
	connect(mNetworkWidget, SIGNAL(closed()), mWidgetCheckBoxes.back(), SLOT(toggle()));
    mNetworkWidget->setVisible(mWidgetCheckBoxes.back()->isChecked());
    outDisplayIndicators.push_back(mWidgetCheckBoxes.back());

	Beagle_StackTraceEndM("void SN::InteractionData::setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators)");
}


/*! 
 *  \brief Update the status of the widgets in the evolution tab.
 *  \param inNetwork The graph of the individual being evaluated.
 *  \param inContext The context of evolution.
 */
void SN::InteractionData::update(const Beagle::SN::Network& inNetwork, const Beagle::Context& inContext)
{
    Beagle_StackTraceBeginM();

    // Update the network displays
    if (mNetworkWidget->setupComplete()) {
        mNetworkWidget->updateView();
    }
    else {
        mNetworkWidget->reset();
        mNetworkWidget->setup(inNetwork, inContext);
    }

    Beagle_StackTraceEndM("void SN::InteractionData::update(const Beagle::SN::Network& inNetwork, const Beagle::Context& inContext)");
}


/*!
 *  \brief Process the close event.
 */
void SN::InteractionData::closeEvent(QCloseEvent *inEvent)
{
	Beagle_StackTraceBeginM();

    if (mNetworkWidget != 0) mNetworkWidget->close();

	Beagle_StackTraceEndM("void SN::InteractionData::closeEvent(QCloseEvent *inEvent)");
}
