
#include "explorer/SN.hpp"

using namespace Explorer;


/*!
 *  \brief Construct package with basic Explorer SN objects.
 */
SN::PackageBase::PackageBase() :
		Beagle::Package("Explorer-SN-PackageBase")
{ }


/*!
 *  \brief Configure system by adding necessary components into it.
 *  \param ioSystem System to configure.
 */
void SN::PackageBase::configure(Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();
	Beagle::Factory& lFactory = ioSystem.getFactory();

	// Add available basic types to the factory
	lFactory.insertAllocator("Explorer::SN::Individual", new Explorer::SN::Individual::Alloc);
	lFactory.insertAllocator("Explorer::SN::Interaction", new Explorer::SN::Interaction::Alloc);

	// Set aliases
	lFactory.aliasAllocator("Explorer::SN::Individual", "Explorer-SN-Individual");
	lFactory.aliasAllocator("Explorer::SN::Interaction", "SN-Interaction");

	// Set some basic concept-type associations in factory
	lFactory.setConcept("Explorer-Individual", "Explorer-SN-Individual");
	lFactory.setConcept("Interaction", "SN-Interaction");

	Beagle_StackTraceEndM("void SN::PackageBase::configure(System&)");
}


/*!
 *  \brief List packages on which current package depends.
 *  \return Bag with the packages instanciation on which current package depends.
 *
 *  Explorer::PackageBase depends on Beagle::PackageBase.
 */
Beagle::Package::Bag SN::PackageBase::listDependencies(void)
{
	Beagle_StackTraceBeginM();
	Package::Bag lDependencies;
	lDependencies.push_back(new Explorer::PackageBase());
	return lDependencies;
	Beagle_StackTraceEndM("Beagle::Package::Bag SN::PackageBase::listDependencies(void)");
}
