
#ifndef Explorer_AllocatorT_hpp
#define Explorer_AllocatorT_hpp

#include <beagle/Beagle.hpp>


namespace Explorer
{

/*!
 *  This is a modified version of Beagle::AllocatorT, not implementing the
 *  clone and copy methods because of compiler errors when classes are derived
 *  from both Beagle::Object and QObject/QWidget.
 */
template <class T, class BaseType>
class AllocatorT : public BaseType
{

public:

	//! AllocatorT allocator type.
	typedef Beagle::AllocatorT<AllocatorT<T,BaseType>,typename BaseType::Alloc> Alloc;
	//! AllocatorT handle type.
	typedef Beagle::PointerT<AllocatorT<T,BaseType>,typename BaseType::Handle> Handle;
	//! AllocatorT bag type.
	typedef Beagle::ContainerT<AllocatorT<T,BaseType>,typename BaseType::Bag> Bag;

	AllocatorT()
	{ }
	virtual ~AllocatorT()
	{ }

	virtual Beagle::Object* allocate() const;
	virtual Beagle::Object* clone(const Beagle::Object& inOrigObj) const;
	virtual void            copy(Beagle::Object& outCopy, const Beagle::Object& inOriginal) const;

};

}


/*!
 *  \brief  Allocate a new T-type object and the heap.
 *  \return Pointer to the allocated T-type object.
 */
template <class T, class BaseType>
Beagle::Object* Explorer::AllocatorT<T,BaseType>::allocate() const
{
	Beagle_StackTraceBeginM();
#ifndef BEAGLE_NDEBUG
	T* lT = new T;
	if(!lT) throw Beagle_InternalExceptionM("Out of memory!");
	return lT;
#else // BEAGLE_NDEBUG
	return new T;
#endif // BEAGLE_NDEBUG
	Beagle_StackTraceEndM("Object* AllocatorT<T,BaseType>::allocate() const");
}


/*!
 *  \brief  Clone method not implemented.
 *  \throw  Beagle::InternalException If the method is called.
 */
template <class T, class BaseType>
Beagle::Object* Explorer::AllocatorT<T,BaseType>::clone(const Beagle::Object& inOriginal) const
{
  Beagle_StackTraceBeginM();
  throw Beagle_UndefinedMethodInternalExceptionM("clone", "Explorer::AllocatorT", "Explorer::AllocatorT");
  Beagle_StackTraceEndM("Object* AllocatorT<T,BaseType>::clone(const Object&) const");
}


/*!
 *  \brief  Copy method not implemented.
 *  \throw  Beagle::InternalException If the method is called.
 */
template <class T, class BaseType>
void Explorer::AllocatorT<T,BaseType>::copy(Beagle::Object& outCopy,
                                            const Beagle::Object& inOriginal) const
{
  Beagle_StackTraceBeginM();
  throw Beagle_UndefinedMethodInternalExceptionM("copy", "Explorer::AllocatorT", "Explorer::AllocatorT");
  Beagle_StackTraceEndM("void AllocatorT<T,BaseType>::copy(Object&, const Object&) const");
}


#endif // Explorer_AllocatorT_hpp
