
#ifndef Explorer_FitnessSimple_hpp
#define Explorer_FitnessSimple_hpp

#include <beagle/Beagle.hpp>

#include "explorer/Fitness.hpp"

namespace Explorer
{

class FitnessSimple : public Fitness
{

public:

	//! FitnessSimple allocator type.
	typedef Beagle::AllocatorT<FitnessSimple,Fitness::Alloc>
	Alloc;
	//! FitnessSimple handle type.
	typedef Beagle::PointerT<FitnessSimple,Fitness::Handle>
	Handle;
	//! FitnessSimple bag type.
	typedef Beagle::ContainerT<FitnessSimple,Fitness::Bag>
	Bag;


	FitnessSimple();
	virtual ~FitnessSimple()
	{ }

	virtual void        reset();
	virtual void        setup(Beagle::Fitness::Handle inFitness, Beagle::System& ioSystem);

	virtual int         getDisplaySize() const;
	virtual std::string getDisplayName(int inIndex) const;
	virtual float       getDisplayValue(int inIndex) const;
	virtual float       getMinDisplayValue(int inIndex) const;
	virtual float       getMaxDisplayValue(int inIndex) const;

private:

	static std::string  smDisplayName;       //!< Name of the display value.
	static float        smMinDisplayValue;   //!< The minimum of the display value.
	static float        smMaxDisplayValue;   //!< The maximum of the display value.
	float               mDisplayValue;       //!< Value to be displayed.

};

}


inline int Explorer::FitnessSimple::getDisplaySize() const
{
	Beagle_StackTraceBeginM();

	return Fitness::getDisplaySize() + 1;

	Beagle_StackTraceEndM("int FitnessSimple::getDisplaySize() const");
}


inline std::string Explorer::FitnessSimple::getDisplayName(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Fitness::getDisplaySize())
		return Fitness::getDisplayName(inIndex);
	else
		return smDisplayName;

	Beagle_StackTraceEndM("std::string FitnessSimple::getDisplayName(int inIndex) const");
}


inline float Explorer::FitnessSimple::getDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Fitness::getDisplaySize())
		return Fitness::getDisplayValue(inIndex);
	else
		return mDisplayValue;

	Beagle_StackTraceEndM("float FitnessSimple::getDisplayValue(int inIndex) const");
}


inline float Explorer::FitnessSimple::getMinDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Fitness::getDisplaySize())
		return Fitness::getMinDisplayValue(inIndex);
	else
		return smMinDisplayValue;

	Beagle_StackTraceEndM("float FitnessSimple::getMinDisplayValue(int inIndex) const");
}


inline float Explorer::FitnessSimple::getMaxDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	if (inIndex < Fitness::getDisplaySize())
		return Fitness::getMaxDisplayValue(inIndex);
	else
		return smMaxDisplayValue;

	Beagle_StackTraceEndM("float FitnessSimple::getMaxDisplayValue(int inIndex) const");
}


#endif // Explorer_FitnessSimple_hpp
