
#ifndef Explorer_History_hpp
#define Explorer_History_hpp

#include <beagle/Beagle.hpp>

#include "explorer/Individual.hpp"
#include "explorer/HistoryEntry.hpp"


namespace Explorer
{

class History : public Beagle::Object
{

public:

	//! History allocator type.
	typedef Beagle::AllocatorT<History,Beagle::Object::Alloc>
	Alloc;
	//! History handle type.
	typedef Beagle::PointerT<History,Beagle::Object::Handle>
	Handle;
	//! History bag type.
	typedef Beagle::ContainerT<History,Beagle::Object::Bag>
	Bag;

	History();
	virtual ~History()
	{ }

	virtual void reset();
	virtual void setup(Beagle::History::Handle inHistory, Beagle::System& ioSystem);

	unsigned int getVivariumSize(unsigned int inGeneration) const;
	unsigned int getMinGeneration() const;
	unsigned int getMaxGeneration() const;

    const HistoryEntry::Handle getEntry(unsigned int inHistoryId, unsigned int inHistoryVar) const;
	const HistoryEntry::Handle getMarkedEntry(unsigned int inGeneration, unsigned int inVivariumIndex) const;

	void findGenealogy(HistoryEntry::Bag& ioSelectedEntries, int inAncestryDepth, int inDescendancyDepth);
	void writeGenealogy(std::ostream& outStream, HistoryEntry::Bag& ioSelectedEntries, int inAncestryDepth, int inDescendancyDepth);

private:

	std::vector< std::vector<HistoryEntry::Handle> > mEntries;          //!< All history entries
	std::vector< std::vector<HistoryEntry::Handle> > mMarkedEntries;    //!< Entries with a generation mark
	unsigned int                                     mMinGeneration;    //!< The minimum generation that was read

	void genealogySearchBFS(HistoryEntry::Handle ioEntry, int inDepth, bool inAncestry);

};

}


inline unsigned int Explorer::History::getVivariumSize(unsigned int inGeneration) const
{
	Beagle_StackTraceBeginM();

	return mMarkedEntries[inGeneration].size();

	Beagle_StackTraceEndM("unsigned int History::getVivariumSize(unsigned int inGeneration) const");
}


inline unsigned int Explorer::History::getMinGeneration() const
{
	Beagle_StackTraceBeginM();

	return mMinGeneration;

	Beagle_StackTraceEndM("unsigned int History::getMinGeneration() const");
}


inline unsigned int Explorer::History::getMaxGeneration() const
{
	Beagle_StackTraceBeginM();

	return mMarkedEntries.size();

	Beagle_StackTraceEndM("unsigned int History::getMaxGeneration() const");
}


inline const Explorer::HistoryEntry::Handle Explorer::History::getEntry(unsigned int inHistoryId, unsigned int inHistoryVar) const
{
	Beagle_StackTraceBeginM();

    return mEntries[inHistoryId][inHistoryVar];

    Beagle_StackTraceEndM("const HistoryEntry::Handle History::getEntry(unsigned int inHistoryId, unsigned int inHistoryVar) const");
}


inline const Explorer::HistoryEntry::Handle Explorer::History::getMarkedEntry(unsigned int inGeneration, unsigned int inVivariumIndex) const
{
	Beagle_StackTraceBeginM();

	return mMarkedEntries[inGeneration][inVivariumIndex];

	Beagle_StackTraceEndM("const HistoryEntry::Handle History::getMarkedEntry(unsigned int inGeneration, unsigned int inVivariumIndex) const");
}


#endif // Explorer_History_hpp
