
#ifndef Explorer_HistoryEntry_hpp
#define Explorer_HistoryEntry_hpp

#include <beagle/Beagle.hpp>

#include "explorer/Individual.hpp"

namespace Explorer
{

class HistoryEntry : public Beagle::Object
{

public:

	//! HistoryEntry allocator type.
	typedef Beagle::AllocatorT<HistoryEntry,Beagle::Object::Alloc>
	Alloc;
	//! HistoryEntry handle type.
	typedef Beagle::PointerT<HistoryEntry,Beagle::Object::Handle>
	Handle;
	//! HistoryEntry bag type.
	typedef Beagle::ContainerT<HistoryEntry,Beagle::Object::Bag>
	Bag;

	//! Color values used in search algorithms.
	enum SearchColor {eWhite, eGray, eBlack};

	//! Action that created this individual.
	enum CreationAction {eUnknownAction, eSelection, eElitism, eMutation, eCrossover, eFlush};

	explicit HistoryEntry();
	virtual ~HistoryEntry()
	{ }

	virtual void reset();
	virtual void setup(Beagle::HistoryEntry::Handle inHistoryEntry, Beagle::System& ioSystem);

	std::vector<Handle>& getParents();
	std::vector<Handle>& getChildren();
	void                 addParent(Handle inHistoryEntry);
	void                 addChild(Handle inHistoryEntry);

	void                 setGenealogyFlag(bool inGenealogyFlag);
	bool                 getGenealogyFlag() const;

	void                 setCreationAction(const std::string& inAction);
	CreationAction       getCreationAction() const;

	void                 setMarkedFlag(bool inMarkedFlag);
	bool                 getMarkedFlag() const;

	void                 setHistoryId(unsigned int inHistoryId);
	unsigned int         getHistoryId() const;

	void                 setHistoryVar(unsigned int inHistoryVar);
	unsigned int         getHistoryVar() const;

	void                 setGeneration(unsigned int inGeneration);
	unsigned int         getGeneration() const;

	void                 setIndividual(Individual::Handle inIndividual);
	Individual::Handle   getIndividual() const;

	void                 setSearchColor(SearchColor inSearchColor);
	SearchColor          getSearchColor() const;

	virtual std::string getToolTip() const;

	bool isMember(HistoryEntry::Handle inHistoryEntry, HistoryEntry::Bag::Handle inBag) const;

private:

	unsigned int        mHistoryId;
	unsigned int        mHistoryVar;
	Individual::Handle  mIndividual;
	std::vector<Handle> mParents;
	std::vector<Handle> mChildren;
	CreationAction      mCreationAction;
	bool                mMarkedFlag;
	bool                mGenealogyFlag;
	unsigned int        mGeneration;
	SearchColor         mSearchColor;

};

}


inline std::vector<Explorer::HistoryEntry::Handle>& Explorer::HistoryEntry::getParents()
{
	Beagle_StackTraceBeginM();

	return mParents;

	Beagle_StackTraceEndM("std::vector<HistoryEntry::Handle>& HistoryEntry::getParents()");
}


inline std::vector<Explorer::HistoryEntry::Handle>& Explorer::HistoryEntry::getChildren()
{
	Beagle_StackTraceBeginM();

	return mChildren;

	Beagle_StackTraceEndM("std::vector<HistoryEntry::Handle>& HistoryEntry::getChildren()");
}


inline void Explorer::HistoryEntry::addParent(Handle inHistoryEntry)
{
	Beagle_StackTraceBeginM();

	mParents.push_back(inHistoryEntry);

	Beagle_StackTraceEndM("void HistoryEntry::addParent(Handle inHistoryEntry)");
}


inline void Explorer::HistoryEntry::addChild(Handle inHistoryEntry)
{
	Beagle_StackTraceBeginM();

	mChildren.push_back(inHistoryEntry);

	Beagle_StackTraceEndM("void HistoryEntry::addChild(Handle inHistoryEntry)");
}


inline void Explorer::HistoryEntry::setGenealogyFlag(bool inGenealogyFlag)
{
	Beagle_StackTraceBeginM();

	mGenealogyFlag = inGenealogyFlag;

	Beagle_StackTraceEndM("void HistoryEntry::setGenealogyFlag(bool inGenealogyFlag)");
}


inline bool Explorer::HistoryEntry::getGenealogyFlag() const
{
	Beagle_StackTraceBeginM();

	return mGenealogyFlag;

	Beagle_StackTraceEndM("bool HistoryEntry::getGenealogyFlag() const");
}


inline void Explorer::HistoryEntry::setCreationAction(const std::string& inAction)
{
	Beagle_StackTraceBeginM();

	if (inAction == "selection")
		mCreationAction = eSelection;
	else if (inAction == "elitism")
		mCreationAction = eElitism;
	else if (inAction == "mutation")
		mCreationAction = eMutation;
	else if (inAction == "crossover")
		mCreationAction = eCrossover;
	else if (inAction == "flush")
		mCreationAction = eFlush;
	else
		mCreationAction = eUnknownAction;

	Beagle_StackTraceEndM("void HistoryEntry::setCreationAction(const std::string& inAction)");
}


inline Explorer::HistoryEntry::CreationAction Explorer::HistoryEntry::getCreationAction() const
{
	Beagle_StackTraceBeginM();

	return mCreationAction;

	Beagle_StackTraceEndM("HistoryEntry::CreationAction HistoryEntry::getCreationAction() const");
}


inline void Explorer::HistoryEntry::setMarkedFlag(bool inMarkedFlag)
{
	Beagle_StackTraceBeginM();

	mMarkedFlag = inMarkedFlag;

	Beagle_StackTraceEndM("void HistoryEntry::setMarkedFlag(bool inMarkedFlag)");
}


inline bool Explorer::HistoryEntry::getMarkedFlag() const
{
	Beagle_StackTraceBeginM();

	return mMarkedFlag;

	Beagle_StackTraceEndM("bool HistoryEntry::getMarkedFlag() const");
}


inline void Explorer::HistoryEntry::setHistoryId(unsigned int inHistoryId)
{
	Beagle_StackTraceBeginM();

	mHistoryId = inHistoryId;

	Beagle_StackTraceEndM("void HistoryEntry::setHistoryId(unsigned int inHistoryId)");
}


inline unsigned int Explorer::HistoryEntry::getHistoryId() const
{
	Beagle_StackTraceBeginM();

	return mHistoryId;

	Beagle_StackTraceEndM("unsigned int HistoryEntry::getHistoryId() const");
}


inline void Explorer::HistoryEntry::setHistoryVar(unsigned int inHistoryVar)
{
	Beagle_StackTraceBeginM();

	mHistoryVar = inHistoryVar;

	Beagle_StackTraceEndM("void HistoryEntry::setHistoryVar(unsigned int inHistoryVar)");
}


inline unsigned int Explorer::HistoryEntry::getHistoryVar() const
{
	Beagle_StackTraceBeginM();

	return mHistoryVar;

	Beagle_StackTraceEndM("unsigned int HistoryEntry::getHistoryVar() const");
}


inline void Explorer::HistoryEntry::setGeneration(unsigned int inGeneration)
{
	Beagle_StackTraceBeginM();

	mGeneration = inGeneration;

	Beagle_StackTraceEndM("void HistoryEntry::setGeneration(unsigned int inGeneration)");
}


inline unsigned int Explorer::HistoryEntry::getGeneration() const
{
	Beagle_StackTraceBeginM();

	return mGeneration;

	Beagle_StackTraceEndM("unsigned int HistoryEntry::getGeneration() const");
}


inline void Explorer::HistoryEntry::setIndividual(Individual::Handle inIndividual)
{
	Beagle_StackTraceBeginM();

	mIndividual = inIndividual;

	Beagle_StackTraceEndM("void HistoryEntry::setIndividual(Individual::Handle inIndividual)");
}


inline Explorer::Individual::Handle Explorer::HistoryEntry::getIndividual() const
{
	Beagle_StackTraceBeginM();

	return mIndividual;

	Beagle_StackTraceEndM("Individual::Handle HistoryEntry::getIndividual() const");
}


inline void Explorer::HistoryEntry::setSearchColor(SearchColor inSearchColor)
{
	Beagle_StackTraceBeginM();

	mSearchColor = inSearchColor;

	Beagle_StackTraceEndM("void HistoryEntry::setSearchColor(SearchColor inSearchColor)");
}


inline Explorer::HistoryEntry::SearchColor Explorer::HistoryEntry::getSearchColor() const
{
	Beagle_StackTraceBeginM();

	return mSearchColor;

	Beagle_StackTraceEndM("HistoryEntry::SearchColor HistoryEntry::getSearchColor() const");
}


#endif // Explorer_HistoryEntry_hpp
