
#ifndef Explorer_Individual_hpp
#define Explorer_Individual_hpp

#include <beagle/Beagle.hpp>

#include "explorer/DisplayObject.hpp"
#include "explorer/Fitness.hpp"


namespace Explorer
{

class Individual : public DisplayObject
{

public:

	//! Individual allocator type.
	typedef Beagle::AllocatorT<Individual,DisplayObject::Alloc> Alloc;
	//! Individual handle type.
	typedef Beagle::PointerT<Individual,DisplayObject::Handle> Handle;
	//! Individual bag type.
	typedef Beagle::ContainerT<Individual,DisplayObject::Bag> Bag;

	Individual();
	virtual ~Individual()
	{ }

	virtual void reset();
	virtual void setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem);

	void                 setHistoryId(unsigned int inHistoryId);
	unsigned int         getHistoryId() const;

	void                 setHistoryVar(unsigned int inHistoryVar);
	unsigned int         getHistoryVar() const;

	void                 setFitness(Fitness::Handle inFitness);
	const Fitness::Handle      getFitness() const;

	virtual int         getDisplaySize() const;
	virtual std::string getDisplayName(int inIndex) const;
	virtual float       getDisplayValue(int inIndex) const;
	virtual float       getMinDisplayValue(int inIndex) const;
	virtual float       getMaxDisplayValue(int inIndex) const;

	virtual std::string getToolTip() const;

private:

	unsigned int    mHistoryId;
	unsigned int    mHistoryVar;
	Fitness::Handle mFitness;

};

}


inline void Explorer::Individual::setHistoryId(unsigned int inHistoryId)
{
	Beagle_StackTraceBeginM();

	mHistoryId = inHistoryId;

	Beagle_StackTraceEndM("void Individual::setHistoryId(unsigned int inHistoryId)");
}


inline unsigned int Explorer::Individual::getHistoryId() const
{
	Beagle_StackTraceBeginM();

	return mHistoryId;

	Beagle_StackTraceEndM("unsigned int Individual::getHistoryId() const");
}


inline void Explorer::Individual::setHistoryVar(unsigned int inHistoryVar)
{
	Beagle_StackTraceBeginM();

	mHistoryVar = inHistoryVar;

	Beagle_StackTraceEndM("void Individual::setHistoryVar(unsigned int inHistoryVar)");
}


inline unsigned int Explorer::Individual::getHistoryVar() const
{
	Beagle_StackTraceBeginM();

	return mHistoryVar;

	Beagle_StackTraceEndM("unsigned int Individual::getHistoryVar() const");
}


inline void Explorer::Individual::setFitness(Fitness::Handle inFitness)
{
	Beagle_StackTraceBeginM();

	mFitness = inFitness;

	Beagle_StackTraceEndM("void Individual::setFitness(Fitness::Handle inFitness)");
}


inline const Explorer::Fitness::Handle Explorer::Individual::getFitness() const
{
	Beagle_StackTraceBeginM();

	return mFitness;

	Beagle_StackTraceEndM("const Fitness::Handle Individual::getFitness() const");
}


inline int Explorer::Individual::getDisplaySize() const
{
	Beagle_StackTraceBeginM();

	return mFitness->getDisplaySize();

	Beagle_StackTraceEndM("int Individual::getDisplaySize() const");
}


inline std::string Explorer::Individual::getDisplayName(int inIndex) const
{
	Beagle_StackTraceBeginM();

	return mFitness->getDisplayName(inIndex);

	Beagle_StackTraceEndM("std::string Individual::getDisplayName(int inIndex) const");
}


inline float Explorer::Individual::getDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	return mFitness->getDisplayValue(inIndex);

	Beagle_StackTraceEndM("float Individual::getDisplayValue(int inIndex) const");
}


inline float Explorer::Individual::getMinDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	return mFitness->getMinDisplayValue(inIndex);

	Beagle_StackTraceEndM("float Individual::getMinDisplayValue(int inIndex) const");
}


inline float Explorer::Individual::getMaxDisplayValue(int inIndex) const
{
	Beagle_StackTraceBeginM();

	return mFitness->getMaxDisplayValue(inIndex);

	Beagle_StackTraceEndM("float Individual::getMaxDisplayValue(int inIndex) const");
}


#endif // Explorer_Individual_hpp
