
#ifndef Explorer_InteractionData_hpp
#define Explorer_InteractionData_hpp

#include <QObject>
#include <QCloseEvent>
#include <QWidget>
#include <QLabel>
#include <QSpinBox>
#include <QVector>
#include <QTime>
#include <QEventLoop>

#include <beagle/Beagle.hpp>


namespace Explorer
{

/*!
 *  \brief This class is a singleton and contains the data that enables evolution
 *  to interact with the GUI.
 */
class InteractionData : public QObject
{

	Q_OBJECT

public:

    static InteractionData& instance();

	void setup(Beagle::System::Handle ioSystem);
	void setupStatusIndicators(QWidget* inParent, QVector< QVector<QWidget*> >& outStatusIndicators);
	void setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators);

	void update(const Beagle::Context& inContext);
    bool testPauseOrBreak(const Beagle::Context& inContext);
    bool testPauseOrPastBreak(const Beagle::Context& inContext);
    bool testAtBreak(const Beagle::Context& inContext);
    bool testStatusUpdateTime();

    void enterEventLoop();
    void exitEventLoop();

	void closeEvent(QCloseEvent *event)
    { }

public slots:

    void setPauseEnabled();
    void setBreakEnabled(int inEnabledState);
	void continueFromPauseOrBreak();

signals:

    void enteringEventLoop();
    void exitingEventLoop();

private:

	InteractionData();
    InteractionData(const InteractionData&)
    { }
    InteractionData& operator=(const InteractionData&)
    { }
 	~InteractionData();

    QLabel*         mLabelGeneration;
    QLabel*         mCurrentGeneration;
    QSpinBox*       mBreakGeneration;

    QLabel*         mLabelDemeIndex;
    QLabel*         mCurrentDemeIndex;
    QSpinBox*       mBreakDemeIndex;

    QLabel*         mLabelIndividualIndex;
    QLabel*         mCurrentIndividualIndex;
    QSpinBox*       mBreakIndividualIndex;

    bool            mPauseEnabled;
    bool            mBreakEnabled;

	QTime*          mStatusUpdateTime;          //!< Time object to determine when to update status.
	int             mStatusUpdateTimeInterval;  //!< Minimum time interval to update status.
    QEventLoop*     mEventLoop;                 //!< A local event loop for GUI interaction.

};

}


/*! 
 * \brief Enter the local event loop.
 */
inline void Explorer::InteractionData::enterEventLoop()
{
	Beagle_StackTraceBeginM();
    emit enteringEventLoop();
    mEventLoop->exec();
	Beagle_StackTraceEndM("void InteractionData::enterEventLoop()");
}


/*! 
 * \brief Exit the local event loop.
 */
inline void Explorer::InteractionData::exitEventLoop()
{
	Beagle_StackTraceBeginM();
    emit exitingEventLoop();
    mEventLoop->exit();
	Beagle_StackTraceEndM("void InteractionData::exitEventLoop()");
}


#endif // Explorer_InteractionData_hpp
