
#ifndef Explorer_MainWindow_hpp
#define Explorer_MainWindow_hpp

#include <QMainWindow>
#include <QString>
#include <beagle/Beagle.hpp>

#include "explorer/EvolutionTab.hpp"
#include "explorer/HistoryTab.hpp"
#include "designer/ui_MainWindow.h"

namespace Explorer
{

class MainWindow : public QMainWindow, private Ui::MainWindow
{

	Q_OBJECT

public:

	MainWindow(Beagle::System::Handle ioSystem, QWidget* inParent=0, Qt::WFlags inFlags=0);
	~MainWindow()
	{ }

    void setCurrentTab(QWidget* inTab);

public slots:

	void openMilestone();
	void showConfigureDialog();
	void enterWhatsThisMode();
	void aboutExplorer();

protected:

	virtual void closeEvent(QCloseEvent *event);

	virtual void readSettings();
	virtual void writeSettings();

private:

	Beagle::System::Handle mSystem;
	EvolutionTab*          mEvolutionTab;
	HistoryTab*            mHistoryTab;

};

}


/*! 
 * \brief Set the current tab of the main window.
 * \param inTab The specified tab.
 */
inline void Explorer::MainWindow::setCurrentTab(QWidget* inTab)
{
	Beagle_StackTraceBeginM();
	mTabWidget->setCurrentWidget(inTab);
	Beagle_StackTraceEndM("void MainWindow::setCurrentTab(QWidget* inTab)");
}


#endif // Explorer_MainWindow_hpp

