
#ifndef Explorer_VivariumSettings_hpp
#define Explorer_VivariumSettings_hpp

#include "explorer/HistorySettings.hpp"
#include "explorer/VivariumNode.hpp"


namespace Explorer
{

// Forward declaration
class HistoryTab;

class VivariumSettings : public HistorySettings
{

	Q_OBJECT

public:

	VivariumSettings(History::Handle inHistory=NULL, QWidget* inParent=0, Qt::WFlags inFlags=0);
	virtual ~VivariumSettings()
    { }

	virtual void reset();
	virtual void setup();

public slots:

	virtual void updateView();

private slots:

    void setupScene(int inSortIndex);

protected:

    /*!
     *  \brief Binary predicate functor that tests the ordering of two vivarium nodes
     *      based on the specified sort index.
     *  \par Note:
     *      This class is compliant with the STL Strict Weak Ordering.
     *      See http://www.sgi.com/tech/stl/StrictWeakOrdering.html
     */
    class IsLessNodePredicate
    {
    public:
        IsLessNodePredicate(int inSortIndex) : mSortIndex(inSortIndex)
        { }
        ~IsLessNodePredicate()
        { }

        /*!
         *  \brief Test the ordering of two vivarium nodes.
         *  \param inLeftNode Left node to compare.
         *  \param inRightNode Right node to compare.
         */
        inline bool operator()(const VivariumNode* inLeftNode, const VivariumNode* inRightNode) const
        {
            Beagle_StackTraceBeginM();
            if (inLeftNode->getDisplayObject() == NULL)
                return true;
            else if (inRightNode->getDisplayObject() == NULL)
                return false;
            else
                return (inLeftNode->getDisplayObject()->getDisplayValue(mSortIndex) < inRightNode->getDisplayObject()->getDisplayValue(mSortIndex));
            Beagle_StackTraceEndM("bool ENSO::Tree::IsLessNodePredicate::operator()(const VivariumNode* inLeftNode, const VivariumNode* inRightNode) const");
        }

    private:
        int             mSortIndex;     // Index specifying the sort criterion.
    };

    HistoryTab*         mHistoryTab;
    QLabel*             mSortLabel;
    QComboBox*          mSortCombo;

};

}


#endif // Explorer_VivariumSettings_hpp
