
#ifndef Explorer_macros_hpp
#define Explorer_macros_hpp

#include <QColor>


namespace Explorer
{

/*!
 *  \brief Helper function used by hsl2rgb() for computing an r/g/b value.
 */
inline unsigned int hue2rgb(double inM1, double inM2, double inH)
{
	Beagle_StackTraceBeginM();

	if (inH < 0.0)
		inH = inH+1.0;
	if (inH > 1.0)
		inH = inH-1.0;

	double lC;
	if (inH*6.0 < 1.0)
		lC = inM1+(inM2-inM1)*inH*6.0;
	else if (inH*2.0 < 1.0)
		lC = inM2;
	else if (inH*3.0 < 2.0)
		lC = inM1+(inM2-inM1)*(2.0/3.0-inH)*6.0;
	else
		lC = inM1;

	return (unsigned int)(lC*255);

	Beagle_StackTraceEndM("unsigned int hue2rgb(double inM1, double inM2, double inH)");
}


/*!
 *  \brief Convert an hsl color value to rgb color.
 *  \param inH Hue
 *  \param inS Saturation
 *  \param inL Lightness
 *  \return The rgb color string
 *
 *  Conversion algorithm reference: http://www.w3.org/TR/css3-color/#hsl-color
 *  The h, s and l values are assumed to be normalized to lie in [0,1].
 */
inline QColor hsl2rgb(double inH, double inS, double inL)
{
	Beagle_StackTraceBeginM();

	double lM2 = (inL<=0.5) ? inL*(inS+1.0) : inL+inS-inL*inS;
	double lM1 = inL*2.0-lM2;
	unsigned int lR = hue2rgb(lM1, lM2, inH+1.0/3.0);
	unsigned int lG = hue2rgb(lM1, lM2, inH);
	unsigned int lB = hue2rgb(lM1, lM2, inH-1.0/3.0);

	return QColor(lR, lG, lB);

	Beagle_StackTraceEndM("QColor hsl2rgb(double inH, double inS, double inL)");
}

}


#endif // Explorer_macros_hpp
