
#include <QLinearGradient>
#include <QPainter>

#include "designer/ColorBar.hpp"


ColorBar::ColorBar(QWidget* inParent, Qt::WFlags inFlags) :
		QWidget(inParent, inFlags)
{
	setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
}


QSize ColorBar::sizeHint() const
{
	QFontMetrics lFontMetrics = fontMetrics();
	int lHeight = lFontMetrics.height();
	return QSize(lHeight, lHeight);
}


void ColorBar::setEndColors(const QColor &inColor0, const QColor &inColor1)
{
	mColor0 = inColor0;
	mColor1 = inColor1;
	update();
}


void ColorBar::paintEvent(QPaintEvent *inEvent)
{
	Q_UNUSED(inEvent);

	QPainter lPainter(this);

	qreal lH0, lS0, lV0;
	qreal lH1, lS1, lV1;
	mColor0.getHsvF(&lH0, &lS0, &lV0);
	mColor1.getHsvF(&lH1, &lS1, &lV1);

	int lWidth = width();
	int lHeight = height();

	qreal lHRatio = (lH1 - lH0)/lWidth;
	qreal lSRatio = (lS1 - lS0)/lWidth;
	qreal lVRatio = (lV1 - lV0)/lWidth;
	for (int i = 0; i < lWidth; ++i) {
		qreal lH = lH0 + i*lHRatio;
		qreal lS = lS0 + i*lSRatio;
		qreal lV = lV0 + i*lVRatio;
		QColor lColor = QColor::fromHsvF(lH, lS, lV);
		lPainter.fillRect(i, 0, 1, lHeight, lColor);
	}

	// Draw a border.
	lPainter.setPen(QColor(146, 146, 146));
	lPainter.drawRect(0, 0, lWidth-1, lHeight-1);
}
