
#include "explorer/Explorer.hpp"

using namespace Explorer;

/*!
 *  \brief DisplayScene constructor.
 *
 *  The member variables for node properties must be set to valid values
 *  before using the history scene.
 */
DisplayScene::DisplayScene(QObject *parent) :
		QGraphicsScene(parent),
		mNodeHueIndex(-1),
		mNodeSatIndex(-1),
		mNodeValIndex(-1),
		mNodeBoundingSize(-1.0)
{
	Beagle_StackTraceBeginM();

	connect(this, SIGNAL(selectionChanged()), this, SLOT(processSelection()));

	Beagle_StackTraceEndM("DisplayScene::DisplayScene(QObject *parent)");
}


/*!
 *  \brief Find the current selection of history nodes.
 */
QList<DisplayNode*> DisplayScene::getSelectedNodes() const
{
	Beagle_StackTraceBeginM();

	QList<DisplayNode*> lSelectedNodes;
	foreach (QGraphicsItem* lSelectedItem, selectedItems()) {
		DisplayNode* lDisplayNode = qgraphicsitem_cast<DisplayNode*>(lSelectedItem);
		if (lDisplayNode)
			lSelectedNodes.push_back(lDisplayNode);
	}

	return lSelectedNodes;

	Beagle_StackTraceEndM("HistoryEntry::Bag::Handle DisplayScene::getSelectedNodes() const");
}


/*!
 *  \brief Emits nodeSelectionChanged() signal if selection of nodes changes.
 */
void DisplayScene::processSelection()
{
	Beagle_StackTraceBeginM();

	// When item selection is changed, the selectionChanged() signal is emitted
	// once for deselection of the old item and again for selection of the new
	// item.  We want to update only for the second signal when the number of
	// selected items is nonzero.  However, if the cancels all selections, then
	// a second signal is not sent, and therefore, we need to update on the
	// first signal itself.  This is accomplished by also checking if any item
	// is currently grabbing the mouse.

	QList<DisplayNode*> lSelections = getSelectedNodes();
	DisplayNode* lDisplayNode = qgraphicsitem_cast<DisplayNode*>(mouseGrabberItem());

	if ((lSelections.size() != 0 && lDisplayNode != 0) || (lSelections.size() == 0 && lDisplayNode == 0)) {
		emit nodeSelectionChanged();
	}

	Beagle_StackTraceEndM("void DisplayScene::processSelection()");
}
