
#include <QSettings>
#include <QCloseEvent>

#include "explorer/Explorer.hpp"

using namespace Explorer;


DisplayWidget::DisplayWidget(DisplaySettings* inDisplaySettings, QString inName, QWidget* inParent, Qt::WFlags inFlags) :
		QWidget(inParent, inFlags),
		Ui::DisplayWidget(),
        mDisplaySettings(inDisplaySettings),
        mName(inName)
{
	Beagle_StackTraceBeginM();

	setupUi(this);
	readSettings();

	Beagle_StackTraceEndM("DisplayWidget::DisplayWidget(DisplaySettings* inDisplaySettings, QString inName, QWidget* inParent, Qt::WFlags inFlags)");
}


void DisplayWidget::closeEvent(QCloseEvent *inEvent)
{
	Beagle_StackTraceBeginM();

	writeSettings();
    emit closed();
	inEvent->accept();

	Beagle_StackTraceEndM("void DisplayWidget::closeEvent(QCloseEvent *inEvent)");
}


void DisplayWidget::reset()
{
	Beagle_StackTraceBeginM();

	mDisplaySettings->reset();

	Beagle_StackTraceEndM("void DisplayWidget::reset()");
}


void DisplayWidget::setup()
{
	Beagle_StackTraceBeginM();

	// Add widgets to the splitter.
	mSplitter->addWidget(mDisplaySettings->getView());
	mSplitter->addWidget(mDisplaySettings);

	Beagle_StackTraceEndM("void DisplayWidget::setup()");
}


void DisplayWidget::updateView()
{
	Beagle_StackTraceBeginM();

	mDisplaySettings->updateView();

	Beagle_StackTraceEndM("void DisplayWidget::updateView()");
}


void DisplayWidget::readSettings()
{
	Beagle_StackTraceBeginM();

	QSettings lSettings("OpenBEAGLE", "Explorer");

	lSettings.beginGroup(mName);
	resize(lSettings.value("size", QSize(600, 400)).toSize());
	move(lSettings.value("pos", QPoint(200, 200)).toPoint());
	mSplitter->restoreState(lSettings.value("splitterSizes").toByteArray());
	lSettings.endGroup();

	Beagle_StackTraceEndM("void DisplayWidget::readSettings()");
}


void DisplayWidget::writeSettings()
{
	Beagle_StackTraceBeginM();

	QSettings lSettings("OpenBEAGLE", "Explorer");

	lSettings.beginGroup(mName);
	lSettings.setValue("pos", pos());
	lSettings.setValue("size", size());
	lSettings.setValue("splitterSizes", mSplitter->saveState());
	lSettings.endGroup();

	Beagle_StackTraceEndM("void DisplayWidget::writeSettings()");
}
