
#include "explorer/Explorer.hpp"

using namespace Explorer;


std::string FitnessSimple::smDisplayName = "Fitness Simple";
float       FitnessSimple::smMinDisplayValue = FLT_MAX;
float       FitnessSimple::smMaxDisplayValue = -FLT_MAX;


FitnessSimple::FitnessSimple() :
		Fitness(),
		mDisplayValue(-FLT_MAX)
{}


/*!
 *  \brief Reset information.
 */
void FitnessSimple::reset()
{
	Beagle_StackTraceBeginM();

	Fitness::reset();

	smMinDisplayValue = FLT_MAX;
	smMaxDisplayValue = -FLT_MAX;
	mDisplayValue = -FLT_MAX;

	Beagle_StackTraceEndM("void FitnessSimple::reset()");
}


/*!
 *  \brief Setup the object with the given Beagle fitness.
 */
void FitnessSimple::setup(Beagle::Fitness::Handle inFitness, Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();

	Fitness::setup(inFitness, ioSystem);

	Beagle::FitnessSimple::Handle lFitness = Beagle::castHandleT<Beagle::FitnessSimple>(inFitness);

	if (lFitness->isValid()) {
		mDisplayValue = lFitness->getValue();

		// Keep track of the min and max display values.
		if (mDisplayValue < smMinDisplayValue && mDisplayValue > -FLT_MAX)
			smMinDisplayValue = mDisplayValue;
		if (mDisplayValue > smMaxDisplayValue && mDisplayValue < FLT_MAX)
			smMaxDisplayValue = mDisplayValue;
	} else {
		mDisplayValue = -FLT_MAX;
	}

	Beagle_StackTraceEndM("void FitnessSimple::setup(Beagle::Fitness::Handle inFitness, Beagle::System& ioSystem)");
}
