
#include "explorer/Explorer.hpp"

using namespace Explorer;


HistoryEntry::HistoryEntry() :
		mHistoryId(Beagle::HistoryID::eDummyID),
		mHistoryVar(0),
		mCreationAction(eUnknownAction),
		mMarkedFlag(false),
		mGenealogyFlag(false),
		mGeneration(UINT_MAX)
{}


/*!
 *  \brief Reset static information in members.
 */
void HistoryEntry::reset()
{
	Beagle_StackTraceBeginM();

	if (mIndividual != NULL)
		mIndividual->reset();

	Beagle_StackTraceEndM("void HistoryEntry::reset()");
}


/*!
 *  \brief Setup the member variables with data from the provided Beagle HistoryEntry.
 */
void HistoryEntry::setup(Beagle::HistoryEntry::Handle inHistoryEntry, Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();

    const Beagle::Factory& lFactory = ioSystem.getFactory();
    Individual::Alloc::Handle lIndividualAlloc = 
        Beagle::castHandleT<Individual::Alloc>(lFactory.getConceptAllocator("Explorer-Individual"));

	if (inHistoryEntry != NULL) {
		mHistoryVar = inHistoryEntry->getIndividualVar();
		if (lIndividualAlloc != NULL && inHistoryEntry->getIndividual() != NULL) {
			mIndividual = Beagle::castHandleT<Individual>(lIndividualAlloc->allocate());
			mIndividual->setup(inHistoryEntry->getIndividual(), ioSystem);
		}
		const std::vector<std::string>& lActions = inHistoryEntry->getActions();
		setCreationAction(lActions.front());
		if (lActions.back() == "mark")
			mMarkedFlag = true;
		mGeneration = inHistoryEntry->getGeneration();
	}

	Beagle_StackTraceEndM("void HistoryEntry::setup(Beagle::HistoryEntry::Handle inHistoryEntry, Beagle::System& ioSystem)");
}


/*!
 *  \brief Check if a given history entry is contained in a collection of entries.
 */
bool HistoryEntry::isMember(HistoryEntry::Handle inHistoryEntry, HistoryEntry::Bag::Handle inBag) const
{
	Beagle_StackTraceBeginM();

	bool lResult = false;
	for (unsigned int i = 0; i < inBag->size() && lResult == false; ++i) {
		lResult = inHistoryEntry == inBag->at(i);
	}
	return lResult;

	Beagle_StackTraceEndM("bool HistoryEntry::isMember(HistoryEntry::Handle inHistoryEntry, HistoryEntry::Bag::Handle inBag) const");
}


std::string HistoryEntry::getToolTip() const
{
	Beagle_StackTraceBeginM();

	std::string lToolTip = "(hid, var) = (" + Beagle::uint2str(mHistoryId) + ", " + Beagle::uint2str(mHistoryVar) + ")";
	if (mIndividual != NULL)
		lToolTip += "\n" + mIndividual->getToolTip();
	return lToolTip;

	Beagle_StackTraceEndM("std::string HistoryEntry::getToolTip() const");
}

