
#include "explorer/Explorer.hpp"

using namespace Explorer;


HistorySettings::HistorySettings(History::Handle inHistory, QWidget* inParent, Qt::WFlags inFlags) :
		DisplaySettings(inParent, inFlags),
		mHistory(inHistory)
{
	Beagle_StackTraceBeginM();

    // Add the spin boxes for specifying the display ancestry and descendancy depths
    // by augmenting the interface in Ui::DisplaySettings.

    mAncestryLabel = new QLabel(this);
    mAncestryLabel->setObjectName(QString::fromUtf8("mAncestryLabel"));
    mAncestryLabel->setText(QApplication::translate("HistorySettings", "&Ancestry", 0, QApplication::UnicodeUTF8));
    gridLayout->addWidget(mAncestryLabel, 1, 0, 1, 1);

    mAncestrySpin = new QSpinBox(this);
    mAncestrySpin->setObjectName(QString::fromUtf8("mAncestrySpin"));
    mAncestrySpin->setMinimum(-1);
    mAncestrySpin->setMaximum(99999);
    mAncestrySpin->setValue(-1);
    mAncestrySpin->setSpecialValueText(QApplication::translate("HistorySettings", "Full", 0, QApplication::UnicodeUTF8));
    gridLayout->addWidget(mAncestrySpin, 1, 1, 1, 1);

    mAncestryLabel->setBuddy(mAncestrySpin);

    mDescendancyLabel = new QLabel(this);
    mDescendancyLabel->setObjectName(QString::fromUtf8("mDescendancyLabel"));
    mDescendancyLabel->setText(QApplication::translate("HistorySettings", "&Descendancy", 0, QApplication::UnicodeUTF8));
    gridLayout->addWidget(mDescendancyLabel, 2, 0, 1, 1);

    mDescendancySpin = new QSpinBox(this);
    mDescendancySpin->setObjectName(QString::fromUtf8("mDescendancySpin"));
    mDescendancySpin->setMinimum(-1);
    mDescendancySpin->setMaximum(99999);
    mDescendancySpin->setValue(0);
    mDescendancySpin->setSpecialValueText(QApplication::translate("HistorySettings", "Full", 0, QApplication::UnicodeUTF8));
    gridLayout->addWidget(mDescendancySpin, 2, 1, 1, 1);

    mDescendancyLabel->setBuddy(mDescendancySpin);

	connect(mAncestrySpin, SIGNAL(valueChanged(int)), this, SLOT(updateView()));
	connect(mDescendancySpin, SIGNAL(valueChanged(int)), this, SLOT(updateView()));

	mAncestrySpin->setWhatsThis(tr("Specify generations of ancestry to show in the view."));
	mDescendancySpin->setWhatsThis(tr("Specify generations of descendancy to show in the view."));

	mZoomSlider->setValue(mZoomSlider->maximum()/2);

	Beagle_StackTraceEndM("HistorySettings::HistorySettings(History::Handle inHistory, QWidget* inParent, Qt::WFlags inFlags)");
}


/*!
 *  \brief Setup history display.
 */
void HistorySettings::setup()
{
	Beagle_StackTraceBeginM();

    for (unsigned int i = mHistory->getMinGeneration(); i < mHistory->getMaxGeneration() && mPrototypeObject == NULL; ++i) {
        for (unsigned int j = 0; j < mHistory->getVivariumSize(i) && mPrototypeObject == NULL; ++j) {
            mPrototypeObject = mHistory->getMarkedEntry(i, j)->getIndividual();
        }
    }

    DisplaySettings::setup();

	Beagle_StackTraceEndM("void HistorySettings::setup()");
}


/*!
 *  \brief Find the history entries belonging to the selected nodes.
 */
HistoryEntry::Bag::Handle HistorySettings::getSelectedHistoryEntries() const
{
	Beagle_StackTraceBeginM();

	HistoryEntry::Bag::Handle lSelectedEntries = new HistoryEntry::Bag;
	if (mScene) {
		foreach (DisplayNode* lDisplayNode, mScene->getSelectedNodes()) {
            HistoryNode* lHistoryNode = qgraphicsitem_cast<HistoryNode*>(lDisplayNode);
			lSelectedEntries->push_back(lHistoryNode->getHistoryEntry());
		}
	}

	return lSelectedEntries;

	Beagle_StackTraceEndM("HistoryEntry::Bag::Handle HistorySettings::getSelectedHistoryEntries() const");
}
