
#include "explorer/Explorer.hpp"

using namespace Explorer;


Individual::Individual() :
		mHistoryId(Beagle::HistoryID::eDummyID),
		mHistoryVar(0)
{}


/*!
 *  \brief Reset information in members.
 */
void Individual::reset()
{
	Beagle_StackTraceBeginM();

	if (mFitness != NULL)
        mFitness->reset();

	Beagle_StackTraceEndM("void Individual::reset()");
}


/*!
 *  \brief Setup the member variables with data from the provided Beagle Individual.
 */
void Individual::setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();

    const Beagle::Factory& lFactory = ioSystem.getFactory();
    Fitness::Alloc::Handle lFitnessAlloc = 
        Beagle::castHandleT<Fitness::Alloc>(lFactory.getConceptAllocator("Explorer-Fitness"));

	if (inIndividual != NULL) {
        Beagle::HistoryID::Handle lHID = Beagle::castHandleT<Beagle::HistoryID>(inIndividual->getMember("HistoryID"));
		mHistoryId = lHID->getID();
        mHistoryVar = lHID->getVar();
		if (lFitnessAlloc != NULL && inIndividual->getFitness() != NULL) {
            mFitness = Beagle::castHandleT<Fitness>(lFitnessAlloc->allocate());
            mFitness->setup(inIndividual->getFitness(), ioSystem);
        }
	}

	Beagle_StackTraceEndM("void Individual::setup(Beagle::Individual::Handle inIndividual, Beagle::System& ioSystem)");
}


std::string Explorer::Individual::getToolTip() const
{
	Beagle_StackTraceBeginM();

	std::string lToolTip = "Ref (hid, var) = (" + Beagle::uint2str(mHistoryId) + ", " + Beagle::uint2str(mHistoryVar) + ")";
    for (unsigned int i=1; i < getDisplaySize(); i++) {  // Note that i=0 corresponds to no display
		lToolTip += "\n" + getDisplayName(i) + " = " + Beagle::dbl2str(getDisplayValue(i));
    }
	return lToolTip;

	Beagle_StackTraceEndM("std::string Individual::getToolTip() const");
}

