
#include "explorer/Explorer.hpp"

using namespace Explorer;


InteractionData&    Interaction::mInteractionData = InteractionData::instance();


/*!
 *  \brief Construct an interaction object.
 */
Interaction::Interaction() :
        Beagle::Object()
{ }


/*!
 *  \brief Setup the interaction object.
 */
void Interaction::setup(Beagle::System::Handle ioSystem)
{
	Beagle_StackTraceBeginM();
    mInteractionData.setup(ioSystem);
	Beagle_StackTraceEndM("void Interaction::setup(Beagle::System::Handle ioSystem)");
}


/*!
 *  \brief Create and setup the evolution status indicators.
 *  \param inParent The parent widget of the new status indicator widgets.
 *  \param outStatusIndicators The new status indicator widgets are returned in this array.
 */
void Interaction::setupStatusIndicators(QWidget* inParent, QVector< QVector<QWidget*> >& outStatusIndicators)
{
	Beagle_StackTraceBeginM();
    mInteractionData.setupStatusIndicators(inParent, outStatusIndicators);
	Beagle_StackTraceEndM("void Interaction::setupStatusIndicators(QWidget* inParent, QVector< QVector<QWidget*> >& outStatusIndicators)");
}


/*!
 *  \brief Create and setup the display window indicators.
 *  \param inParent The parent widget of the new display indicator widgets.
 *  \param outDisplayIndicators The new display indicator widgets are returned in this array.
 */
void Interaction::setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators)
{	Beagle_StackTraceBeginM();
    mInteractionData.setupDisplayIndicators(inParent, outDisplayIndicators);
	Beagle_StackTraceEndM("void Interaction::setupDisplayIndicators(QWidget* inParent, QVector<QWidget*>& outDisplayIndicators)");
}


/*!
 *  \brief Controls GUI updates and entering Qt's event loop.
 *  \param inContext The context of evolution.
 */
void Interaction::update(const Beagle::Context& inContext)
{
	Beagle_StackTraceBeginM();
    if (mInteractionData.testPauseOrBreak(inContext)) {
        mInteractionData.update(inContext);
        mInteractionData.enterEventLoop();
    }
    else if (mInteractionData.testStatusUpdateTime()) {
        mInteractionData.update(inContext);
    }
	Beagle_StackTraceEndM("void Interaction::update(const Beagle::Context& inContext)");
}


/*!
 *  \brief Process the close event.
 */
void Interaction::closeEvent(QCloseEvent *event)
{
	Beagle_StackTraceBeginM();
    mInteractionData.closeEvent(event);
	Beagle_StackTraceEndM("void Interaction::closeEvent(QCloseEvent *event)");
}
