
#include "explorer/Explorer.hpp"

using namespace Explorer;


/*!
 *  \brief Construct package with basic Explorer objects.
 */
PackageBase::PackageBase() :
		Beagle::Package("Explorer-PackageBase")
{ }


/*!
 *  \brief Configure system by adding necessary components into it.
 *  \param ioSystem System to configure.
 */
void PackageBase::configure(Beagle::System& ioSystem)
{
	Beagle_StackTraceBeginM();
    Beagle::Factory& lFactory = ioSystem.getFactory();

	// Add available basic types to the factory
	lFactory.insertAllocator("Explorer::FitnessSimple", new Explorer::FitnessSimple::Alloc);
	lFactory.insertAllocator("Explorer::History", new Explorer::History::Alloc);
	lFactory.insertAllocator("Explorer::Individual", new Explorer::Individual::Alloc);
	lFactory.insertAllocator("Explorer::Interaction", new Explorer::Interaction::Alloc);

	// Set aliases
	lFactory.aliasAllocator("Explorer::FitnessSimple", "Explorer-FitnessSimple");
	lFactory.aliasAllocator("Explorer::History", "Explorer-History");
	lFactory.aliasAllocator("Explorer::Individual", "Explorer-Individual");
	lFactory.aliasAllocator("Explorer::Interaction", "Interaction");

	// Set some basic concept-type associations in factory
	lFactory.setConcept("Explorer-Fitness", "Explorer-FitnessSimple");
	lFactory.setConcept("Explorer-Individual", "Explorer-Individual");
	lFactory.setConcept("Interaction", "Interaction");

	Beagle_StackTraceEndM("void PackageBase::configure(System&)");
}


/*!
 *  \brief List packages on which current package depends.
 *  \return Bag with the packages instanciation on which current package depends.
 *
 *  Explorer::PackageBase depends on Beagle::PackageBase.
 */
Beagle::Package::Bag PackageBase::listDependencies(void)
{
	Beagle_StackTraceBeginM();
	Package::Bag lDependencies;
	lDependencies.push_back(new Beagle::PackageBase());
	return lDependencies;
	Beagle_StackTraceEndM("Beagle::Package::Bag PackageBase::listDependencies(void)");
}
