
#include "explorer/Explorer.hpp"

using namespace Explorer;


VivariumNode::VivariumNode(HistoryEntry::Handle inHistoryEntry) :
		HistoryNode(inHistoryEntry),
		mGenealogyFlag(false)
{}


void VivariumNode::paint(QPainter *ioPainter, const QStyleOptionGraphicsItem *inOption, QWidget *inWidget)
{
	Beagle_StackTraceBeginM();

	Q_UNUSED(inWidget);

	DisplayScene* lScene = qobject_cast<DisplayScene*>(scene());
	if (lScene == 0)
		return;
	qreal lBoundingSize = lScene->getNodeBoundingSize();

	static const qreal lOuterPenW = 1;

	QColor lOuterColor = getOuterColor();
	ioPainter->setPen(QPen(Qt::black, lOuterPenW));
	ioPainter->setBrush(lOuterColor);

	static const qreal lOuterRectW = lBoundingSize - lOuterPenW;
	static const qreal lOuterRectX = lOuterPenW/2;
	static const QRectF lOuterRect(lOuterRectX, lOuterRectX, lOuterRectW, lOuterRectW);
	ioPainter->drawRect(lOuterRect);

	if (mGenealogyFlag) {
		QColor lInnerColor = getInnerColor(lOuterColor);
		ioPainter->setPen(QPen(Qt::transparent, 0));
		ioPainter->setBrush(lInnerColor);

		// Use an inner plus sign to mark selected nodes.
		if (inOption->state & QStyle::State_Selected) {
			static const qreal lPlusRectW = (lOuterRectW - lOuterPenW)/4;
			static const qreal lPlusRectH = lOuterRectW - lOuterPenW;
			static const qreal lPlusRectX = lOuterPenW + (lOuterRectW - lPlusRectW)/2;
			static const qreal lPlusRectY = lOuterPenW;
			static const QRectF lPlusRect1(lPlusRectX, lPlusRectY, lPlusRectW, lPlusRectH);
			static const QRectF lPlusRect2(lPlusRectY, lPlusRectX, lPlusRectH, lPlusRectW);
			ioPainter->drawRect(lPlusRect1);
			ioPainter->drawRect(lPlusRect2);
		}
		// Use an inner circle to mark other nodes in genealogy tree.
		else {
			static const qreal lCircleRectW = (lOuterRectW - lOuterPenW)/1.5;
			static const qreal lCircleRectX = lOuterPenW + (lOuterRectW - lCircleRectW)/2;
			static const QRectF lCircleRect(lCircleRectX, lCircleRectX, lCircleRectW, lCircleRectW);
			ioPainter->drawEllipse(lCircleRect);
		}
	}

	Beagle_StackTraceEndM("void VivariumNode::paint(QPainter *ioPainter, const QStyleOptionGraphicsItem *inOption, QWidget *inWidget)");
}
